#include "stdafx.h"

#include "Category.h"
#include "emule.h"

CCat::CCatArray		CCat::g_arrCat;
CCat::CCatIDMap		CCat::g_mapCatID;	// Reverse lookup map for category IDs
uint8				CCat::g_nAllCatType = 0;
uint8			CCat::g_iNumPermanentCats = 0;

/**************************************************************************************************************************/
CCat::CCat()
	: m_iPriority(0), m_bIsPermanent(false), m_iCatID(0)
{
}
/**************************************************************************************************************************/
CCat::CCat(LPCTSTR strTitle, LPCTSTR strSavePath/*=NULL*/, LPCTSTR strComment/*=NULL*/, LPCTSTR strAutoCatExt/*=NULL*/)
	: m_iPriority(0), m_bIsPermanent(false), m_iCatID(0)
{
	m_strTitle = strTitle;
	m_strSavePath = strSavePath;
	m_strComment = strComment;
	m_strAutoCatExt = strAutoCatExt;
}
/**************************************************************************************************************************/
/*static*/ void CCat::Finalize()
{
	CCat	*pCatToDel = NULL;

	while (!g_arrCat.IsEmpty())
	{
		pCatToDel = g_arrCat[0]; 
		g_arrCat.RemoveAt(0); 
		delete pCatToDel;
	}
}
/**************************************************************************************************************************/
//	GetNewCatID() returns the next available user category ID.
/*static*/ uint8 CCat::GetNewCatID()
{
	uint8		iNewCatID = 0;

	for (int i = g_iNumPermanentCats; i < g_arrCat.GetCount(); i++)
	{
		uint8		iCatID;

		if ((iCatID = g_arrCat[i]->m_iCatID) > iNewCatID)
			iNewCatID = iCatID;
	}

	return ++iNewCatID;
}
/**************************************************************************************************************************/
//	MoveCat() moves the category at 'iFromIndex' to just before the category at 'iToIndex'.
//		It returns true on success and false on failure.
/*static*/ bool CCat::MoveCat(uint8 iFromIndex, uint8 iToIndex)
{
//	If either of the indices are past the end of the array
//		 or are the same, just return.
	if ( iFromIndex >= (UINT)g_arrCat.GetCount()
	  || iToIndex >= (UINT)g_arrCat.GetCount()+1
	  || iFromIndex == iToIndex )
	{
		return false;
	}

	CCat	*pCatToMove = NULL;
	uint8	iFromCatID = 0, iToCatID = 0;

	pCatToMove = g_arrCat[iFromIndex];
	iFromCatID = pCatToMove->m_iCatID;
	iToCatID = g_arrCat[iToIndex]->m_iCatID;

//	If the source is to the left of the destination...
	if (iFromIndex < iToIndex)
	{
		g_arrCat.RemoveAt(iFromIndex);
		g_arrCat.InsertAt(iToIndex-1,pCatToMove);

	//	The indices of all cats from 'iFromIndex' to the end of the array have changed;
	//		Update the reverse lookup map.
		for (uint8 i = iFromIndex; i < g_arrCat.GetCount(); i++)
		{
			g_mapCatID[g_arrCat[i]->m_iCatID] = i;
		}
	}
//	If the source is to the right of the destination...
	else
	{
		g_arrCat.InsertAt(iToIndex,pCatToMove);
		g_arrCat.RemoveAt(iFromIndex+1);

	//	The indices of all cats from 'iToIndex' to the end of the array have changed;
	//		Update the reverse lookup map.
		for (uint8 i = iToIndex; i < g_arrCat.GetCount(); i++)
		{
			g_mapCatID[g_arrCat[i]->m_iCatID] = i;
		}
	}

	theApp.glob_prefs->SaveCats();

	return true;
}
/**************************************************************************************************************************/
//	AddCat() adds the new category 'pCat' and returns it's newly assigned ID.
/*static*/ int CCat::AddCat(CCat *pCat, bool bAssignNewID/*=true*/)
{
	uint8		iIndex = 0;
	uint8		iNewCatID = 0;
	
	iIndex = g_arrCat.Add(pCat);

	if (bAssignNewID)
	{
		iNewCatID = GetNewCatID();
		pCat->m_iCatID = iNewCatID;
	}
	else
	{
		iNewCatID = pCat->m_iCatID;
	}
	g_mapCatID[iNewCatID] = iIndex;

	return iNewCatID;
}
/**************************************************************************************************************************/
//	RemoveCat() removes and destroys the category at index 'iIndex'.
/*static*/ void CCat::RemoveCatByIndex(int iIndex)
{
//	If the index is within bounds...
	if (iIndex >= 0 && iIndex < g_arrCat.GetCount())
	{ 
		CCat	*pCatToDelete;

		pCatToDelete = g_arrCat[iIndex]; 
		g_arrCat.RemoveAt(iIndex);

	//	The indices of all cats from 'iIndex' to the end of the array have changed;
	//		Update the reverse lookup map.
		for (uint8 i = iIndex; i < g_arrCat.GetCount(); i++)
		{
			g_mapCatID[g_arrCat[i]->m_iCatID] = i;
		}

		delete pCatToDelete;
	}
}
/**************************************************************************************************************************/
/*static*/ int CCat::GetCatIndexByID(uint8 iCatID)
{
	CCatIDMap::iterator		it;

	if ((it = g_mapCatID.find(iCatID)) != g_mapCatID.end())
		return it->second;
	else
		return -1;
}
/**************************************************************************************************************************/
