//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


#include "StdAfx.h"
#include "zlib/zlib.h"
#include "updownclient.h"
#include "partfile.h"
#include "opcodes.h"
#include "packets.h"
#include "emule.h"
#include "memcpy_amd.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


//	members of CUpDownClient
//	which are mainly used for downloading functions 

void CUpDownClient::DrawStatusBar(CDC* dc, RECT* rect, bool onlygreyrect, bool  bFlat)
{ 
	EMULE_TRY

	const COLORREF crBoth=(bFlat) ? RGB(0, 150, 0) : RGB(0, 192, 0);	//Cax2 - same green used for files & clients 
	const COLORREF crNeither=(bFlat) ? RGB(224, 224, 224) : RGB(240, 240, 240); 
	const COLORREF crClientOnly=(bFlat) ? RGB(0, 0, 0) : RGB(95, 95, 95); 
	const COLORREF crPending = RGB(255,208,0);
	const COLORREF crNextPending = RGB(255,255,100);
	//--- xrmb:seetheneed ---
	const COLORREF crMeOnly = RGB(190,190,255); 
	//--- :xrmb ---
	
	uint32 filesize=(reqfile)? reqfile->GetFileSize():1;

	CBarShader statusBar(rect->bottom - rect->top, rect->right - rect->left);
	statusBar.SetFileSize(filesize);
	statusBar.Fill(crNeither);

	if(reqfile)
	{
		if (!onlygreyrect && reqfile && m_abyPartStatus) { 
			_Bvector gettingParts;
			bool isDLing=(m_nDownloadState == DS_DOWNLOADING);
			if (isDLing)	//Cax2 - no unneded calls to showdownloadingparts...
				ShowDownloadingParts(gettingParts);		// Barry - was only showing one part from client, even when reserved bits from 2 parts
			uint32 uEnd; //Cax2 less cpu usage
			for (uint32 i = 0;i < m_nPartCount;i++)
			{ 	
				//--- xrmb:seetheneed --- & Cax2 graphic glitch fix
				if (PARTSIZE*(i+1) > filesize) 
					uEnd = filesize; 
				else 
					uEnd = PARTSIZE*(i+1); 
				if (m_abyPartStatus[i]){ //they have this part
					// do i have it ?
					if  (reqfile->IsComplete(PARTSIZE*i,PARTSIZE*(i+1)-1))
						statusBar.FillRange(PARTSIZE*i,  uEnd, crBoth); 
					else if (isDLing && m_nLastBlockOffset < uEnd && 
														m_nLastBlockOffset >= PARTSIZE*i)
							statusBar.FillRange(PARTSIZE*i,  uEnd, crPending);      
					else if (isDLing && gettingParts[i])	//Cax2 - no more flashes of yellow across all clients...
						statusBar.FillRange(PARTSIZE*i, uEnd, crNextPending);
	    			else
						statusBar.FillRange(PARTSIZE*i, uEnd, crClientOnly);
				}
				else //they don't have a part. Do I have it?
					if (reqfile->IsComplete(PARTSIZE*i,PARTSIZE*(i+1)-1)) 
						statusBar.FillRange(PARTSIZE*i,  uEnd, crMeOnly); 
				//--- :xrmb --- & Cax2 graphic glitch fix
			} 
   		} 
	}
   	statusBar.Draw(dc, rect->left, rect->top, bFlat); 

	EMULE_CATCH
} 


bool CUpDownClient::Compare(CUpDownClient* tocomp){
	EMULE_TRY

	if (GetIP() && tocomp->GetIP() && GetIP() == tocomp->GetIP() && GetUserPort() == tocomp->GetUserPort())
		return true;
	if(HasValidHash() && tocomp->HasValidHash())
		return !md4cmp(GetUserHash(), tocomp->GetUserHash());
	if (HasLowID())
		return ((GetUserID() == tocomp->GetUserID()) && (GetServerIP() == tocomp->GetServerIP())&& (GetUserPort() == tocomp->GetUserPort()));
	else
		return ((GetUserID() == tocomp->GetUserID() && GetUserPort() == tocomp->GetUserPort()) || (GetIP() && (GetIP() == tocomp->GetIP() && GetUserPort() == tocomp->GetUserPort())) );

	EMULE_CATCH

	return false;
}

//eklmn: return TRUE if client still connected, otherwise return false
bool CUpDownClient::AskForDownload()
{
	EMULE_TRY

#ifdef OLD_SOCKETS_ENABLED
	if (theApp.listensocket->TooManySockets() && !(socket && socket->IsConnected()) )
	{
		if (GetDownloadState() != DS_TOOMANYCONNS &&  GetDownloadState() != DS_DOWNLOADING)
			SetDownloadState(DS_TOOMANYCONNS);

		// Added by Tarod [Dnowo] 
		// Vorlost halved the code: asking time random only at the beginning for each new source 
		if (m_dwLastAskedTime == 0) 
			m_dwLastAskedTime = GetTickCount() - FILEREASKTIME + (uint32)ROUND(((float)rand() / RAND_MAX) * 300000); 

		return true;
	}
#endif //OLD_SOCKETS_ENABLED

	m_bUDPPending = false;
	m_dwLastAskedTime = GetTickCount();
	SetDownloadState(DS_CONNECTING);

	if (TryToConnect())
		return true;
	else
	{
		theApp.downloadqueue->RemoveSource(this);
		return false;
	}

	EMULE_CATCH
	return false;
}

bool CUpDownClient::IsSourceRequestAllowed() 
{
	EMULE_TRY

	DWORD dwTickCount = GetTickCount() + CONNECTION_LATENCY;
	int nTimePassedClient = dwTickCount - GetLastSrcAnswerTime();
	int nTimePassedFile   = dwTickCount - reqfile->GetLastAnsweredTime();
	bool bNeverAskedBefore = GetLastSrcReqTime() == 0;

	return (
	         //if client has the correct extended protocol
	         ExtProtocolAvailable() && m_bySourceExchangeVer >= 1 &&
			 //AND if we can send sources
			 !theApp.glob_prefs->IsDisabledXS() &&
	         //AND if we need more sources
	         theApp.glob_prefs->GetMaxSourcePerFileSoft() > reqfile->GetSourceCount() &&
	         //AND if...
	         (
	           //source is not complete and file is rare, allow once every 10 minutes
	           ( !m_bCompleteSource &&
	             ( reqfile->GetSourceCount() - reqfile->GetValidSourcesCount() <= RARE_FILE / 4 ||
			       reqfile->GetSourceCount() <= RARE_FILE * 2
			     ) &&
	             (bNeverAskedBefore || nTimePassedClient > SOURCECLIENTREASK)
	           ) ||
	           // otherwise, allow every 90 minutes, but only if we haven't
	           //   asked someone else in last 10 minutes
			   ( (bNeverAskedBefore || nTimePassedClient > SOURCECLIENTREASK * reqfile->GetCommonFilePenalty()) &&
		         (nTimePassedFile > SOURCECLIENTREASK)
	           )
	         )
	       );

	EMULE_CATCH

	return false;
}

void CUpDownClient::SendFileRequest()
{
	EMULE_TRY

	ASSERT(reqfile != NULL);
	if(!reqfile)
		return;
	AddAskedCountDown();

	CSafeMemFile data1(16+16);
	data1.Write(reqfile->GetFileHash(),16);
	if( GetExtendedRequestsVersion() > 0 )
		reqfile->WritePartStatus(&data1);
	if( GetExtendedRequestsVersion() > 1 )
		reqfile->WriteCompleteSourcesCount(&data1); //netwolf: complete sources (zegzav) 13.05.03
	Packet* packet = new Packet(&data1);
	packet->opcode=OP_FILEREQUEST;
	theApp.uploadqueue->AddUpDataOverheadFileRequest(packet->size);
#ifdef OLD_SOCKETS_ENABLED
	socket->SendPacket(packet, true);
#endif //OLD_SOCKETS_ENABLED
	//AddDebugLogLine(false,"File Status: %s was requested from %s", reqfile->GetFileName(), this->GetUserName());
	
	CSafeMemFile data2(16);
	data2.Write(reqfile->GetFileHash(),16);
	packet = new Packet(&data2);
	packet->opcode = OP_SETREQFILEID;
	theApp.uploadqueue->AddUpDataOverheadFileRequest(packet->size);
#ifdef OLD_SOCKETS_ENABLED
	socket->SendPacket(packet, true);	
#endif //OLD_SOCKETS_ENABLED
	if ( IsEmuleClient() ){
		SetRemoteQueueFull( true );
	SetRemoteQueueRank(0);
	}	
	if (theApp.glob_prefs->GetMaxSourcePerFileSoft() < reqfile->GetSourceCount() || theApp.glob_prefs->IsDisabledXS())
		return;
	if (theApp.glob_prefs->DisableXSUpTo() && theApp.glob_prefs->XSUpTo() < reqfile->GetSourceCount()) // DonGato - Disable XS for up to X sources
		return;
	if(IsSourceRequestAllowed()) {
		reqfile->SetLastAnsweredTimeTimeout();
		Packet* packet = new Packet(OP_REQUESTSOURCES,16,OP_EMULEPROT);
		md4cpy(packet->pBuffer,reqfile->GetFileHash());
		theApp.uploadqueue->AddUpDataOverheadSourceExchange(packet->size);
#ifdef OLD_SOCKETS_ENABLED
		socket->SendPacket(packet,true,true);
#endif //OLD_SOCKETS_ENABLED
		//AddDebugLogLine(false, "Send:Source Request User(%s) File(%s)", GetUserName(), reqfile->GetFileName());
	}

	EMULE_CATCH
}

void CUpDownClient::ProcessFileInfo(char* packet,uint32 size)
{
	EMULE_TRY

	CSafeMemFile data((BYTE*)packet,size);
	uchar cfilehash[16];
	data.Read(cfilehash,16);
	uint16 namelength;
	data.Read(&namelength,2);	
	data.Read(m_strClientFilename.GetBuffer(namelength),namelength);	//SyruS (0.29c)
	m_strClientFilename.ReleaseBuffer(namelength);
	m_dwLastAskedTime = ::GetTickCount(); //<<-- enkeyDEV(th1) -L2HAC- highid side
	
	//netwolf: patch 'fix disappearing reqfile' START (taab)
	if (!reqfile)
	{
		CKnownFile* locreqfile = theApp.sharedfiles->GetFileByID(cfilehash);
		if (locreqfile && locreqfile->IsPartFile())
			reqfile = (CPartFile *)locreqfile;
	}
	//netwolf END
	
	if (reqfile==NULL)	//SyruS (0.29c)
	{
		AddDebugLogLine(false, GetResString(IDS_ERR_WRONGFILEID)+ _T(" (ProcessFileInfo; reqfile==NULL)"));
		return;
	}
	if (md4cmp(cfilehash,reqfile->GetFileHash()))
		AddDebugLogLine(false, GetResString(IDS_ERR_WRONGFILEID)+ _T(" (ProcessFileInfo; reqfile!=cfilehash)"));

	EMULE_CATCH
}

void CUpDownClient::ProcessFileStatus(char* packet,uint32 size){
	EMULE_TRY

	CSafeMemFile data((BYTE*)packet,size);	//TwoBottle Mod (exceptions!)
	uchar cfilehash[16];
	data.Read(cfilehash,16);	
	//netwolf: patch 'fix disappearing reqfile part2' START (taab)
	if (!reqfile)
	{
		CKnownFile* locreqfile = theApp.sharedfiles->GetFileByID(cfilehash);
		if (locreqfile && locreqfile->IsPartFile())
			reqfile = (CPartFile *)locreqfile;
	}
	//newolf END
	
	if ( (!reqfile) || md4cmp(cfilehash,reqfile->GetFileHash())){
		//delete data;	//mf	//TwoBottle Mod
		if (reqfile==NULL)	//SyruS (0.29c)
			AddDebugLogLine(false, GetResString(IDS_ERR_WRONGFILEID) + _T(" (ProcessFileStatus; reqfile==NULL)"));
		else
			AddDebugLogLine(false, GetResString(IDS_ERR_WRONGFILEID)+ _T(" (ProcessFileStatus; reqfile!=cfilehash)"));
		return;
	}
	data.Read(&m_nPartCount,2);
	if (m_abyPartStatus){
		delete[] m_abyPartStatus;
		m_abyPartStatus = NULL;
    }
	bool bPartsNeeded = false;
	if (!m_nPartCount)
	{
		m_nPartCount = reqfile->GetPartCount();
		m_abyPartStatus = new uint8[m_nPartCount];
		memset(m_abyPartStatus,1,m_nPartCount);
		bPartsNeeded = true;
		m_bCompleteSource = true;
	}
	else
	{
		if (reqfile->GetPartCount() != m_nPartCount){
			//delete data;	//mf	//TwoBottle Mod
			m_nPartCount = 0;	//SyruS (0.29c)
			throw GetResString(IDS_ERR_WRONGPARTNUMBER);
	}
		m_bCompleteSource = false;
		uint16 nCompleteSourcePart= 0;	// zegzav:completesrc (add)
		m_abyPartStatus = new uint8[m_nPartCount];
		uint16 done = 0;
		while (done != m_nPartCount){
			uint8 toread;
			data.Read(&toread,1);
			for (sint32 i = 0;i != 8;i++){
				m_abyPartStatus[done] = ((toread>>i)&1)? 1:0; 	
				// netwolf: zegzav completesrc - BEGIN
				if (m_abyPartStatus[done])
				{
					nCompleteSourcePart++;
					if (!reqfile->IsComplete(done*PARTSIZE,((done+1)*PARTSIZE)-1))
					bPartsNeeded = true;
				}
				// netwolf: zegzav completesrc - END
				done++;
				if (done == m_nPartCount)
					break;
			}
		}
		// netwolf: zegzav completesrc - BEGIN
		if (nCompleteSourcePart == m_nPartCount)	
			m_bCompleteSource= true;		
		// netwolf: zegzav completesrc - END
	}

	//theApp.emuledlg->transferwnd.m_wndDownloadList.UpdateItem(this);
	UpdateDisplayedInfo();
	reqfile->NewSrcPartsInfo(); // eklmn: changes(00): NewSrcPartsInfo & UpdateAvailablePartsCount merged (CPU load)
	//reqfile->UpdateAvailablePartsCount();

    CheckForUpPartsInfo(); // obaldin

	if (!bPartsNeeded)
		SetDownloadState(DS_NONEEDEDPARTS);
	else if (reqfile->hashsetneeded){
		Packet* packet = new Packet(OP_HASHSETREQUEST,16);
		md4cpy(packet->pBuffer,reqfile->GetFileHash());
		theApp.uploadqueue->AddUpDataOverheadFileRequest(packet->size);
#ifdef OLD_SOCKETS_ENABLED
		socket->SendPacket(packet, true, true);
#endif //OLD_SOCKETS_ENABLED
		SetDownloadState(DS_REQHASHSET);
		reqfile->hashsetneeded = false;
	}
	else
	{
		SetDownloadState(DS_ONQUEUE);
		//delete data;	//TwoBottle Mod
		CSafeMemFile data1(16);	//TwoBottle Mod
		data1.Write(reqfile->GetFileHash(),16);
		Packet* packet = new Packet(&data1);
		packet->opcode = OP_STARTUPLOADREQ;
		theApp.uploadqueue->AddUpDataOverheadFileRequest(packet->size);
#ifdef OLD_SOCKETS_ENABLED
		socket->SendPacket(packet, true, true);
#endif //OLD_SOCKETS_ENABLED
		m_dwLastAskedTime = ::GetTickCount(); //<<-- enkeyDEV(th1) -L2HAC- highid side
		//AddDebugLogLine(false,"File Upload: %s was requested from %s", reqfile->GetFileName(), this->GetUserName());
	}
		
	//delete data;	//TwoBottle Mod

	EMULE_CATCH
}

bool CUpDownClient::AddRequestForAnotherFile(CPartFile* file)
{
	EMULE_TRY

	if (file == NULL) return false;
	
	// check NNS list
	if (m_OtherNoNeeded_list.Find(file) != 0) return false;	// eklmn: optimization(01): search with Find

	//check Other list
	if (m_OtherRequests_list.Find(file) != 0) return false;	// eklmn: optimization(01): search with Find

	m_OtherRequests_list.AddTail(file);
	file->A4AF_srclist.AddTail(this);	// add in A4AF file list
	
	return true;

	EMULE_CATCH

	return false;
}

void CUpDownClient::SetDownloadState(eDLQState byNewState)
{
	EMULE_TRY

	if (m_nDownloadState != byNewState)
	{
		if(byNewState == DS_DOWNLOADING)
		{
			reqfile->AddDownloadingSource(this);
			if ( IsEmuleClient() )
				SetRemoteQueueFull(false);
			SetRemoteQueueRank(0);
			SetAskedCountDown(0);
			m_dwLastTimeCalculatedDR = ::GetTickCount();	//eklmn: new DL measurements system
			//v2: eklmn(khaos): update stattistical variables (start time of DL & minimal amount)
			m_dwDownStartTime = ::GetTickCount();
			m_bTransferredDownMini=false;
		}
		else if (byNewState == DS_CONNECTED)
			m_dwEnteredConnectedState = ::GetTickCount();
		else if (m_nDownloadState == DS_DOWNLOADING )
		{
			// -khaos--+++> Extended Statistics (Successful/Failed Download Sessions)
			if ( m_bTransferredDownMini && byNewState != DS_ERROR )
				theApp.glob_prefs->Add2DownSuccessfulSessions(); // Increment our counters for successful sessions (Cumulative AND Session)
			else
				theApp.glob_prefs->Add2DownFailedSessions(); // Increment our counters failed sessions (Cumulative AND Session)
			theApp.glob_prefs->Add2DownSAvgTime((::GetTickCount() - m_dwDownStartTime)/1000);
			// <-----khaos-
			
			reqfile->RemoveDownloadingSource(this);
			m_nDownloadState = byNewState;
			for (POSITION pos = m_DownloadBlocks_list.GetHeadPosition();pos != 0;m_DownloadBlocks_list.GetNext(pos)){
				Requested_Block_Struct* cur_block = m_DownloadBlocks_list.GetAt(pos);
				reqfile->RemoveBlockFromList(cur_block->StartOffset,cur_block->EndOffset);
				delete m_DownloadBlocks_list.GetAt(pos);
			}
			m_DownloadBlocks_list.RemoveAll();

			for (POSITION pos = m_PendingBlocks_list.GetHeadPosition();pos != 0;m_PendingBlocks_list.GetNext(pos))
			{
				Pending_Block_Struct *pending = m_PendingBlocks_list.GetAt(pos);
				if (reqfile)
					reqfile->RemoveBlockFromList(pending->block->StartOffset, pending->block->EndOffset);
				
				delete pending->block;
				// Not always allocated
				if (pending->zStream) {
					inflateEnd(pending->zStream);
					delete pending->zStream;
				}
				delete pending;
			}
			m_PendingBlocks_list.RemoveAll();
			m_nDownDatarate = 0;
			if (byNewState == DS_NONE){
				if (m_abyPartStatus)
					delete[] m_abyPartStatus;
				m_abyPartStatus = 0;
				m_nPartCount = 0;	//SyruS (0.29c)
			}
			m_LastPartAsked = 0xffff; //jicxicmic
			
#ifdef OLD_SOCKETS_ENABLED
			if (socket && byNewState != DS_ERROR)
					socket->DisableDownloadLimit();
#endif //OLD_SOCKETS_ENABLED
		}

		// switch the state & update info
		m_nDownloadState = byNewState;
		UpdateDisplayedInfo(true);
	}

	EMULE_CATCH
}

void CUpDownClient::ProcessHashSet(char* packet,uint32 size)
{
	EMULE_TRY

	if ( (!reqfile) || md4cmp(packet,reqfile->GetFileHash()))
		throw CString(GetResString(IDS_ERR_WRONGFILEID)+_T(" (ProcessHashSet)"));	
	CSafeMemFile data1((BYTE*)packet,size);
	if (reqfile->LoadHashsetFromFile(data1,true)){
	}
	else{
		reqfile->hashsetneeded = true;		
		throw GetResString(IDS_ERR_BADHASHSET);
	}
	SetDownloadState(DS_ONQUEUE);
	
	CSafeMemFile data2(16);
	data2.Write(this->reqfile->GetFileHash(),16);
	Packet* opacket = new Packet(&data2);
	opacket->opcode = OP_STARTUPLOADREQ;
	theApp.uploadqueue->AddUpDataOverheadFileRequest(opacket->size);
#ifdef OLD_SOCKETS_ENABLED
	socket->SendPacket(opacket, true, true);
#endif //OLD_SOCKETS_ENABLED
	m_dwLastAskedTime = ::GetTickCount(); //<<-- enkeyDEV(th1) -L2HAC- highid side

	EMULE_CATCH
}

void CUpDownClient::SendBlockRequests()
{
	EMULE_TRY

	m_dwLastBlockReceived = GetTickCount();

	if (!reqfile)
		return;

	//jicxicmic - get blocks from the same chunk first
	if (m_DownloadBlocks_list.IsEmpty())
	{
		// check if part was completed
		if (reqfile->GetBytesInPart(m_LastPartAsked, true, true) == 0 
			|| reqfile->IsCorruptedPart(m_LastPartAsked))  m_LastPartAsked = 0xffff; //ICR 2.10
		
		//first time we ask?
		if (m_LastPartAsked == 0xffff)
			m_LastPartAsked = reqfile->GetNextRequestedBlock(this);

		// Barry - instead of getting 3, just get how many is needed
		uint16 count, savedcount;
		savedcount = count = 3 - m_PendingBlocks_list.GetCount();
		Requested_Block_Struct** toadd = new Requested_Block_Struct*[count];
		if (m_LastPartAsked != 0xffff)		//previous active part ?
		{
			reqfile->GetGapsInPart(this, m_LastPartAsked, toadd, &count);
			if (count == 0)		//finished block (part ?)
			{
				// check if part was completed
				if (reqfile->GetBytesInPart(m_LastPartAsked, true, true) == 0 
					|| reqfile->IsCorruptedPart(m_LastPartAsked))  m_LastPartAsked = 0xffff; //ICR 2.10

				m_LastPartAsked = reqfile->GetNextRequestedBlock(this); //get next part
				if (m_LastPartAsked != 0xffff)		//more parts ?
				{
					count = savedcount;
					reqfile->GetGapsInPart(this, m_LastPartAsked, toadd, &count);
				}
			}
		}
		else
			count=0;

		for (int i = 0; i < count; i++) {
			m_DownloadBlocks_list.AddTail(toadd[i]);
		}
		delete[] toadd;
	}

	// Barry - Why are unfinished blocks requested again, not just new ones?
	while (m_PendingBlocks_list.GetCount() < 3 && !m_DownloadBlocks_list.IsEmpty()){
		Pending_Block_Struct* pblock = new Pending_Block_Struct;
		pblock->block = m_DownloadBlocks_list.RemoveHead();
		pblock->zStream = NULL;
		pblock->totalUnzipped = 0;
		m_PendingBlocks_list.AddTail(pblock);
	}
	if (m_PendingBlocks_list.IsEmpty()){
		Packet* packet = new Packet(OP_CANCELTRANSFER,0);
		theApp.uploadqueue->AddUpDataOverheadFileRequest(packet->size);
#ifdef OLD_SOCKETS_ENABLED
		socket->SendPacket(packet,true,true);
#endif //OLD_SOCKETS_ENABLED
		//SetDownloadState(DS_NONEEDEDPARTS);
		if ((GetTickCount()-GetLastAskedTime())>FILEREASKTIME) 
      		AskForDownload(); //<<--enkeyDEV(Ottavio84) modified by lovelace - reask for file 
      		AddDebugLogLine(false, "-> Cancel download from %s due empty block list", this->GetUserName());
		return;
	}
	Packet* packet = new Packet(OP_REQUESTPARTS,40);
	CSafeMemFile data((BYTE*)packet->pBuffer,40);
	data.Write(reqfile->GetFileHash(),16);
	POSITION pos = m_PendingBlocks_list.GetHeadPosition();
	uint32 null = 0;
	Requested_Block_Struct* block;
	for (uint32 i = 0; i != 3; i++){
		if (pos){
			block = m_PendingBlocks_list.GetAt(pos)->block;
			m_PendingBlocks_list.GetNext(pos);
			data.Write(&block->StartOffset,4);
		}
		else
			data.Write(&null,4);
	}
	pos = m_PendingBlocks_list.GetHeadPosition();
	for (uint32 i = 0; i != 3; i++){
		if (pos){
			block = m_PendingBlocks_list.GetAt(pos)->block;
			m_PendingBlocks_list.GetNext(pos);
			uint32 endpos = block->EndOffset+1;
			data.Write(&endpos,4);
		}
		else
			data.Write(&null,4);
	}
	theApp.uploadqueue->AddUpDataOverheadFileRequest(packet->size);
#ifdef OLD_SOCKETS_ENABLED
	socket->SendPacket(packet,true,true);
#endif //OLD_SOCKETS_ENABLED

	EMULE_CATCH
}

/* Barry - Originally this only wrote to disk when a full 180k block 
           had been received from a client, and only asked for data in 
		   180k blocks.

		   This meant that on average 90k was lost for every connection
		   to a client data source. That is a lot of wasted data.

		   To reduce the lost data, packets are now written to a buffer
		   and flushed to disk regularly regardless of size downloaded.
		   This includes compressed packets.

		   Data is also requested only where gaps are, not in 180k blocks.
		   The requests will still not exceed 180k, but may be smaller to
		   fill a gap.
*/
void CUpDownClient::ProcessBlockPacket(char* packet, uint32 size, bool packed)
{
	EMULE_TRY

	// Ignore if no data required
	if (!(GetDownloadState() == DS_DOWNLOADING || GetDownloadState() == DS_NONEEDEDPARTS))
		return;

	const int HEADER_SIZE = 24;

	// Update stats
	m_dwLastBlockReceived = ::GetTickCount();

	// Read data from packet
	CSafeMemFile data((BYTE*)packet, size);
	uchar fileID[16];
	data.Read(fileID, 16);

	// Check that this data is for the correct file
	if ( (!reqfile) || md4cmp(packet, reqfile->GetFileHash()))
	{
		throw CString(GetResString(IDS_ERR_WRONGFILEID)+_T(" (ProcessBlockPacket)"));
	}

	// Find the start & end positions, and size of this chunk of data
	uint32 nStartPos;
	uint32 nEndPos;
	uint32 nBlockSize = 0;
	data.Read(&nStartPos, 4);
	if (packed)
	{
		data.Read(&nBlockSize, 4);
		nEndPos = nStartPos + (size - HEADER_SIZE);
		usedcompressiondown = true;
	}
	else
		data.Read(&nEndPos,4);
		
	// Check that packet size matches the declared data size + header size (24)
	if ( size != ((nEndPos - nStartPos) + HEADER_SIZE))
		throw CString(GetResString(IDS_ERR_BADDATABLOCK)+_T(" (ProcessBlockPacket)"));

	// -khaos--+++>
	// Extended statistics information based on which client and remote port sent this data.
	// The new function adds the bytes to the grand total as well as the given client/port.
	// bFromPF is not relevant to downloaded data.  It is purely an uploads statistic.
	theApp.glob_prefs->Add2SessionTransferData(GetClientSoft(), GetUserPort(), false, Download, size - HEADER_SIZE);
	// theApp.UpdateReceivedBytes(size - HEADER_SIZE);
	// <-----khaos-

	// Move end back one, should be inclusive 
	uint32 DownDataRateMS = size - HEADER_SIZE;
	credits->AddDownloaded(size - HEADER_SIZE,this->GetIP());
	nEndPos--;

	m_nTransferedDown += DownDataRateMS;
	m_TransferedInLastPeriod += DownDataRateMS;
	
	// Loop through to find the reserved block that this is within
	Pending_Block_Struct* cur_block;
	POSITION pos1, pos2;
	for (pos1 = m_PendingBlocks_list.GetHeadPosition(); ( pos2 = pos1 ) != NULL;)
	{
		cur_block = m_PendingBlocks_list.GetNext( pos1 );
		//Cax2: I'm not happy with this whole if clause
		if ((cur_block->block->StartOffset <= nStartPos) && (cur_block->block->EndOffset >= nStartPos))
		{
			// Found reserved block

			// Remember this start pos, used to draw part downloading in list
			m_nLastBlockOffset = nStartPos;  

			// Occasionally packets are duplicated, no point writing it twice
			// This will be 0 in these cases, or the length written otherwise
			uint32 lenWritten = 0;

			// Handle differently depending on whether packed or not
			if (!packed)
			{
				// Write to disk (will be buffered in part file class)
				lenWritten = reqfile->WriteToBuffer(size - HEADER_SIZE, 
													(BYTE *) (packet + HEADER_SIZE),
													nStartPos,
													nEndPos,
													cur_block->block );
			}
			else // Packed
			{
				// Create space to store unzipped data, the size is only an initial guess, will be resized in unzip() if not big enough
				uint32 lenUnzipped = (size * 2); 
				// Don't get too big
				if (lenUnzipped > (EMBLOCKSIZE + 300))
					lenUnzipped = (EMBLOCKSIZE + 300);
				BYTE *unzipped = new BYTE[lenUnzipped];

				// Try to unzip the packet
				int result = unzip(cur_block, (BYTE *)(packet + HEADER_SIZE), (size - HEADER_SIZE), &unzipped, &lenUnzipped);
				if (result == Z_OK)
				{
					// Write any unzipped data to disk
					if (lenUnzipped > 0)
					{
						// Use the current start and end positions for the uncompressed data
						nStartPos = cur_block->block->StartOffset + cur_block->totalUnzipped - lenUnzipped;
						nEndPos = cur_block->block->StartOffset + cur_block->totalUnzipped - 1;
					
						if(nBlockSize*25 < lenUnzipped) {
							
							m_LastPartAsked = 0xFFFF;	//eklmn: feature(00): ICR (reset choosen part in case suspicious block)
								
				       		CString buffer;
				       		CString buffer2 = _T("");
				       		for (uint16 i = 0;i != 16;i++) {
								buffer.Format(_T("%02X"),GetUserHash()[i]);
								buffer2+=buffer;
				       		}

							AddDebugLogLine(true,  _T("Received suspicious block: file '%s', part %i, block %i, blocksize %i, comp. blocksize %i)"), reqfile->GetFileName(), cur_block->block->StartOffset/PARTSIZE, cur_block->block->StartOffset/EMBLOCKSIZE, lenUnzipped, nBlockSize);
       						AddDebugLogLine(false, _T("username '%s', ip %i.%i.%i.%i:%i, hash %s"), m_pszUsername, (uint8)m_dwUserIP, (uint8)(m_dwUserIP>>8), (uint8)(m_dwUserIP>>16),(uint8)(m_dwUserIP>>24), GetUserPort(), buffer2);

							if(reqfile->GetDiscardSuperCompressed()) {
								//--- (un)comment the block-dropping as you wish---
								reqfile->RemoveBlockFromList(cur_block->block->StartOffset,cur_block->block->StartOffset+1);
								uint32 ip1 = 0;
								CString buf, temp;
								int counter = 0;
								buf.Format(_T("%i.%i.%i.%i"), (uint8)m_dwUserIP, (uint8)(m_dwUserIP>>8), (uint8)(m_dwUserIP>>16),(uint8)(m_dwUserIP>>24));
								for(int i=0; i<4; i++) { 
									temp = buf.Tokenize(_T("."),counter);
									ip1 += _tstoi(temp) * pow(256,3-i);
								}

								theApp.ipfilter->AddBannedIPRange(ip1,ip1,1,_T("autobanned"));
								theApp.downloadqueue->RemoveSource(this);
								AddDebugLogLine(false, _T("block dropped & IP locked out!"));
							}
							// BavarianSnail
							else {
								if(nEndPos > cur_block->block->EndOffset) {
									AddDebugLogLine(false, IDS_ERR_CORRUPTCOMPRPKG, reqfile->GetFileName(), 666);
									reqfile->RemoveBlockFromList(cur_block->block->StartOffset, cur_block->block->EndOffset);
								}
								else {
									// Write uncompressed data to file
									lenWritten = reqfile->WriteToBuffer(size - HEADER_SIZE,
						                                                 unzipped,
						                                                 nStartPos,
						                                                 nEndPos,
						                                                 cur_block->block);
								}
							}
							//- BavarianSnail
						}
						else {
							if( nEndPos > cur_block->block->EndOffset)
							{
								AddDebugLogLine(false, IDS_ERR_CORRUPTCOMPRPKG,reqfile->GetFileName(),666);
								reqfile->RemoveBlockFromList(cur_block->block->StartOffset, cur_block->block->EndOffset);
							}
							else
							{
						    	// Write uncompressed data to file
						    	lenWritten = reqfile->WriteToBuffer(size - HEADER_SIZE,
																    unzipped,
																    nStartPos,
															 	   nEndPos,
															 	   cur_block->block );
							}
						}
					}
				}
				else
				{
					AddDebugLogLine(false, IDS_ERR_CORRUPTCOMPRPKG, reqfile->GetFileName(), result);
					reqfile->RemoveBlockFromList(cur_block->block->StartOffset, cur_block->block->EndOffset);
				}
				try{
					if(unzipped)
						delete [] unzipped;
				} catch(...){}
			}

			// These checks only need to be done if any data was written
			if (lenWritten > 0)
			{
				//m_nTransferedDown += lenWritten;
				m_bTransferredDownMini = true;		// -khaos--+++> For determining whether the current download session was a success or not.
				
				// If finished reserved block
				// in this place becouse reqfile->WriteToBuffer was called
				// cur_block can be already deleted (if file was completed)
				// so to avoid access violation here we will check
				// m_PendingBlocks_list for emptyness....possibly there is
				// better check [TwoBottle Mod]
				if (!m_PendingBlocks_list.IsEmpty() && nEndPos == cur_block->block->EndOffset)
				{
					m_PendingBlocks_list.RemoveAt(pos2);

					reqfile->RemoveBlockFromList(cur_block->block->StartOffset, cur_block->block->EndOffset);
					delete cur_block->block;
					// Not always allocated
					if (cur_block->zStream) {
						inflateEnd(cur_block->zStream);
						delete cur_block->zStream;
					}
					delete cur_block;

					// Request next block
					SendBlockRequests();	
				}
			}

			// Stop looping and exit method
			return;
		}
	}
	EMULE_CATCH
}

int CUpDownClient::unzip(Pending_Block_Struct *block, BYTE *zipped, uint32 lenZipped, BYTE **unzipped, uint32 *lenUnzipped, bool recursive)
{
  int err = Z_DATA_ERROR;
  EMULE_TRY
	// Save some typing
	z_stream *zS = block->zStream;

	// Is this the first time this block has been unzipped
	if (zS == NULL)
	{
		// Create stream
		block->zStream = new z_stream;
		zS = block->zStream;

		// Initialise stream values
		zS->zalloc = (alloc_func)0;
		zS->zfree = (free_func)0;
		zS->opaque = (voidpf)0;

		// Set output data streams, do this here to avoid overwriting on recursive calls
		zS->next_out = (*unzipped);
		zS->avail_out = (*lenUnzipped);

		// Initialise the z_stream
		err = inflateInit(zS);
		if (err != Z_OK)
			return err;
	}

	// Use whatever input is provided
	zS->next_in  = zipped;
	zS->avail_in = lenZipped;

	// Only set the output if not being called recursively
	if (!recursive)
	{
		zS->next_out = (*unzipped);
		zS->avail_out = (*lenUnzipped);
	}

	// Try to unzip the data
	err = inflate(zS, Z_SYNC_FLUSH);

	// Is zip finished reading all currently available input and writing all generated output
	if (err == Z_STREAM_END)
	{
		// Finish up
		err = inflateEnd(zS);
		block->zStream = NULL;
				
		if (err != Z_OK) {
			delete zS;
			return err;
		}

		// Got a good result, set the size to the amount unzipped in this call (including all recursive calls)
		(*lenUnzipped) = (zS->total_out - block->totalUnzipped);
		block->totalUnzipped = zS->total_out;
		delete zS;
		zS = NULL; //just for future
	}
	else if ((err == Z_OK) && (zS->avail_out == 0))
	{
		// Output array was not big enough, call recursively until there is enough space

		// What size should we try next
		uint32 newLength = (*lenUnzipped) *= 2;
		if (newLength == 0)
			newLength = lenZipped * 2;

		// Copy any data that was successfully unzipped to new array
		BYTE *temp = new BYTE[newLength];
		#ifndef AMD		
		memcpy(temp, (*unzipped), (zS->total_out - block->totalUnzipped));
		#else
		memcpy_amd(temp, (*unzipped), (zS->total_out - block->totalUnzipped));
		#endif
		delete [] (*unzipped);
		(*unzipped) = temp;
		(*lenUnzipped) = newLength;

		// Position stream output to correct place in new array
		zS->next_out = (*unzipped) + (zS->total_out - block->totalUnzipped);
		zS->avail_out = (*lenUnzipped) - (zS->total_out - block->totalUnzipped);

		// Try again
		err = unzip(block, zS->next_in, zS->avail_in, unzipped, lenUnzipped, true);
	}
	else if ((err == Z_OK) && (zS->avail_in == 0))
	{
		// All available input has been processed, everything ok.
		// Set the size to the amount unzipped in this call (including all recursive calls)
		(*lenUnzipped) = (zS->total_out - block->totalUnzipped);
		block->totalUnzipped = zS->total_out;
		//Cax2 - not needed here, it stops downloads prematurely
		//inflateEnd(zS);
	}
	else
	{
		// Should not get here unless input data is corrupt
		AddDebugLogLine(false,_T("Unexpected zip error"));
		//DebugBreak(); I removed this so that we could let the client run for more the five minutes.. Barry needs to see if there is or isn't a preoblem here..
		err=inflateEnd(zS);
		block->zStream = NULL;
		delete zS;
		zS = NULL; //just for future
	}

	if (err != Z_OK)
		(*lenUnzipped) = 0;
  EMULE_CATCH
  return err;
}

uint32 CUpDownClient::CalculateDownloadRate()
{
	EMULE_TRY

	uint32 curTick = GetTickCount();

	if ((curTick- m_dwLastBlockReceived) > DOWNLOADTIMEOUT)
	{
		if (socket != NULL) {
			Packet* packet = new Packet(OP_CANCELTRANSFER,0);
			theApp.uploadqueue->AddUpDataOverheadFileRequest(packet->size);
#ifdef OLD_SOCKETS_ENABLED
			socket->SendPacket(packet,true,true);
#endif //OLD_SOCKETS_ENABLED
		}
		SetDownloadState(DS_ONQUEUE);
		m_nDownDatarate = 0;
		AddDebugLogLine(false, "-> Client %s (%s) was dropped due timeout %u", GetUserName(), GetClientNameAndVersionString(this),curTick - m_dwLastBlockReceived);
	} 
	else 
	{
		uint32 time_diff_ms = curTick - m_dwLastTimeCalculatedDR;

		//update average sum
		m_Client_Sum_DL_DR_over_last_N_measurements += m_TransferedInLastPeriod;
		m_Client_Sum_DL_tick_over_last_N_measurements += time_diff_ms;
		// update a lists
		m_Average_DL_DR_list.push_front(m_TransferedInLastPeriod);
		m_Average_DL_tick_list.push_front(time_diff_ms);
		// check lists size
		while(m_Average_DL_tick_list.size() > 500) 
		{
			m_Client_Sum_DL_DR_over_last_N_measurements -= m_Average_DL_DR_list.back();
			m_Average_DL_DR_list.pop_back();
			m_Client_Sum_DL_tick_over_last_N_measurements -= m_Average_DL_tick_list.back();
			m_Average_DL_tick_list.pop_back();
		}

		// calculate average trasfered data (bytes)
		uint64 AverageTransferedData = m_Client_Sum_DL_DR_over_last_N_measurements/m_Average_DL_DR_list.size();
		// calculate average time period between measuremets (ms)
		uint64 AverageTransferTimePeriod = m_Client_Sum_DL_tick_over_last_N_measurements/m_Average_DL_tick_list.size();
		
		// calculate a DL rate (b/s)
		if (AverageTransferTimePeriod != 0)
			m_nDownDatarate = 1000*AverageTransferedData/AverageTransferTimePeriod;
		else
			m_nDownDatarate = 0;

		//update a time & reset data transfered over last period
		m_dwLastTimeCalculatedDR = curTick;
		m_TransferedInLastPeriod = 0;
	}
	//Cax2 - the gain was minimal, not worth keeping that...
	UpdateDisplayedInfo();

	return m_nDownDatarate;

	EMULE_CATCH

	return 0;
}

uint16 CUpDownClient::GetAvailablePartCount()
{
	EMULE_TRY

	uint16 result = 0;
	for (int i = 0;i < m_nPartCount;i++){
		if (IsPartAvailable(i))
			result++;
	}
	return result;

	EMULE_CATCH

	return 0;
}

void CUpDownClient::SetRemoteQueueRank(uint16 nr)
{
	EMULE_TRY

	m_nDifferenceQueueRank = (nr-m_nRemoteQueueRank); //cendre-eau
	m_nRemoteQueueRank = nr;
	UpdateDisplayedInfo();

	EMULE_CATCH
}

//eklmn: bugfix(08): A4AF switching
bool CUpDownClient::SwapToAnotherFile(CPartFile* SwapTo, bool bIgnoreNoNeeded)
{
	EMULE_TRY

	// check if we are downloading something from client
	if (GetDownloadState() == DS_DOWNLOADING)
		return false;

	POSITION SwapToPos = 0;
	POSITION cur_FilePos;
	POSITION cur_FileNNSPos = 0;
	CPartFile* cur_file;		//Cax2 CPU optimization
	
	// check: target file is specified? if is not, then find another one 
	if (!SwapTo) {
		// check: does client have another files?
		if (!m_OtherRequests_list.IsEmpty()){
			for (cur_FilePos = m_OtherRequests_list.GetHeadPosition();cur_FilePos != 0;m_OtherRequests_list.GetNext(cur_FilePos)){
				cur_file = m_OtherRequests_list.GetAt(cur_FilePos);
				// get first high priority files
				if (cur_file != reqfile && theApp.downloadqueue->IsPartFile(cur_file) 
					&& (cur_file->GetStatus(false) == PS_READY || cur_file->GetStatus(false) == PS_EMPTY) ) {
						// check: is new file in NNS-list?
						if (!bIgnoreNoNeeded || m_OtherNoNeeded_list.Find(cur_file) == 0 ) {
							SwapTo = cur_file;
							SwapToPos = cur_FilePos ;
							break; // break for cycle, since  file was found
						}
				}
			} // end for
		} // list check
	}
	else if (SwapTo->GetStatus(false) == PS_READY || SwapTo->GetStatus(false) == PS_EMPTY)
		SwapToPos = m_OtherRequests_list.Find(SwapTo); // otherwise get position with .Find()

	// now let's swap the file to another
	if (SwapTo && SwapToPos != 0) {

		// to prevent loop swaping remember NNS-Part
		if (GetDownloadState() == DS_NONEEDEDPARTS && !m_OtherNoNeeded_list.Find(reqfile)) 
			m_OtherNoNeeded_list.AddTail(reqfile);
		// switch state to NONE
		SetDownloadState(DS_NONE);
		if (m_abyPartStatus) {
			delete[] m_abyPartStatus;
			m_abyPartStatus = 0;
		}
		m_nPartCount = 0;	//SyruS (0.29c)
		m_dwLastAskedTime = 0;
		m_LastPartAsked = 0xffff; //jicxicmic
		
		// #zegzav:pastcomment (BEGIN)
		// add to past comment
		reqfile->AddPastComment(this);
		m_iRate=0;
		m_strComment="";
		// #zegzav:pastcomment (END)
		// find current file position in sourceslosts
		cur_FilePos=reqfile->srclists[sourcesslot].Find(this);
		if (cur_FilePos) reqfile->srclists[sourcesslot].RemoveAt(cur_FilePos); // remove current file
		// add them in A4AF list
		reqfile->A4AF_srclist.AddTail(this);
		// add old file to m_OtherRequests_list 
		m_OtherRequests_list.AddTail(reqfile);
		// let's update current information for the removed file
		reqfile->NewSrcPartsInfo(); // eklmn: changes(00): NewSrcPartsInfo & UpdateAvailablePartsCount merged (CPU load)
		// update download list (old file update)
		reqfile->UpdateDisplayedInfo();
		//theApp.emuledlg->transferwnd.m_wndDownloadList.UpdateItem(reqfile);
		
		reqfile = 0;	//reset before delete m_abyPartStatus
		// actual switch
		reqfile = SwapTo;

		// remove new file from OtherRequests list
		m_OtherRequests_list.RemoveAt(SwapToPos);
		// remove current source from A4AF list in new file
		POSITION pos_A4AF = SwapTo->A4AF_srclist.Find(this);
		if (pos_A4AF) SwapTo->A4AF_srclist.RemoveAt(pos_A4AF);
		// to prevent loop swaping remember NNS-Part
		cur_FileNNSPos = m_OtherNoNeeded_list.Find(SwapTo);
		if (cur_FileNNSPos != 0) m_OtherNoNeeded_list.RemoveAt(cur_FileNNSPos);
		// add to souceslots
		if (!SwapTo->srclists[sourcesslot].Find(this))  SwapTo->srclists[sourcesslot].AddTail(this);
		// remove from past comment
		SwapTo->RemovePastComment(this);	// #zegzav:pastcomment
		// Update info for target file
		SwapTo->NewSrcPartsInfo();	// eklmn: changes(00): NewSrcPartsInfo & UpdateAvailablePartsCount merged (CPU load)
		// update download list (new file update)
		SwapTo->UpdateDisplayedInfo();
		//theApp.emuledlg->transferwnd.m_wndDownloadList.UpdateItem(SwapTo);

		// now update the client item
		UpdateDisplayedInfo(true);
		return true;
	}
		
	EMULE_CATCH
	return false;
}
//eklmn: bugfix(08): A4AF switching

void CUpDownClient::UDPReaskACK(uint16 nNewQR)
{
	EMULE_TRY
	m_bUDPPending = false;
	SetRemoteQueueRank(nNewQR);
	m_dwLastAskedTime = ::GetTickCount();

	EMULE_CATCH
}

void CUpDownClient::UDPReaskFNF()
{
	EMULE_TRY
	m_bUDPPending = false;
	//SyruS (0.29a) avoid premature deletion of 'this' client
	if (GetDownloadState() != DS_DOWNLOADING)
	{ 
		AddDebugLogLine(false,CString("UDP ANSWER FNF : %s"),GetUserName());
		theApp.downloadqueue->RemoveSource(this);
#ifdef OLD_SOCKETS_ENABLED
		if (socket)
			Disconnected();
#endif //OLD_SOCKETS_ENABLED
	} 
	else 
		AddDebugLogLine(false,CString("UDP ANSWER FNF : %s (client not removed)"),GetUserName());
	EMULE_CATCH
}

void CUpDownClient::UDPReaskForDownload()
{
	EMULE_TRY
	ASSERT ( reqfile );
	if (!reqfile || m_bUDPPending)
		return;

	// Maverick Disable UDP when connecting to a PROXY
	ProxySettings proxy = theApp.glob_prefs->GetProxy();
#ifdef OLD_SOCKETS_ENABLED
	if(m_byEmuleVersion >= 0x23 && m_nUDPPort != 0 && theApp.glob_prefs->GetUDPPort() != 0 &&
	   !theApp.serverconnect->IsLowID() && !HasLowID() && !(socket && socket->IsConnected()) && (!proxy.UseProxy)) {

			//don't use udp to ask for sources
			if(IsSourceRequestAllowed())
				return;
			m_bUDPPending = true;
		// netwolf: complete sources UDP - BEGIN
		uint16 packetsize= 16;
		if (GetUDPVersion() >= 3)
			packetsize+= 2;
		// netwolf: complete sources UDP - END
		Packet* response = new Packet(OP_REASKFILEPING,packetsize,OP_EMULEPROT);	// netwolf: complete sources UDP
		md4cpy(response->pBuffer,reqfile->GetFileHash());
		// netwolf: complete sources UDP - BEGIN
		if (GetUDPVersion() >= 3)
		{
			uint16 completecount= reqfile->GetCompleteSourcesCount();
			#ifndef AMD
			memcpy(response->pBuffer+16, &completecount, 2);
			#else
			memcpy_amd(response->pBuffer+16, &completecount, 2);
			#endif
		}
		// netwolf: complete sources UDP - END
	        theApp.uploadqueue->AddUpDataOverheadFileRequest(response->size);
			theApp.clientudp->SendPacket(response,GetIP(),GetUDPPort());
		}
#endif //OLD_SOCKETS_ENABLED

   EMULE_CATCH
}

// Barry - Sets string to show parts downloading, eg NNNYNNNNYYNYN
void CUpDownClient::ShowDownloadingParts(_Bvector& parts)
{
	EMULE_TRY

	parts.resize(m_nPartCount);
	for (POSITION pos = m_PendingBlocks_list.GetHeadPosition();pos;)
		parts[m_PendingBlocks_list.GetNext(pos)->block->StartOffset / PARTSIZE]=true;

	EMULE_CATCH
}

void CUpDownClient::UpdateDisplayedInfo(boolean force) 
{
	DWORD curTick = ::GetTickCount();
	if (force || curTick-m_lastRefreshedDLDisplay > MINWAIT_BEFORE_DLDISPLAY_WINDOWUPDATE+(uint32)(rand()*1000/RAND_MAX))
	{
		theApp.emuledlg->transferwnd.m_wndDownloadList.UpdateItem(this);
		m_lastRefreshedDLDisplay = curTick;
	}
}

// New functions by Mr.Fry
uint16 CUpDownClient::GetAvailableOfNeededPartsCount()
{
	EMULE_TRY

	uint16 count = 0;
	if(m_abyPartStatus != NULL && m_nPartCount > 0){
		for(int i=0; i < m_nPartCount ; i++)
			if (m_abyPartStatus[i] && !reqfile->IsComplete(i*PARTSIZE,((i+1)*PARTSIZE)-1))
				count++;
		//TRACE("Counted %u Chunks (parts) that %s has and I need (to download)",count,GetUserName());
	}
	return count;

	EMULE_CATCH

	return 0;
}

