//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#pragma once
#include "types.h"
#include "memdc.h"
#include "titlemenu.h"
#include "Loggable.h"
#include <map>

// Forward declaration
class CPartFile;
class CUpDownClient;

enum ItemType
{
	FILE_TYPE = 1,
	AVAILABLE_SOURCE,
	UNAVAILABLE_SOURCE
};

struct ST_CTRLITEM
{
	ItemType			eType;
	CPartFile*			pOwner;
	void*				pValue;	// could be both CPartFile or CUpDownClient
	ST_CTRLITEM*		pParent;
	DWORD				dwUpdated;
	CBitmap				bmpStatus;

	~ST_CTRLITEM()	
	{
		bmpStatus.DeleteObject();
	}
};

bool IsAskedForAnotherFile(ST_CTRLITEM *pItem);

// CDownloadListCtrl
class CDownloadListCtrl : public CMuleListCtrl, public CLoggable
{
	DECLARE_DYNAMIC(CDownloadListCtrl)

public:
	CDownloadListCtrl();
	virtual ~CDownloadListCtrl();

	void	Init();

	void	AddFile(CPartFile* toadd);
	void	AddSource(CPartFile* owner,CUpDownClient* source,bool notavailable);
	void	RemoveFile(CPartFile* toremove);
	void	RemoveSource(CUpDownClient* source,CPartFile* owner);
	void	ClearCompleted(bool ignorecats=false);
	void	ClearCompleted(const uchar* FileHash /*=NULL*/);

	virtual void DrawItem(LPDRAWITEMSTRUCT lpDrawItemStruct);
	void	UpdateItem(void* toupdate);
		
	void	SetStyle();
	void	Localize();
	
	void	HideFile(CPartFile* tohide);
	void	ShowFile(CPartFile* tohide);
	void	HideSources(CPartFile* toCollapse,bool isShift = false,bool isCtrl = false,bool isAlt = false);

	void	ShowFilesCount();
	void	ShowSelectedFileDetails();	

	CString getTextList();

	void	ExpandCollapseItem(int item,uint8 expand,bool collapsesource=false);
	bool	GetDisplayedFiles(CArray<CPartFile*,CPartFile*> *list);
	
	void	ChangeCategory(int newsel);
	void	MoveCompletedfilesCat(uint8 from, uint8 to);
	
	virtual BOOL OnCommand(WPARAM wParam,LPARAM lParam );

// PLUS -----------------------------------------------------------------------
	void	RestartWaitingDownloads(); 				// Rod
	void	RefreshList();							// Cax2
	void	SortInit(int override);					// Cax2
	bool	IsShowingSources() {return m_ShowSrc;}	// Cax2
	int8	FilesSortIndex;
// ----------------------------------------------------------------------------

protected:
	void	DrawFileItem(CDC *dc, int nColumn, LPRECT lpRect, ST_CTRLITEM* lpCtrlItem);
	void	DrawSourceItem(CDC *dc, int nColumn, LPRECT lpRect, ST_CTRLITEM* lpCtrlItem);
	
	afx_msg void OnItemActivate(NMHDR *pNMHDR, LRESULT *pResult);
	afx_msg void OnContextMenu(CWnd* pWnd, CPoint point);
	afx_msg	void OnColumnClick( NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnListModified(NMHDR *pNMHDR, LRESULT *pResult);
	afx_msg void OnNMDblclkDownloadlist(NMHDR *pNMHDR, LRESULT *pResult);
	afx_msg void OnGetDispInfo(NMHDR* pNMHDR, LRESULT* pResult);

	static int CALLBACK SortProc(LPARAM lParam1, LPARAM lParam2, LPARAM lParamSort);
    static int Compare(CPartFile* file1, CPartFile* file2, LPARAM lParamSort);
	static int Compare(CUpDownClient* client1,CUpDownClient* client2, LPARAM lParamSort, int sortMod);

	DECLARE_MESSAGE_MAP()

// PLUS -----------------------------------------------------------------------	
	afx_msg void OnNMClick(NMHDR *pNMHDR, LRESULT *pResult);
	afx_msg void OnSize(UINT nType, int cx, int cy);
	afx_msg void OnKeyUp(UINT nChar, UINT nRepCnt, UINT nFlags);
	void RefreshInfo(void);	
// ----------------------------------------------------------------------------	

public:
	uint8	m_nCurTab;

private:
	CImageList  	m_ImageList;
	CTitleMenu		m_ClientMenu;
	CMenu			m_PrioMenu;
	CTitleMenu		m_FileMenu;
	CMenu			m_Web;

	typedef std::pair<void*, ST_CTRLITEM*> ListItemsPair;
	typedef std::multimap<void*, ST_CTRLITEM*> ListItems;
    ListItems	m_ListItems;

// PLUS -----------------------------------------------------------------------	
	int         	m_CurrentSortItem;
	bool        	m_SortAscending;
	bool			m_ShowSrc;  		// Cax2 
	int				m_lastSort;
	DWORD			m_sortTime;
	bool			m_dblclicked;		// DonGato
	CMutex			m_MutexAccess;
// ----------------------------------------------------------------------------	

public:
	
};
