//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#pragma once
#include "StdAfx.h"
#include "partfile.h"
#include "opcodes.h"
#include "types.h"
#include "sharedfilelist.h"
#include "preferences.h"
#include "WebServer.h" // kuchin
#include "Loggable.h"

class CClientSource;
class CHostnameSourceWnd;
class CDownloadQueue : public CLoggable
{
	friend class CAddFileThread;
	//friend class CServerSocket;
	friend class CWebServer; // kuchin
public:
	CDownloadQueue(CPreferences* in_prefs,CSharedFileList* in_sharedfilelist);
	~CDownloadQueue();
	void	Process();
	void	Init();
	void	AddSearchToDownload(CSearchFile* toadd,uint8 cat=0);
	void	AddSearchToDownload(CString link,uint8 cat=0);
	void	AddFileLinkToDownload(class CED2KFileLink* pLink,uint8 cat=0);
	bool	IsFileExisting(const uchar* fileid);
	bool	IsPartFile(void* totest);
	CPartFile*	GetFileByID(const uchar* filehash);
	CPartFile*	GetFileByIndex(int index);
	void    CheckAndAddSource(CPartFile* sender,CUpDownClient* source, bool saveSources = true); //modified for saving sources (by Mr.Fry)
	BOOL    CheckAndAddKnownSource(CPartFile* sender,CUpDownClient* source); // Lord KiRon
	bool	RemoveSource(CUpDownClient* toremove, bool updatewindow = true);
	void	DeleteAll();
	void	ResumeFiles();
	void	RemoveFile(CPartFile* toremove);
	uint32	GetDatarate()			{return averageDatarate;}
	void	SortByPriority();
	void	StopUDPRequests();
	CServer*	cur_udpserver;
	void		GetDownloadStats(uint16 sources[], uint64  data[]);
	void	AddPartFilesToShare();
	void	AddDownload(CPartFile* newfile);
	CUpDownClient* 	GetDownloadClientByIP(uint32 dwIP);
	CUpDownClient* GetDownloadClientByIP_UDP(uint32 dwIP, uint16 nUDPPort);
	void	UpdateDisplayedInfo(boolean force=false);
	void	AddDownDataOverheadSourceExchange(uint32 data, uint32 dummy=0)	{ m_nDownDataRateMSOverhead += data;
															  m_nDownDataOverheadSourceExchange += data;
															  m_nDownDataOverheadSourceExchangePackets++;}
	void	AddDownDataOverheadFileRequest(uint32 data, uint32 dummy=0)		{ m_nDownDataRateMSOverhead += data;
															  m_nDownDataOverheadFileRequest += data;
															  m_nDownDataOverheadFileRequestPackets++;}
	void	AddDownDataOverheadServer(uint32 data, uint32 dummy=0)			{ m_nDownDataRateMSOverhead += data;
															  m_nDownDataOverheadServer += data;
															  m_nDownDataOverheadServerPackets++;}
	void	AddDownDataOverheadOther(uint32 data, uint32 dummy=0)			{ m_nDownDataRateMSOverhead += data;
															  m_nDownDataOverheadOther += data;
															  m_nDownDataOverheadOtherPackets++;}
	uint32	GetDownDatarateOverhead()					{return m_nDownDatarateOverhead;}
	uint64	GetDownDataOverheadSourceExchange()			{return m_nDownDataOverheadSourceExchange;}
	uint64	GetDownDataOverheadFileRequest()			{return m_nDownDataOverheadFileRequest;}
	uint64	GetDownDataOverheadServer()					{return m_nDownDataOverheadServer;}
	uint64	GetDownDataOverheadOther()					{return m_nDownDataOverheadOther;}
	uint64	GetDownDataOverheadSourceExchangePackets()	{return m_nDownDataOverheadSourceExchangePackets;}
	uint64	GetDownDataOverheadFileRequestPackets()		{return m_nDownDataOverheadFileRequestPackets;}
	uint64	GetDownDataOverheadServerPackets()			{return m_nDownDataOverheadServerPackets;}
	uint64	GetDownDataOverheadOtherPackets()			{return m_nDownDataOverheadOtherPackets;}
	void	CompDownDatarateOverhead();

	int		GetFileCount() 								{return filelist.GetCount();} 	// added by FoRcHa
	int     GetTransferingFiles();														// added by FoRcHa	
	int     GetPausedFileCount();														// added by Purity
	void	StartPausedDownload(int cat) ; 												// Added by Tarod
	// v:- ekmn: A4AF management
	CPartFile* 	GetA4AFAutoFile()						{return m_A4AF_auto_file;}
	void 		SetA4AFAutoFile(CPartFile* file)	;
	// ^:- ekmn: A4AF management
	void AddClientHostnameToResolve(CTypedPtrList<CPtrList, CClientSource*>* pLink);
	void ResolveNextSourceHostname();
	void    SourceHostnameResolved(WPARAM wp, LPARAM lp);
	CString	GetUDPSearchStatus();

	void	ResetCatParts(int cat);
	void	SetCatPrio(int cat, uint8 newprio);
	void	SetCatStatus(int cat, int newstatus);
	void	MoveCat(uint8 from, uint8 to);
	void	SetAutoCat(CPartFile* newfile);

protected:
	bool	SendNextUDPPacket();
private:
	list<uint64> average_dr_list;	//Cax2 - memory optimization
	list<DWORD> average_tick_list;	//Cax2 - memory optimization
	CTypedPtrList<CPtrList, CPartFile*> filelist;
	CSharedFileList* sharedfilelist;
	CPreferences*	 app_prefs;
	uint16	filesrdy;
	uint32	datarate;
	
	CPartFile*	lastfile;
	uint32		lastudpsearchtime;
	uint32		lastudpstattime;
	uint8		udcounter;
	long		m_nUDPSearchFile;
	long		m_nUDPSearchServer;

	uint32		m_nDownDatarateOverhead;
	uint32		m_nDownDataRateMSOverhead;
	uint64		m_nDownDataOverheadSourceExchange;
	uint64		m_nDownDataOverheadSourceExchangePackets;
	uint64		m_nDownDataOverheadFileRequest;
	uint64		m_nDownDataOverheadFileRequestPackets;
	uint64		m_nDownDataOverheadServer;
	uint64		m_nDownDataOverheadServerPackets;
	uint64		m_nDownDataOverheadOther;
	uint64		m_nDownDataOverheadOtherPackets;

	CList<int,int>				 m_AvarageDDRO_list;

 	DWORD m_lastRefreshedDLDisplay;
    uint32      averageDatarate;
	//--- xrmb:keepPartFileStats ---
	uint32	m_lastPartFileStatsSave;
	void	SaveAllPartFileStats();
	//--- :xrmb ---
	// START enkeyDEV(kei-kun) -hostname sources in ED2K superLink-  
	CTypedPtrList<CPtrList, CClientSource*> hostnameResolveQueue;
	CHostnameSourceWnd* hsCallbackWnd;
	char    hostentBuffer[MAXGETHOSTSTRUCT];
	int8    sourceHostnameResolveRetry;
	bool    isResolving;
	// END enkeyDEV(kei-kun) -hostname sources in ED2K superLink-
	bool	m_bIsInitialized;
	// v1:- ekmn: A4AF management
	CPartFile* m_A4AF_auto_file;

};

// START - enkeyDEV(kei-kun) -hostname sources in ED2K superLink-
class CHostnameSourceWnd : public CWnd {
public:
CHostnameSourceWnd();
CDownloadQueue* m_pOwner;

protected:
DECLARE_MESSAGE_MAP()
afx_msg LRESULT OnSourceHostnameResolved(WPARAM wParam,LPARAM lParam);  
};
// END - enkeyDEV(kei-kun) -hostname sources in ED2K superLink-
