// EngineData.cpp
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Sockets/EmEngine.h"
#include "EngineData.h"
#include "resource.h"
#include "ServerList.h"
#include "emule.h"

#ifdef NEW_SOCKETS

CEngineData::CEngineData(CEmEngine& stEngine)
: m_stEngine(stEngine)
{
	m_nConnectionState = SERVER_DISCONNECTED;
	m_nClientID = 0;
	m_pCurrentServer = NULL;
	m_pNewServer = NULL;
	m_hServer = NULL;
	m_hNewServer = NULL;
	m_pServerList = theApp.serverlist;
}

CEngineData::~CEngineData()
{
}

bool CEngineData::IsConnecting()
{
	return (m_nConnectionState == SERVER_CONNECTING || 
			m_nConnectionState == SERVER_WAITFORLOGIN); 
}

bool CEngineData::IsConnected()
{ 
	return (m_nConnectionState == SERVER_CONNECTED);
}

bool CEngineData::IsLowID()
{ 
	return (m_nClientID < 16777216); 
}

// Runs in GUI thread
void CEngineData::ImportServerState(CTask_ServerState* pTask)
{
	if(pTask)
	{
		m_pCurrentServer = pTask->m_pCurrentServer;
		m_pNewServer = pTask->m_pNewServer;
		if(m_nConnectionState != pTask->m_nConnectionState)
		{
			m_nConnectionState = pTask->m_nConnectionState;
			switch(m_nConnectionState)
			{
			case SERVER_DISCONNECTED:
				break;
			case SERVER_FULL:
				break;
			case SERVER_CONNECTING:
				break;
			case SERVER_WAITFORLOGIN:
				if(m_pNewServer)
					AddLogLine(false, IDS_CONNECTEDTOREQ,
						m_pNewServer->GetListName(),
						m_pNewServer->GetFullIP(),
						m_pNewServer->GetPort());
				break;
			case SERVER_CONNECTED:
				if(m_pCurrentServer)
				{
					AddLogLine(true, IDS_CONNECTEDTO, m_pCurrentServer->GetListName());
					theApp.emuledlg->ShowConnectionState(true, m_pCurrentServer->GetListName());
				}
				break;
			}
		}
		if(m_nClientID != pTask->m_nClientID)
		{
			m_nClientID = pTask->m_nClientID;
			AddLogLine(false, IDS_NEWCLIENTID, m_nClientID);
		}
	}
}

void CEngineData::SetConnectionState(short nConnectionState)
{
	m_nConnectionState = nConnectionState;	

	switch(m_nConnectionState)
	{
	case SERVER_DISCONNECTED:
		m_pCurrentServer = NULL;
		m_hServer = NULL;
		break;
	case SERVER_FULL:
		break;
	case SERVER_CONNECTING:
		break;
	case SERVER_WAITFORLOGIN:
		break;
	case SERVER_CONNECTED:
		m_pCurrentServer = m_pNewServer;
		m_hNewServer = NULL;
		break;
	}

	ExportServerState();
}

void CEngineData::SetClientID(long cID)
{
	m_nClientID = cID; 

	ExportServerState();
}

void CEngineData::ConnectingTo(CEmClient_Server* pClient)
{
	m_hNewServer = pClient->m_hSocket;
	CServer* pServer = m_pServerList->GetServerByIP(pClient->m_nAddr, pClient->m_nPort);
	if(pServer)
	{
		m_pNewServer = pServer;
		ExportServerState();
	}
}

void CEngineData::ConnectedTo(CEmClient_Server* pClient)
{
	m_hServer = pClient->m_hSocket;
	CServer* pServer = m_pServerList->GetServerByIP(pClient->m_nAddr, pClient->m_nPort);
	if(pServer)
	{
		m_pCurrentServer = pServer;
		ExportServerState();
	}
}

void CEngineData::ExportServerState()
{
	CTask_ServerState* pTask = new CTask_ServerState(m_nConnectionState, m_nClientID, m_pCurrentServer, m_pNewServer);
	if(pTask)
		m_stEngine.m_stMain.Push(pTask);
	else
	{
		ASSERT(FALSE);
		AddDebugLogLine(false, "no memory");
	}
}

BYTE* CEngineData::GetUserHash()
{
	return theApp.glob_prefs->GetUserHash();
}

LONG CEngineData::GetPort()
{
	return theApp.glob_prefs->GetPort();
}

CString CEngineData::GetUserNick()
{
	return theApp.glob_prefs->GetUserNick();
}

#endif //NEW_SOCKETS