//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

// QueueListCtrl.cpp : implementation file
//

#include "stdafx.h"
#include "emule.h"
#include "QueueListCtrl.h"
#include "otherfunctions.h"
#include "opcodes.h"
#ifdef _USE_NEW_DETAILS_
#include ".\\Details\\ClientDetails.h"
#else
#include "ClientDetailDialog.h"
#endif
#include "memcpy_amd.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

// CQueueListCtrl

IMPLEMENT_DYNAMIC(CQueueListCtrl, CMuleListCtrl)
CQueueListCtrl::CQueueListCtrl(){
	memset(&asc_sort, 0, sizeof(asc_sort)); // i_a

	// Barry - Refresh the queue every 10 secs
	m_hTimer = ::SetTimer(NULL,				// Window to associate with
						NULL,				// ID of timer (ignored if window = null)
						30000,				// Time out duration
						(TIMERPROC)CQueueListCtrl::QueueUpdateTimer);	// Proc to run
	if (!m_hTimer)
		AddLogLine(false, IDS_ERR_TIMERCREATEFAILED);
}

void CQueueListCtrl::Init(){
	CImageList ilDummyImageList;
	ilDummyImageList.Create(1, 17,ILC_COLOR32 | ILC_MASK, 1, 1); 
	SetImageList(&ilDummyImageList, LVSIL_SMALL);
	ilDummyImageList.Detach();

	SetExtendedStyle(LVS_EX_FULLROWSELECT);
	InsertColumn(0,GetResString(IDS_QL_USERNAME),LVCFMT_LEFT,150,0);
	InsertColumn(1,GetResString(IDS_FILE),LVCFMT_LEFT,275,1);
	InsertColumn(2,GetResString(IDS_FILEPRIO),LVCFMT_LEFT,110,2);
	InsertColumn(3,GetResString(IDS_UP_PARTS),LVCFMT_LEFT,110,3);
	InsertColumn(4,GetResString(IDS_DL_PROGRESS),LVCFMT_LEFT,110,4);
	InsertColumn(5,GetResString(IDS_QL_RATING),LVCFMT_LEFT,60,5);
	InsertColumn(6,GetResString(IDS_SCORE),LVCFMT_LEFT,60,6);
	InsertColumn(7,GetResString(IDS_SFRATIO),LVCFMT_LEFT,60,7); // Tarod
	InsertColumn(8,GetResString(IDS_RFRATIO),LVCFMT_LEFT,60,8); // Tarod
	InsertColumn(9,GetResString(IDS_ASKED),LVCFMT_LEFT,60,9);
	InsertColumn(10,GetResString(IDS_LASTSEEN),LVCFMT_LEFT,110,10);
	InsertColumn(11,GetResString(IDS_ENTERQUEUE),LVCFMT_LEFT,110,11);
	InsertColumn(12,GetResString(IDS_BANNED),LVCFMT_LEFT,60,12);

	imagelist.Create(16,16,ILC_COLOR32|ILC_MASK,0,10);
	imagelist.SetBkColor(CLR_NONE);
	imagelist.Add(LoadImgLstIcon(MAKEINTRESOURCE(IDI_NORMAL),16,16));			// 0
	imagelist.Add(LoadImgLstIcon(MAKEINTRESOURCE(IDI_COMPROT),16,16));			// 1
	imagelist.Add(LoadImgLstIcon(MAKEINTRESOURCE(IDI_MLDONKEY),16,16));			// 2
    imagelist.Add(LoadImgLstIcon(MAKEINTRESOURCE(IDI_EDONKEYHYBRID),16,16));  	// 3
    imagelist.Add(LoadImgLstIcon(MAKEINTRESOURCE(IDI_CREDIT_ONLY),16,16));      // 4
    imagelist.Add(LoadImgLstIcon(MAKEINTRESOURCE(IDI_FRIEND_ONLY),16,16));      // 5
    imagelist.Add(LoadImgLstIcon(MAKEINTRESOURCE(IDI_BANNED_ONLY),16,16));      // 6
    imagelist.Add(LoadImgLstIcon(MAKEINTRESOURCE(IDI_UNKNOWN),16,16));  		// 7
    imagelist.Add(LoadImgLstIcon(MAKEINTRESOURCE(IDI_CDONKEY),16,16));  		// 8
    imagelist.Add(LoadImgLstIcon(MAKEINTRESOURCE(IDI_LMULE),16,16));  			// 9
    imagelist.Add(LoadImgLstIcon(MAKEINTRESOURCE(IDI_SHAREAZA),16,16));  		// 10
	imagelist.Add(LoadImgLstIcon(MAKEINTRESOURCE(IDI_SECUREHASH),16,16));  		// 11
	LoadImgLstIcon(NULL,0,0);	// delete temp-icon
	SetImageList(&imagelist,LVSIL_SMALL);

	Localize(); // i_a

	LoadSettings(CPreferences::tableQueue);
	if (theApp.glob_prefs->DoUseSort())
		SortInit(theApp.glob_prefs->GetQueueSortCol()); //sortdlg Cax2
	else
		SortInit(105);	//Cax2 - changed 16/01/03
}

CQueueListCtrl::~CQueueListCtrl()
{
	imagelist.DeleteImageList();	//eklmn: bugfix(01): resource cleanup due to ImageLists recreation
	// Barry - Kill the timer that was created
	try
	{
		if (m_hTimer)
			::KillTimer(NULL, m_hTimer);
	} catch (...) {}
}

void CQueueListCtrl::SortInit(int sortCode){	//Cax2 sortdlg
	m_sortTime=::GetTickCount();
	int m_CurrentSortItem=(sortCode<100)?sortCode:sortCode-100;
	SetSortArrow(m_CurrentSortItem,(sortCode<100));
	CUpDownClient* update = theApp.uploadqueue->GetNextClient(NULL);
	while( update ){
		RefreshClient( update);
		update = theApp.uploadqueue->GetNextClient(update);
	}
	SortItems(&SortProc, sortCode);
}

void CQueueListCtrl::Localize() 
{
	if(GetSafeHwnd())
	{
		CHeaderCtrl* pHeaderCtrl = GetHeaderCtrl();
		HDITEM hdi;
		hdi.mask = HDI_TEXT;

		if(pHeaderCtrl->GetItemCount() != 0) 
		{
			CString strRes;

			strRes = GetResString(IDS_QL_USERNAME);
			hdi.pszText = strRes.GetBuffer();
			pHeaderCtrl->SetItem(0, &hdi);
			strRes.ReleaseBuffer();

			strRes = GetResString(IDS_FILE);
			hdi.pszText = strRes.GetBuffer();
			pHeaderCtrl->SetItem(1, &hdi);
			strRes.ReleaseBuffer();

			strRes = GetResString(IDS_FILEPRIO);
			hdi.pszText = strRes.GetBuffer();
			pHeaderCtrl->SetItem(2, &hdi);
			strRes.ReleaseBuffer();

			strRes = GetResString(IDS_UP_PARTS);
			hdi.pszText = strRes.GetBuffer();
			pHeaderCtrl->SetItem(3, &hdi);
			strRes.ReleaseBuffer();

			strRes = GetResString(IDS_DL_PROGRESS);
			hdi.pszText = strRes.GetBuffer();
			pHeaderCtrl->SetItem(4, &hdi);
			strRes.ReleaseBuffer();

			strRes = GetResString(IDS_QL_RATING);
			hdi.pszText = strRes.GetBuffer();
			pHeaderCtrl->SetItem(5, &hdi);
			strRes.ReleaseBuffer();

			strRes = GetResString(IDS_SCORE);
			hdi.pszText = strRes.GetBuffer();
			pHeaderCtrl->SetItem(6, &hdi);
			strRes.ReleaseBuffer();

			strRes = GetResString(IDS_SFRATIO);
			hdi.pszText = strRes.GetBuffer();
			pHeaderCtrl->SetItem(7, &hdi);
			strRes.ReleaseBuffer();

			strRes = GetResString(IDS_RFRATIO);
			hdi.pszText = strRes.GetBuffer();
			pHeaderCtrl->SetItem(8, &hdi);
			strRes.ReleaseBuffer();

			strRes = GetResString(IDS_ASKED);
			hdi.pszText = strRes.GetBuffer();
			pHeaderCtrl->SetItem(9, &hdi);
			strRes.ReleaseBuffer();

			strRes = GetResString(IDS_LASTSEEN);
			hdi.pszText = strRes.GetBuffer();
			pHeaderCtrl->SetItem(10, &hdi);
			strRes.ReleaseBuffer();

			strRes = GetResString(IDS_ENTERQUEUE);
			hdi.pszText = strRes.GetBuffer();
			pHeaderCtrl->SetItem(11, &hdi);
			strRes.ReleaseBuffer();

			strRes = GetResString(IDS_BANNED);
			hdi.pszText = strRes.GetBuffer();
			pHeaderCtrl->SetItem(12, &hdi);
			strRes.ReleaseBuffer();
		}
	}
}

void CQueueListCtrl::AddClient(CUpDownClient* client, bool reset)
{
	if (!theApp.emuledlg->IsRunning()) return;
	if (!client) return;	//check NULL case
	
	uint32 itemnr = GetItemCount();
	itemnr = InsertItem(LVIF_TEXT|LVIF_PARAM,itemnr,LPSTR_TEXTCALLBACK,0,0,1,(LPARAM)client);

	if(reset == true) 
	{
		client->SetWaitStartTime();
		client->SetAskedCount(1);
	}
	//RefreshClient(client);
	//if (theApp.glob_prefs->DoUseSort() && m_sortTime+10000<::GetTickCount())
	//	SortInit(theApp.glob_prefs->GetQueueSortCol());
}

void CQueueListCtrl::RemoveClient(CUpDownClient* client)
{
	if (!theApp.emuledlg->IsRunning()) return;
	if (!client) return;	//check NULL case
	
	if(theApp.emuledlg->transferwnd.m_wndInfoList.GetType() == 1)
	{
		if(theApp.emuledlg->transferwnd.m_wndInfoList.GetClient() == client)
		{
			theApp.emuledlg->transferwnd.m_wndInfoList.SwitchToFile(NULL);
			theApp.emuledlg->transferwnd.UpdateInfoHeader();
		}
	}
	
	LVFINDINFO find;
	find.flags = LVFI_PARAM;
	find.lParam = (LPARAM)client;
	sint32 result = FindItem(&find);
	if(result != (-1))
    	DeleteItem(result);
}

void CQueueListCtrl::RefreshClient(CUpDownClient* client)
{
	//Don't refresh during app shutdown [TwoBottle Mod]
	if (!theApp.emuledlg->IsRunning()) return;
	if (!client) return;	//check NULL case

	LVFINDINFO find;
	find.flags = LVFI_PARAM;
	find.lParam = (LPARAM)client;
	sint16 result = FindItem(&find);
	if(result != -1)
		Update(result);
}

void CQueueListCtrl::DrawItem(LPDRAWITEMSTRUCT lpDrawItemStruct)
{
	if (!lpDrawItemStruct->itemData)
		return;
	CDC* odc = CDC::FromHandle(lpDrawItemStruct->hDC);
	BOOL bCtrlFocused = ((GetFocus() == this ) || (GetStyle() & LVS_SHOWSELALWAYS));
	odc->SetBkColor(GetBkColor());
	CUpDownClient* client = (CUpDownClient*)lpDrawItemStruct->itemData;
	CMemDC dc(odc,&CRect(lpDrawItemStruct->rcItem));
	CFont *pOldFont = dc.SelectObject(GetFont());
	COLORREF crOldTxtColor = dc->SetTextColor(::GetSysColor(COLOR_WINDOWTEXT));
    COLORREF crOldBkColor = dc->GetBkColor();
	RECT cur_rec;
	#ifndef AMD
	memcpy(&cur_rec,&lpDrawItemStruct->rcItem,sizeof(RECT));
	#else
	memcpy_amd(&cur_rec,&lpDrawItemStruct->rcItem,sizeof(RECT));
	#endif

	CString Sbuffer;

	CKnownFile* file = theApp.sharedfiles->GetFileByID(client->reqfileid);
	CHeaderCtrl *pHeaderCtrl = GetHeaderCtrl();
	int iCount = pHeaderCtrl->GetItemCount();
	cur_rec.right = cur_rec.left - 8;
	cur_rec.left += 4;

	for(int iCurrent = 0; iCurrent < iCount; iCurrent++)
	{
		int iColumn = pHeaderCtrl->OrderToIndex(iCurrent);
		if(!IsColumnHidden(iColumn))
		{
			cur_rec.right += GetColumnWidth(iColumn);
			switch(iColumn)
			{
				case 0:
					{
					    POINT point = {cur_rec.left, cur_rec.top+1};
		
						// Display client icon
						switch(client->GetClientSoft()) {
							case SO_CDONKEY:
								imagelist.Draw(dc, 8, point, ILD_NORMAL);
								break;
							case SO_LMULE:
								imagelist.Draw(dc, 9, point, ILD_NORMAL);
								break;
							case SO_SHAREAZA:
								imagelist.Draw(dc, 10, point, ILD_NORMAL);
								break;
							case SO_EDONKEYHYBRID:
								imagelist.Draw(dc, 3, point, ILD_NORMAL);
								break;
							case SO_MLDONKEY:
								imagelist.Draw(dc, 2, point, ILD_NORMAL);
								break;
							case SO_EMULE:
							case SO_OLDEMULE:
								if (client->credits->GetCurrentIdentState(client->GetIP()) != IS_IDENTIFIED)
									imagelist.Draw(dc, 11, point, ILD_NORMAL);
								else
									imagelist.Draw(dc, 1, point, ILD_NORMAL);
								break;
							case SO_EDONKEY:
								imagelist.Draw(dc, 0, point, ILD_NORMAL);
								break;
							default:
								imagelist.Draw(dc, 7, point, ILD_NORMAL);
								break;
						}

						// Display any Extra Overlay icons
	                    if (client->IsBanned())
	                        imagelist.Draw(dc, 6, point, ILD_TRANSPARENT);
	                    else if (client->IsFriend())
	                        imagelist.Draw(dc, 5, point, ILD_TRANSPARENT);
	                    else if (client->credits->GetScoreRatio(client->GetIP()) > 1)
	                        imagelist.Draw(dc, 4, point, ILD_TRANSPARENT);
	
						

	                    Sbuffer = client->GetUserName();
						cur_rec.left +=20;
	                    // EC - Use Dark Green for SUI Clients
						//if (client->credits->GetCurrentIdentState(client->GetIP()) == IS_IDENTIFIED)
						//	crOldTxtColor = dc->SetTextColor((COLORREF)RGB(0,130,0));								

						dc->DrawText(Sbuffer,Sbuffer.GetLength(),&cur_rec,DT_LEFT|DT_SINGLELINE|DT_VCENTER|DT_NOPREFIX|DT_END_ELLIPSIS);

						//if (client->credits->GetCurrentIdentState(client->GetIP()) == IS_IDENTIFIED)
						//	dc->SetTextColor(crOldTxtColor);				
	
						cur_rec.left -=20;
						break;
	    			}
				case 1:
							if(file)
								Sbuffer = file->GetFileName();
							else
								Sbuffer = "?";
							break;
				case 2:
							if (file){
								switch(file->GetPriority()){ 
									case PR_VERYHIGH:
										Sbuffer = GetResString(IDS_PRIORELEASE);		//Cax2 in sync with sharedfiles now
										break;
									case PR_HIGH: 
										Sbuffer = GetResString(IDS_PRIOHIGH);
										break; 
									case PR_LOW: 
										Sbuffer = GetResString(IDS_PRIOLOW);
										break; 
									case PR_VERYLOW:
										Sbuffer = GetResString(IDS_PRIOVERYLOW);
										break;
									default: 
										Sbuffer = GetResString(IDS_PRIONORMAL);
									break; 
								}
							}
							else
								Sbuffer = "?";
							break;
				case 3:
							if( client->GetUpPartCount() ){
							    Sbuffer.Format(_T("%d/%d"), client->GetAvailUpPartCount(), client->GetUpPartCount());
		                    } else {
							    Sbuffer.Format(_T(""));
		                    }
		                    break;
				case 4:
					        if( client->GetUpPartCount() && theApp.glob_prefs->IsUploadPartsEnabled())		//Cax2 re-enabled toggle for progress bars
					        {
						        cur_rec.bottom--;
						        cur_rec.top++;
						        client->DrawUpStatusBar(dc,&cur_rec,false,theApp.glob_prefs->UseFlatBar());
						        cur_rec.bottom++;
						        cur_rec.top--;
					        }
					        break;
				case 5:
							Sbuffer.Format(_T("%.1f"),(float)client->GetScore(false,client->IsDownloading(),true));
							break;
				case 6:
							Sbuffer.Format(_T("%i"),client->GetScore(false));
							break;
				case 7:
							Sbuffer.Format(_T("%i"),(int)client->GetSmallFilePushRatio());		//Cax2 SF bugfix
							break;
				case 8:
							Sbuffer.Format(_T("%.2f"),client->GetRareFilePushRatio());			//Cax2 RF bugfix
							break;
				case 9:
							Sbuffer.Format(_T("%i"),client->GetAskedCount());
							break;
				case 10:
							Sbuffer = CastSecondsToHM((GetTickCount() - client->GetLastUpRequest())/1000);
							break;
				case 11:
							Sbuffer = CastSecondsToHM((GetTickCount() - client->GetWaitStartTime())/1000);
							break;
				case 12:
							if(client->IsBanned())
								Sbuffer = GetResString(IDS_YES);
							else
								Sbuffer = GetResString(IDS_NO);
							break;
			}
			if( iColumn != 4 && iColumn != 0)
				dc->DrawText(Sbuffer,Sbuffer.GetLength(),&cur_rec,DT_LEFT|DT_SINGLELINE|DT_VCENTER|DT_NOPREFIX|DT_END_ELLIPSIS);
			cur_rec.left += GetColumnWidth(iColumn);
		}
	}
	//draw rectangle around selected item(s)
	if ((lpDrawItemStruct->itemAction | ODA_SELECT) && (lpDrawItemStruct->itemState & ODS_SELECTED))
	{
		RECT outline_rec;
		#ifndef AMD
		memcpy(&outline_rec,&lpDrawItemStruct->rcItem,sizeof(RECT));
		#else
		memcpy_amd(&outline_rec,&lpDrawItemStruct->rcItem,sizeof(RECT));
		#endif

		outline_rec.top--;
		outline_rec.bottom++;
		dc->FrameRect(&outline_rec, &CBrush(GetBkColor()));
		outline_rec.top++;
		outline_rec.bottom--;
		outline_rec.left++;
		outline_rec.right--;

		if(bCtrlFocused)
			dc->FrameRect(&outline_rec, &CBrush(m_crFocusLine));
		else
			dc->FrameRect(&outline_rec, &CBrush(m_crNoFocusLine));
	}
	dc.SelectObject(pOldFont);
}

BEGIN_MESSAGE_MAP(CQueueListCtrl, CMuleListCtrl)
	ON_WM_CONTEXTMENU()
	ON_NOTIFY_REFLECT(LVN_COLUMNCLICK, OnColumnClick)
	ON_NOTIFY_REFLECT(NM_CLICK, OnNMClick)
	ON_NOTIFY_REFLECT(NM_DBLCLK, OnNMDblclkQueuelist)
	ON_NOTIFY_REFLECT(LVN_GETDISPINFO, OnGetDispInfo)
	ON_WM_KEYUP()
	ON_WM_ERASEBKGND()
END_MESSAGE_MAP()

// CQueueListCtrl message handlers
void CQueueListCtrl::OnContextMenu(CWnd* pWnd, CPoint point)
{	
//	POINT point;
//	::GetCursorPos(&point);	

	UINT flags=MF_STRING|MF_GRAYED;
	const CUpDownClient* client = NULL;
	if (GetSelectionMark() != -1)
	{
		if(GetSelectedCount()>0)
			flags=MF_STRING;
		if(client = (CUpDownClient*)GetItemData(GetSelectionMark()))
		{
			theApp.emuledlg->transferwnd.m_wndInfoList.SwitchToUser((CUpDownClient*)client);
			theApp.emuledlg->transferwnd.UpdateInfoHeader();
		}
	}

	m_ClientMenu.CreatePopupMenu();
	m_ClientMenu.AddMenuTitle(GetResString(IDS_CLIENTS));
	m_ClientMenu.AppendMenu(flags,MP_DETAIL, GetResString(IDS_SHOWDETAILS));
	if(client && client->IsFriend())
		m_ClientMenu.AppendMenu(flags,MP_REMOVEFRIEND, GetResString(IDS_REMOVEFRIEND));
	else
		m_ClientMenu.AppendMenu(flags,MP_ADDFRIEND, GetResString(IDS_ADDFRIEND));
	m_ClientMenu.AppendMenu(flags,MP_MESSAGE, GetResString(IDS_SEND_MSG));
	m_ClientMenu.AppendMenu(flags,MP_SHOWLIST, GetResString(IDS_VIEWFILES));
	m_ClientMenu.AppendMenu(flags,MP_UNBAN, GetResString(IDS_UNBAN));
	if(client)
		m_ClientMenu.EnableMenuItem(MP_UNBAN,(client->IsBanned() ? MF_ENABLED:MF_GRAYED)); //fix by Maverick
	m_ClientMenu.AppendMenu(MF_SEPARATOR);
	m_ClientMenu.AppendMenu(MF_STRING,MP_SWITCHCTRL, GetResString(IDS_VIEWUPLOADS));
	SetMenu(&m_ClientMenu);
	m_ClientMenu.TrackPopupMenu(TPM_LEFTALIGN |TPM_RIGHTBUTTON, point.x, point.y, this);
//	*pResult = 0;
	m_ClientMenu.DestroyMenu();
}

BOOL CQueueListCtrl::OnCommand(WPARAM wParam,LPARAM lParam ){
	if (GetSelectionMark() != (-1)){
		CUpDownClient* client = (CUpDownClient*)GetItemData(GetSelectionMark());
		switch (wParam){
			case MP_SHOWLIST:
				client->RequestSharedFileList();
				break;
			case MP_MESSAGE:
				theApp.emuledlg->chatwnd.StartSession(client);
				break;
			case MP_ADDFRIEND:
				theApp.friendlist->AddFriend(client);
				break;
			case MP_REMOVEFRIEND:
				theApp.friendlist->RemoveFriend(client);
				break;
			case MP_UNBAN:
				if( client->IsBanned() )
					client->UnBan();
				break;
			case MP_DETAIL:
			{
			#if _USE_NEW_DETAILS_
				CClientDetails dialog("Client Details", client, this, 0);
			#else
				CClientDetailDialog dialog(client);
			#endif
				dialog.DoModal();
				break;
			}
		}
	}
	switch(wParam)
	{
		case MP_SWITCHCTRL:
		{
			theApp.emuledlg->transferwnd.PostMessage(WM_COMMAND, USRMSG_SWITCHUPLOADLIST, (long)m_hWnd);
			break;
		}
	}
	return true;
} 

void CQueueListCtrl::OnColumnClick( NMHDR* pNMHDR, LRESULT* pResult){

	//theApp.uploadqueue->ResetClientPos();
	CUpDownClient* update = theApp.uploadqueue->GetNextClient(NULL);

	while( update ){
		RefreshClient( update);
		update = theApp.uploadqueue->GetNextClient(update);
	}

	NM_LISTVIEW* pNMListView = (NM_LISTVIEW*)pNMHDR;
	// if it's a second click on the same column then reverse the sort order,
	// otherwise sort the new column in ascending order.
	asc_sort[pNMListView->iSubItem] = !asc_sort[pNMListView->iSubItem];
	SetSortArrow(pNMListView->iSubItem, asc_sort[pNMListView->iSubItem]);
	SortItems(&SortProc,pNMListView->iSubItem+ ((asc_sort[pNMListView->iSubItem])? 0:100));
	*pResult = 0;
}

int CQueueListCtrl::SortProc(LPARAM lParam1, LPARAM lParam2, LPARAM lParamSort){
	CUpDownClient* item1 = (CUpDownClient*)lParam1;
	CUpDownClient* item2 = (CUpDownClient*)lParam2;
	CKnownFile* file1 = theApp.sharedfiles->GetFileByID(item1->reqfileid);
	CKnownFile* file2 = theApp.sharedfiles->GetFileByID(item2->reqfileid);
		switch(lParamSort){
		case 0: 
			return CString(item1->GetUserName()).CompareNoCase(item2->GetUserName());
		case 100:
			return CString(item2->GetUserName()).CompareNoCase(item1->GetUserName());
		case 1: 
			if( (file1 != NULL) && (file2 != NULL))
				return CString(file1->GetFileName()).CompareNoCase(file2->GetFileName());
			else if( file1 == NULL )
				return 1;
			else
				return 0;
		case 101:
			if( (file1 != NULL) && (file2 != NULL))
				return CString(file2->GetFileName()).CompareNoCase(file1->GetFileName());
			else if( file1 == NULL )
				return 1;
			else
				return 0;
		case 2: 
			if(file1 != NULL && file2 != NULL)		//Cax2 - right priority sortig
			{
				if(file1->GetPriority() ==PR_VERYLOW) 
					return -1;
				if(file2->GetPriority() ==PR_VERYLOW)
					return 1;
				return file1->GetPriority() - file2->GetPriority();
			}
			else if( file1 == NULL )
				return 1;
			else
				return 0;
		case 102:
			if( file1 != NULL && file2 != NULL)		//Cax2 - right priority sortig
			{
				if(file1->GetPriority() ==PR_VERYLOW) 
					return 1;
				if(file2->GetPriority() ==PR_VERYLOW)
					return -1;
				return file2->GetPriority() - file1->GetPriority();
			}
			else if( file1 == NULL )
				return 1;
			else
				return 0;

		case 3:
		case 103:
		case 4:
		case 104:
            {  // obaldin: sort by upload progress
                int res;
                uint32 partcnt1 = item1->GetUpPartCount();
                uint32 partcnt2 = item2->GetUpPartCount();
                if( (partcnt1==0) && (partcnt2==0))
                    res=0;
                else if(partcnt1==0)
                    res = -1;
                else if(partcnt2==0)
                    res = 1;
                else {
                    float progr1 = float(item1->GetAvailUpPartCount())/float(partcnt1);
                    float progr2 = float(item2->GetAvailUpPartCount())/float(partcnt2);
                    if(progr1<progr2)
                        res = -1;
                    else if(progr1>progr2)
                        res = 1;
                    else 
                        res = 0;
                }

                if(lParamSort<100)
                    return res;
                else 
                    return -res;
            }

        case 5: 
			return (float)item1->GetScore(false,false,true) - (float)item2->GetScore(false,false,true);
		case 105: 
			return (float)item2->GetScore(false,false,true) - (float)item1->GetScore(false,false,true);

		case 6: 
			return item1->GetScore(false) - item2->GetScore(false);
		case 106: 
			return item2->GetScore(false) - item1->GetScore(false);

		// Added by Tarod
		case 7: //small file  asc
			return (int)(item1->GetSmallFilePushRatio()*100 - item2->GetSmallFilePushRatio()*100);
		case 107: //small file desc
			return (int)(item2->GetSmallFilePushRatio()*100 - item1->GetSmallFilePushRatio()*100);
		case 8: //rare file asc
			return (int)(item1->GetRareFilePushRatio()*100 - item2->GetRareFilePushRatio()*100);
		case 108: //rare file desc
			return (int)(item2->GetRareFilePushRatio()*100 - item1->GetRareFilePushRatio()*100);
		// End by Tarod

		case 9: 
		return item1->GetAskedCount() - item2->GetAskedCount();
		case 109: 
			return item2->GetAskedCount() - item1->GetAskedCount();
		case 110: 
			return item1->GetLastUpRequest() - item2->GetLastUpRequest();
		case 10: 
			return item2->GetLastUpRequest() - item1->GetLastUpRequest();
		case 111: 
			return item1->GetWaitStartTime() - item2->GetWaitStartTime();
		case 11: 
			return item2->GetWaitStartTime() - item1->GetWaitStartTime();
		case 12: 
			return item1->IsBanned() - item2->IsBanned();
		case 112: 
			return item2->IsBanned() - item1->IsBanned();

		default:
			return 0;
	}
}

// Barry - Refresh the queue every 10 secs
void CALLBACK CQueueListCtrl::QueueUpdateTimer(HWND hwnd, UINT uiMsg, UINT idEvent, DWORD dwTime)
{
	// Don't do anything if the app is shutting down - can cause unhandled exceptions
	if ((theApp.uploadqueue == NULL) || !theApp.emuledlg->IsRunning() || !theApp.glob_prefs->GetUpdateQueueList())
		return;
	
	CUpDownClient* update = theApp.uploadqueue->GetNextClient(NULL);
	while( update )
	{
		theApp.emuledlg->transferwnd.m_wndQueueList.RefreshClient(update);
		update = theApp.uploadqueue->GetNextClient(update);
	}
}

void CQueueListCtrl::OnNMClick(NMHDR *pNMHDR, LRESULT *pResult)
{
	RefreshInfo();
	*pResult = 0;
}

void CQueueListCtrl::OnNMDblclkQueuelist(NMHDR *pNMHDR, LRESULT *pResult)
{
	if (GetSelectionMark() != (-1))
	{
		CUpDownClient* client = (CUpDownClient*)GetItemData(GetSelectionMark());
		if (theApp.glob_prefs->GetDetailsOnClick())
		{
		#if _USE_NEW_DETAILS_
			CClientDetails dialog("Client Details", client, this, 0);
		#else
			CClientDetailDialog dialog(client);
		#endif
			dialog.DoModal();
		} 
		else 
		{
			theApp.emuledlg->chatwnd.StartSession(client);
		}
	}
	*pResult = 0;
}

void CQueueListCtrl::OnKeyUp(UINT nChar, UINT nRepCnt, UINT nFlags)
{
	RefreshInfo();
	CMuleListCtrl::OnKeyUp(nChar, nRepCnt, nFlags);
}

void CQueueListCtrl::RefreshInfo(void)
{
	POSITION pos = GetFirstSelectedItemPosition();
	if(pos == NULL)
	{			
		theApp.emuledlg->transferwnd.m_wndInfoList.SwitchToFile(NULL);
		theApp.emuledlg->transferwnd.UpdateInfoHeader();
	}
	else
	{
		const CUpDownClient* client = (CUpDownClient*)GetItemData(GetSelectionMark());
		theApp.emuledlg->transferwnd.m_wndInfoList.SwitchToUser((CUpDownClient*) client);
		theApp.emuledlg->transferwnd.UpdateInfoHeader();
	}	
}

BOOL CQueueListCtrl::OnEraseBkgnd(CDC* pDC)
{
	return CListCtrl::OnEraseBkgnd(pDC);
}

BOOL CQueueListCtrl::OnChildNotify(UINT message, WPARAM wParam, LPARAM lParam, LRESULT* pLResult)
{
	if(message != WM_DRAWITEM) {
		//catch the prepaint and copy struct
		if(message == WM_NOTIFY && ((NMHDR*)lParam)->code == NM_CUSTOMDRAW &&
		  ((LPNMLVCUSTOMDRAW)lParam)->nmcd.dwDrawStage == CDDS_ITEMPREPAINT) {

			m_bCustomDraw = CListCtrl::OnChildNotify(message, wParam, lParam, pLResult);
			if(m_bCustomDraw)
				#ifndef AMD
				memcpy(&m_lvcd, (void*)lParam, sizeof(NMLVCUSTOMDRAW));
				#else
				memcpy_amd(&m_lvcd, (void*)lParam, sizeof(NMLVCUSTOMDRAW));
				#endif

			return m_bCustomDraw;
		}

		return CListCtrl::OnChildNotify(message, wParam, lParam, pLResult);
	}
	ASSERT(pLResult == NULL); // no return value expected
	UNUSED(pLResult);         // unused in release builds

	DrawItem((LPDRAWITEMSTRUCT)lParam);
	return TRUE;
}

void CQueueListCtrl::OnGetDispInfo(NMHDR *pNMHDR, LRESULT *pResult)
{
	NMLVDISPINFO *pDispInfo = reinterpret_cast<NMLVDISPINFO*>(pNMHDR);

	if (theApp.emuledlg->IsRunning()){
		// Although we have an owner drawn listview control we store the text for the primary item in the listview, to be
		// capable of quick searching those items via the keyboard. Because our listview items may change their contents,
		// we do this via a text callback function. The listview control will send us the LVN_DISPINFO notification if
		// it needs to know the contents of the primary item.
		//
		// But, the listview control sends this notification all the time, even if we do not search for an item. At least
		// this notification is only sent for the visible items and not for all items in the list. Though, because this
		// function is invoked *very* often, no *NOT* put any time consuming code here in.

		if (pDispInfo->item.mask & LVIF_TEXT){
			CUpDownClient* pClient = reinterpret_cast<CUpDownClient*>(pDispInfo->item.lParam);
			if (pClient != NULL){
				switch (pDispInfo->item.iSubItem){
					case 0:
						if (pClient->GetUserName() != "" && pDispInfo->item.cchTextMax > 0){
							_tcsncpy(pDispInfo->item.pszText, pClient->GetUserName(), pDispInfo->item.cchTextMax);
							pDispInfo->item.pszText[pDispInfo->item.cchTextMax-1] = _T('\0');
						}
						break;
					default:
						// shouldn't happen
						pDispInfo->item.pszText[0] = _T('\0');
						break;
				}
			}
		}
	}
	*pResult = 0;
}
