//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include "StdAfx.h"
#include "searchlist.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

CSearchFile::CSearchFile(CFile& in_data,uint32 nSearchID,CUpDownClient* Sender,LPCTSTR pszDirectory){	// netwolf 20.05.03 enhanced source finding (bluecow)
	EMULE_TRY
	m_nSearchID = nSearchID;
	in_data.Read(&filehash,16);
	in_data.Read(&clientip,4);
	in_data.Read(&clientport,2);
	uint32 tagcount;
	in_data.Read(&tagcount,4);
	for (int i = 0;i != tagcount; i++){
		CTag* toadd = new CTag(in_data);
		taglist.Add(toadd);
	}

	filename = GetStrTagValue(FT_FILENAME);
	if (filename.IsEmpty())
		filename = _T("?");

	filesize = GetIntTagValue(FT_FILESIZE);
	EMULE_CATCH

	if (Sender){		//netwolf 20.05.03 enhanced source finding (bluecow)
		// To be capable of adding the client from which we got that search result
		// as a source, we have to explicitly store that client's data in the search result.
		// NOTE: Do *NOT* store the 'Sender' ptr here! At the time when using the 'toadd'
		// struct, the 'Sender' may have been deleted (just if the user has waited too long before
		// using one of the client's search results) -> Crash!
		m_eType = Client;

		// Explicitly overwrite the already available client's IP, Port with the already validated UserID and Port.
		// So, we have proper (low)UserID+Server pair.
		*((uint32*)clientip) = Sender->GetUserID();
		clientport = Sender->GetUserPort();

		m_nClientServerIP = Sender->GetServerIP();
		m_nClientServerPort = Sender->GetServerPort();
	}
	else{
		m_eType = Server;
		m_nClientServerIP = 0;
		m_nClientServerPort = 0;
	}									// netwolf END
	m_strDirectory= pszDirectory ? pszDirectory : _T("");
}

CSearchFile::~CSearchFile(){
	EMULE_TRY
	for (int i = 0; i < taglist.GetSize();i++)
		safe_delete(taglist[i]);
	taglist.RemoveAll();
	taglist.SetSize(0);
	EMULE_CATCH
}

uint32 CSearchFile::GetIntTagValue(uint8 tagname){
	EMULE_TRY
	for (int i = 0; i < taglist.GetSize(); i++){
		if (taglist[i]->GetSpecialTag() == tagname)
			return taglist[i]->GetIntValue();
	}
	EMULE_CATCH
	return 0;
}

CString CSearchFile::GetStrTagValue(uint8 tagname){
	EMULE_TRY
	for (int i = 0; i < taglist.GetSize(); i++){
		if (taglist[i]->GetSpecialTag() == tagname)
			return taglist[i]->GetStringValue();			
	}
	EMULE_CATCH
	return _T("");
}

uint32 CSearchFile::AddSources(uint32 count){
	EMULE_TRY
	for (int i = 0; i < taglist.GetSize(); i++){
		if (taglist[i]->GetSpecialTag() == FT_SOURCES){
			taglist[i]->SetIntValue(taglist[i]->GetIntValue() + count);
			return taglist[i]->GetIntValue();
		}
	}
	EMULE_CATCH
	return 0;
}

uint32 CSearchFile::GetSourceCount(){
	return GetIntTagValue(FT_SOURCES);
}

CSearchList::CSearchList(){
	outputwnd = 0;
	m_MobilMuleSearch = false;
}

CSearchList::~CSearchList(){
	Clear();
}

void CSearchList::Clear(){
	EMULE_TRY
	for(POSITION pos = list.GetHeadPosition(); pos != NULL;) {
		delete list.GetNext(pos);
	}
	list.RemoveAll();
	EMULE_CATCH
}

void CSearchList::RemoveResults( uint32 nSearchID){
	EMULE_TRY
	// this will not delete the item from the window, make sure your code does it if you call this
	ASSERT( outputwnd );
	POSITION pos1, pos2;
	for (pos1 = list.GetHeadPosition();( pos2 = pos1 ) != NULL;){
		list.GetNext(pos1);
		CSearchFile* cur_file = list.GetAt(pos2);
		if( cur_file->GetSearchID() == nSearchID ){
			list.RemoveAt(pos2);
			delete cur_file;
		}
	}
	EMULE_CATCH
}

void CSearchList::ShowResults( uint32 nSearchID){
	EMULE_TRY
	ASSERT( outputwnd );
	outputwnd->SetRedraw(false);
	for (POSITION pos = list.GetHeadPosition(); pos !=0;list.GetNext(pos)){
		if( ((CSearchFile*)list.GetAt(pos))->GetSearchID() == nSearchID ){
			outputwnd->AddResult(list.GetAt(pos));
		}
	}
	outputwnd->SetRedraw(true);
	EMULE_CATCH
}

void CSearchList::RemoveResults( CSearchFile* todel ){
	EMULE_TRY

	POSITION remove_pos = list.Find(todel);
	if (remove_pos)
	{
		theApp.emuledlg->searchwnd.searchlistctrl.RemoveResult( todel );
		list.RemoveAt(remove_pos);
		delete todel;
	}

	EMULE_CATCH
}

void CSearchList::NewSearch(CSearchListCtrl* in_wnd, CString resTypes, uint16 nSearchID, bool MobilMuleSearch){
	EMULE_TRY
	if(in_wnd)
		outputwnd = in_wnd;

	resultType=resTypes;
	m_nCurrentSearch = nSearchID;
	myHashList="";
    m_MobilMuleSearch = MobilMuleSearch;
	
	foundFilesCount.SetAt(nSearchID,0);
	EMULE_CATCH
}

uint16 CSearchList::ProcessSearchanswer(char* in_packet, uint32 size, CUpDownClient* Sender, LPCTSTR pszDirectory, bool bFirstDir)
{
	EMULE_TRY
	CSafeMemFile packet((BYTE*)in_packet,size,0);
	uint32 results;
	packet.Read(&results,4);

	uint32 mySearchID=( (Sender != NULL)? (uint32)Sender : m_nCurrentSearch);

	if (Sender)
	{
		if (bFirstDir)
		{
			theApp.emuledlg->searchwnd.DeleteSearch(mySearchID); // DonGato
			theApp.emuledlg->searchwnd.CreateNewTab(CString(Sender->GetUserName()),mySearchID);
			foundFilesCount.SetAt(mySearchID,0);
		}
	}
	else
	{
		foundFilesCount.SetAt(mySearchID,0);
	}

	for (int i = 0; i != results; i++){
		CSearchFile* toadd = new CSearchFile(packet, mySearchID, Sender, pszDirectory);	// netwolf 20.05.03 enhanced source finding (bluecow)
		AddToList(toadd, (Sender != NULL) );
	}
#ifdef OLD_SOCKETS_ENABLED
#ifdef MOBILE_MULE
	if (m_MobilMuleSearch)
		theApp.mmserver->SearchFinished(false);
#endif //MOBILE_MULE
#endif //OLD_SOCKETS_ENABLED
	m_MobilMuleSearch = false;
	packet.Close();	
	EMULE_CATCH
	return GetResultCount();
}

uint16 CSearchList::ProcessUDPSearchanswer(char* in_packet, uint32 size){
	EMULE_TRY
	CSafeMemFile packet((BYTE*)in_packet,size,0);
	CSearchFile* toadd = new CSearchFile(packet, m_nCurrentSearch);
	AddToList(toadd);
	packet.Close();	
	EMULE_CATCH
	return GetResultCount();
}

bool CSearchList::AddToList(CSearchFile* toadd, bool bClientResponse){
	EMULE_TRY
	CString thisType=GetFiletypeByName(toadd->GetFileName());	//Cax2 bugfix: now searching for programs shows archives, TODO searching for users shows anything...
	if (
		(!bClientResponse && !(resultType == GetResString(IDS_SEARCH_ANY)||resultType == GetResString(IDS_SEARCH_USER)||resultType == GetResString(IDS_SEARCH_DOC)|| thisType==resultType||(resultType==GetResString(IDS_SEARCH_PRG)&&thisType==GetResString(IDS_SEARCH_ARC)))) ||
		(theApp.emuledlg->searchwnd.IsDocumentSearch(toadd->GetSearchID()) && thisType != GetResString(IDS_SEARCH_DOC)) ||
		(!(toadd->GetIntTagValue(FT_FILESIZE)) || toadd->GetFileName().IsEmpty())
	   )
	{
		delete toadd;
		return false;
	}
	CString sNotSearch = theApp.emuledlg->searchwnd.GetNotSearch(toadd->GetSearchID());
	if(!sNotSearch.IsEmpty())
	{
		CString sFileName = CString(toadd->GetFileName());
		sFileName.MakeLower();
		if(sFileName.Find(sNotSearch) != -1)
		{
			delete toadd;
			return false;
		}
	}
	for (POSITION pos = list.GetHeadPosition(); pos != NULL; list.GetNext(pos))
	{
		CSearchFile* cur_file = list.GetAt(pos);
		if ( (!md4cmp(toadd->GetFileHash(),cur_file->GetFileHash())) && cur_file->GetSearchID() ==  toadd->GetSearchID()){
			cur_file->AddSources(toadd->GetIntTagValue(FT_SOURCES));
			if (outputwnd && !m_MobilMuleSearch)
				outputwnd->UpdateSources(cur_file);
			delete toadd;
			return true;
		}
	}
	
	if (list.AddTail(toadd)) {	
		uint16 tempValue;
		foundFilesCount.Lookup(toadd->GetSearchID(),tempValue);
		foundFilesCount.SetAt(toadd->GetSearchID(),tempValue+1);
	}
	if (outputwnd && !m_MobilMuleSearch)
		outputwnd->AddResult(toadd);
	return true;
	EMULE_CATCH
	return false;
}

uint16 CSearchList::GetResultCount(uint32 nSearchID) {
	EMULE_TRY
	uint16 hits = 0;
	for (POSITION pos = list.GetHeadPosition(); pos != NULL; list.GetNext(pos)){
		if( list.GetAt(pos)->GetSearchID() == nSearchID )
			hits += list.GetAt(pos)->GetIntTagValue(FT_SOURCES);
	}
	return hits;
	EMULE_CATCH
	return 0;
}


uint16 CSearchList::GetResultCount(){
	return GetResultCount(m_nCurrentSearch);
}

CString CSearchList::GetWebList(CString linePattern,int sortby,bool asc,bool column1,bool column2,bool column3,bool column4) const{
	EMULE_TRY

	CString buffer=_T("");
	CString temp;
	CArray<CSearchFile*, CSearchFile*> sortarray;
	int swap;
	bool inserted;

	// insertsort
	CSearchFile* sf1;
	CSearchFile* sf2;
	for (POSITION pos = list.GetHeadPosition(); pos !=0;) {
		
		sf1= (CSearchFile*)list.GetAt(pos);
		uint32 filesize = sf1->GetIntTagValue(FT_FILESIZE); 
		uint32 nSources = sf1->GetIntTagValue(FT_SOURCES);
		if(!filesize || /* !nSources || */ sf1->GetFileName().IsEmpty())
			continue;
		
		inserted=false;
		sf1 = list.GetNext(pos);
		
		for (uint16 i1=0;i1<sortarray.GetCount();++i1) {
			sf2 = sortarray.GetAt(i1);
			
			switch (sortby) {
				case 0: swap=sf1->GetFileName().CompareNoCase(sf2->GetFileName()); break;
				case 1: swap=sf1->GetFileSize()-sf2->GetFileSize();break;
				case 2: swap=CString(sf1->GetFileHash()).CompareNoCase(CString(sf2->GetFileHash())); break;
				case 3: swap=sf1->GetSourceCount()-sf2->GetSourceCount(); break;
			}
			if (!asc) swap=0-swap;
			if (swap<0) {inserted=true; sortarray.InsertAt(i1,sf1);break;}
		}
		if (!inserted) sortarray.Add(sf1);
	}
	
	for (uint16 i=0;i<sortarray.GetCount();++i) {
		CSearchFile* sf1 = sortarray.GetAt(i);

		// colorize
		CString coloraddonB=_T("");
		CString coloraddonE=_T("");
		CKnownFile* sameFile = theApp.sharedfiles->GetFileByID(sf1->GetFileHash());
		if (!sameFile)
			sameFile = theApp.downloadqueue->GetFileByID(sf1->GetFileHash());

		uint32 red = 255;
		uint32 green = 255;
		uint32 blue = 255;
		CString OverlayImage;
		CString SourcesImage;
		int nSources;
		if (sameFile)
		{
			if (sameFile->IsPartFile())
			{
				OverlayImage = "jumpstart";
				blue = 0;
				red = 0;
			}
			else
			{
				OverlayImage = "release";
				blue = 0;
			}
		}
		else
		{
			OverlayImage = "none";
		}
		nSources = sf1->GetSourceCount();
		if (nSources < 5)
			SourcesImage = "0";
		else if (nSources > 4 && nSources < 10)
			SourcesImage = "5";
		else if (nSources > 9 && nSources < 25)
			SourcesImage = "10";
		else if (nSources > 24 && nSources < 50)
			SourcesImage = "25";
		else
			SourcesImage = "50";

		CString strColor = "#";
		CString strTemp;
		strTemp.Format(_T("%x"),red);
		if (strTemp.GetLength() == 1) strColor.AppendFormat("0" + strTemp);
		else strColor.AppendFormat(strTemp);
		strTemp.Format(_T("%x"),green);
		if (strTemp.GetLength() == 1) strColor.AppendFormat("0" + strTemp); 
		else strColor.AppendFormat(strTemp);
		strTemp.Format(_T("%x"),blue);
		if (strTemp.GetLength() == 1) strColor.AppendFormat("0" + strTemp); 
		else strColor.AppendFormat(strTemp);
		coloraddonB="<font color=" + strColor + ">";
		coloraddonE="</font>";

		CString strHash=EncodeBase16(sf1->GetFileHash(),16);
		CString strSize;
		strSize.Format(_T("%d"),sf1->GetFileSize());
		CString strSources;
		strSources.Format(_T("%d"),sf1->GetSourceCount());
		CString strFilename;
		strFilename = sf1->GetFileName();
		strFilename.Replace(_T("'"),_T("\\'"));
		temp.Format(linePattern,
			SourcesImage,
			"ed2k://|file|" + strFilename + "|" + strSize + "|" + strHash + "|/",
			OverlayImage,
			(column1) ? coloraddonB + StringLimit(sf1->GetFileName(),70) + coloraddonE : "",
			(column2) ? coloraddonB + CastItoXBytes(sf1->GetFileSize()) + coloraddonE : "",
			(column3) ? coloraddonB + strHash + coloraddonE : "",
			(column4) ? coloraddonB + strSources + coloraddonE : "",
			strHash
			);
		buffer.Append(temp);
	}
	return buffer;

	EMULE_CATCH
	return _T("");
}

void CSearchList::AddFileToDownloadByHash(uchar* hash,uint8 cat) {
	EMULE_TRY

	CSearchFile* sf;

	for (POSITION pos = list.GetHeadPosition(); pos !=0;list.GetNext(pos)){
		sf=(CSearchFile*)list.GetAt(pos);//->GetSearchID() == nSearchID ){
		if (!md4cmp(hash,sf->GetFileHash())) {
			theApp.downloadqueue->AddSearchToDownload(sf,cat);
			break;
		}
	}

	EMULE_CATCH
}

// mobilemule

CSearchFile* CSearchList::DetachNextFile(uint32 nSearchID) {
	EMULE_TRY
	// the files are NOT deleted, make sure you do this if you call this function
	// find, removes and returns the searchresult with most Sources	
	uint32 nHighSource = 0;
	POSITION resultpos = 0;
	for (POSITION pos = list.GetHeadPosition(); pos != NULL; ){
		CSearchFile* cur_file = list.GetNext(pos);
		if( cur_file->GetSearchID() == nSearchID ){
			if(cur_file->GetIntTagValue(FT_SOURCES) > nHighSource){
				nHighSource = cur_file->GetIntTagValue(FT_SOURCES);
				resultpos = pos;
			}
		}
	}
	if (resultpos == 0){
		ASSERT ( false );
		return NULL;
	}
	CSearchFile* result = list.GetAt(resultpos);
	list.RemoveAt(resultpos);
	return result;
	EMULE_CATCH
	return NULL;
}