//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


#include "stdafx.h"
#include "emule.h"
#include "packets.h"
#include "serversocket.h"
#include "emuleDlg.h"
#include "opcodes.h"
#include "searchlist.h"
#include <time.h>
#include <afxmt.h>
#include "memcpy_amd.h"
#include "Smtp.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#ifdef OLD_SOCKETS_ENABLED

CServerSocket::CServerSocket(CServerConnect* in_serverconnect)
{
	EMULE_TRY

	serverconnect = in_serverconnect;
	connectionstate = 0;
	cur_server = NULL;
	info="";
	oldID = 0;
	messagecount= 0;
	
	EMULE_CATCH
}

CServerSocket::~CServerSocket(){
	if (cur_server)
		delete cur_server;
	cur_server = NULL;
}

void CServerSocket::OnConnect(int nErrorCode)
{
	EMULE_TRY
		
	CAsyncSocketEx::OnConnect(nErrorCode); // deadlake PROXYSUPPORT - changed to AsyncSocketEx
	switch (nErrorCode){
		case 0:{
			if (cur_server->HasDynIP()){
				SOCKADDR_IN sockAddr;
				memset(&sockAddr, 0, sizeof(sockAddr));
				uint32 nSockAddrLen = sizeof(sockAddr);
				GetPeerName((SOCKADDR*)&sockAddr,(int*)&nSockAddrLen);
				cur_server->SetID(sockAddr.sin_addr.S_un.S_addr);
				theApp.serverlist->GetServerByAddress(cur_server->GetAddress(),cur_server->GetPort())->SetID(sockAddr.sin_addr.S_un.S_addr);
			}
			SetConnectionState(CS_WAITFORLOGIN);
			messagecount= 0;
			break;
		}
		case WSAEADDRNOTAVAIL:
		case WSAECONNREFUSED:
		case WSAENETUNREACH:
		case WSAETIMEDOUT: 	
		case WSAEADDRINUSE:
			SetConnectionState(CS_SERVERDEAD);
			//serverconnect->DestroySocket(this);	//eklmn & I2K: merge CloseSocket & DestroySocket (LowID problem)
			return;
		// deadlake PROXYSUPPORT
		case WSAECONNABORTED:
			if (m_ProxyConnectFailed)
			{
				m_ProxyConnectFailed = false;
				SetConnectionState(CS_SERVERDEAD);
				//serverconnect->DestroySocket(this);	//eklmn & I2K: merge CloseSocket & DestroySocket (LowID problem)
				return;
			}
		default:
			SetConnectionState(CS_FATALERROR);
			//serverconnect->DestroySocket(this);	//eklmn & I2K: merge CloseSocket & DestroySocket (LowID problem)
			return;
	}	 
	
	EMULE_CATCH
}

void CServerSocket::OnReceive(int nErrorCode)
{
	if (connectionstate != CS_CONNECTED && !this->serverconnect->IsConnecting()){
		//serverconnect->DestroySocket(this);
		return;
	}
	CEMSocket::OnReceive(nErrorCode);

}

bool CServerSocket::ProcessPacket(char* packet, int32 size, int8 opcode){
	try{
		switch(opcode){
			case OP_SERVERMESSAGE:{
				theApp.downloadqueue->AddDownDataOverheadServer(size);
				
				//fixed "OP_SERVERMESSAGE Format String Vulnerability" - from official 0.29b - bond006
				CString message;
				if (size >= 2){
					UINT uLen = *(uint16*)packet;
					if (uLen > size-2)
						uLen = size-2;
					#ifndef AMD
					memcpy(message.GetBuffer(uLen), packet + sizeof uint16, uLen);
					#else
					memcpy_amd(message.GetBuffer(uLen), packet + sizeof uint16, uLen);
					#endif
					message.ReleaseBuffer(uLen);
				}
				if ((messagecount < 2) && (message.Left(14).CompareNoCase("server version") == 0)){
					CString strVer = message.Mid(14);
					strVer.Trim();
					strVer = strVer.Left(64); // truncate string to avoid misuse by servers in showing ads
					int n= strVer.FindOneOf("\x09\x0A\x0D");
					if (n >= 0)
						strVer= strVer.Left(n);
					CServer* eserver = theApp.serverlist->GetServerByAddress(cur_server->GetAddress(),cur_server->GetPort());
					if (eserver)
						eserver->SetVersion(strVer);
				}
				if (message.Find(_T("[emDynIP: ")) != (-1) && message.Find(_T("]")) != (-1) && message.Find(_T("[emDynIP: ")) < message.Find(_T("]"))){
					CString dynip = message.Mid(message.Find(_T("[emDynIP: "))+10,message.Find(_T("]")) - (message.Find(_T("[emDynIP: "))+10));
					dynip.Trim(_T(" "));
					if ( dynip.GetLength() && dynip.GetLength() < 51){
						CServer* eserver = theApp.serverlist->GetServerByAddress(cur_server->GetAddress(),cur_server->GetPort());
						if (eserver){
							eserver->SetDynIP(dynip.GetBuffer());
							cur_server->SetDynIP(dynip.GetBuffer());
							theApp.emuledlg->serverwnd.serverlistctrl.RefreshServer(*eserver);
						}
					}
				}
				theApp.emuledlg->AddServerMessageLine(message);
				messagecount++;
				break;
			}
			case OP_IDCHANGE:{
				theApp.downloadqueue->AddDownDataOverheadServer(size);
				if (size < sizeof(LoginAnswer_Struct)){
					throw GetResString(IDS_ERR_BADSERVERREPLY);
				}
				LoginAnswer_Struct* la = (LoginAnswer_Struct*) packet;
				uint8 state;
				if (la->clientid == 0){
//					SetConnectionState(CS_ERROR);
					state = theApp.glob_prefs->GetSmartIdState();
					if ( state > 0 ){
//						SetConnectionState(CS_ERROR);
						state++;
						if( state > 3 )
							theApp.glob_prefs->SetSmartIdState(0);
						else
							theApp.glob_prefs->SetSmartIdState(state);
					}
					break;
				}
				if( theApp.glob_prefs->GetSmartIdCheck() ){
					if (la->clientid >= 16777216 )
						theApp.glob_prefs->SetSmartIdState(1);
					else{
						state = theApp.glob_prefs->GetSmartIdState();
						if ( state > 0 ){
//							SetConnectionState(CS_ERROR);
							state++;
							if( state > 3 )
							{
								theApp.glob_prefs->SetSmartIdState(0);
								// Purity - Sending message when connected with lowid
								CString MessageText;
								MessageText.Format(GetResString(IDS_GOTLOWID));
								theApp.smtpconnection->SendMail(MessageText, theApp.glob_prefs->GetNotifierPopOnServerError(), theApp.glob_prefs->IsSMTPWarningEnabled());
								theApp.emuledlg->ShowNotifier(MessageText, TBN_SERVER, false, theApp.glob_prefs->GetNotifierPopOnServerError());
							}
							else
							{
								SetConnectionState(CS_ERROR);
								theApp.emuledlg->AddLogLine	(false, GetResString(IDS_GOTLOWID));		//Cax2 - smartId bugfix
								theApp.glob_prefs->SetSmartIdState(state);
							}
							break;
						}
					}
				}
				if (connectionstate != CS_CONNECTED) {
					SetConnectionState(CS_CONNECTED);
					theApp.OnlineSig();       // Added By Bouc7 
				}
				serverconnect->SetClientID(la->clientid);
				if (la->clientid != oldID)
				{
					// Purity - Sending message when ID change
					CString MessageText;
					MessageText.Format(GetResString(IDS_NEWCLIENTID),la->clientid);
					theApp.smtpconnection->SendMail(MessageText, theApp.glob_prefs->GetNotifierPopOnServerError(), theApp.glob_prefs->IsSMTPWarningEnabled());
					theApp.emuledlg->ShowNotifier(MessageText, TBN_SERVER, false, theApp.glob_prefs->GetNotifierPopOnServerError());
				}
				oldID = la->clientid;
				AddLogLine(false,IDS_NEWCLIENTID,la->clientid);

				theApp.downloadqueue->ResumeFiles();
				
				//theApp.downloadqueue->filelist.GetCount();				
				break;
			}
			case OP_SEARCHRESULT:{
				theApp.downloadqueue->AddDownDataOverheadServer(size);
				theApp.emuledlg->searchwnd.LocalSearchEnd(theApp.searchlist->ProcessSearchanswer(packet,size));
				break;
			}
			case OP_FOUNDSOURCES:{
				theApp.downloadqueue->AddDownDataOverheadServer(size);
				CMemFile sources((BYTE*)packet,size);
				uchar fileid[16];
				sources.Read(fileid,16);
				if (CPartFile* file = theApp.downloadqueue->GetFileByID(fileid))
					file->AddSources(sources,cur_server->GetIP(), cur_server->GetPort());				
				break;
			}
			case OP_SERVERSTATUS:{
				theApp.downloadqueue->AddDownDataOverheadServer(size);
				// FIXME some statuspackets have a different size -> why? structur?
				if (size < 8)
					break;//throw "Invalid status packet";
				uint32 cur_user;
				uint32 cur_files;
				#ifndef AMD
				memcpy(&cur_user,packet,4);
				memcpy(&cur_files,packet+4,4);
				#else
				memcpy_amd(&cur_user,packet,4);
				memcpy_amd(&cur_files,packet+4,4);
				#endif
				CServer* update = theApp.serverlist->GetServerByAddress( cur_server->GetAddress(), cur_server->GetPort() );
				if (update){
					update->SetUserCount(cur_user); 
					update->SetFileCount(cur_files);
					theApp.emuledlg->ShowUserCount(cur_user, cur_files);
					theApp.emuledlg->serverwnd.serverlistctrl.RefreshServer(*update );
				}
				break;
			}
			case OP_SERVERIDENT:{ 
				theApp.downloadqueue->AddDownDataOverheadServer(size);
				if (size<16+4+2+4) { 
					AddDebugLogLine(false,IDS_ERR_KNOWNSERVERINFOREC); 
					break;// throw "Invalid server info received"; 
				} 

				uint8 aucHash[16];
				uint32 nServerIP;
				uint16 nServerPort;
				uint32 nTags;
				CSafeMemFile data((BYTE*)packet, size);
				
				data.Read(aucHash, 16);
				data.Read(&nServerIP, 4);
				data.Read(&nServerPort, 2);
				data.Read(&nTags, 4);

				CString strName;
				CString strDescription;
				for (UINT i = 0; i < nTags; i++){
					CTag tag(data);
					if (tag.GetSpecialTag() == ST_SERVERNAME){
						if (tag.GetType() == 2){
							strName = tag.GetStringValue();
						}
					}
					else if (tag.GetSpecialTag() == ST_DESCRIPTION){
						if (tag.GetType() == 2){
							strDescription = tag.GetStringValue();
						}
					}
				}

				ASSERT( cur_server );
				if (cur_server){
					CServer* update = theApp.serverlist->GetServerByAddress(cur_server->GetAddress(),cur_server->GetPort()); 
					ASSERT( update );
					if (update){
						update->SetListName(strName);
						update->SetDescription(strDescription);
						if (((uint32*)aucHash)[0] == 0x2A2A2A2A){
							const CString& rstrVersion = update->GetVersion();
							if (!rstrVersion.IsEmpty())
								update->SetVersion(_T("eFarm ") + rstrVersion);
							else
								update->SetVersion(_T("eFarm"));
						}
						theApp.emuledlg->ShowConnectionState(true,update->GetListName()); 
						theApp.emuledlg->serverwnd.serverlistctrl.RefreshServer(*update); 
					}
				}
				break;
			} 
			// tecxx 1609 2002 - add server's serverlist to own serverlist
			case OP_SERVERLIST:{
				theApp.downloadqueue->AddDownDataOverheadServer(size);
				try{
					CSafeMemFile servers((BYTE*)packet,size);
					byte count;
					servers.Read(&count, 1);// check if packet is valid 
					if ((int32)(count*6 + 1) > size)	 // (servercount*(4bytes ip + 2bytes port) + 1byte servercount)
						count = 0;
					int addcount = 0;
					while(count)
					{
						uint32 ip;
						unsigned short port;
						servers.Read(&ip, 4);
						servers.Read(&port, 2);
						in_addr host;
						host.S_un.S_addr = ip;
						CString s(inet_ntoa(host));
						CServer* srv = new CServer(port, s);
						srv->SetListName(srv->GetFullIP());
						if (!theApp.emuledlg->serverwnd.serverlistctrl.AddServer(srv, true))
							delete srv;
						else
							addcount++;
						count--;
					}					
					if (addcount)
						AddLogLine(false,IDS_NEWSERVERS, addcount);
				}
				catch(CFileException* error){
					OUTPUT_DEBUG_TRACE();
					AddDebugLogLine(false,IDS_ERR_BADSERVERLISTRECEIVED);
					error->Delete();	//memleak fix
				}
				break;
			}
			case OP_CALLBACKREQUESTED:{
				theApp.downloadqueue->AddDownDataOverheadServer(size);
				if (size == 6){
					uint32 dwIP;
					uint16 nPort;
					#ifndef AMD
					memcpy(&dwIP,packet,4);
					memcpy(&nPort,packet+4,2);
					#else
					memcpy_amd(&dwIP,packet,4);
					memcpy_amd(&nPort,packet+4,2);
					#endif
					CUpDownClient* client = theApp.clientlist->FindClientByIP(dwIP,nPort);
					if (client)
						client->TryToConnect();
					else{
						client = new CUpDownClient(nPort,dwIP,0,0,0);
						theApp.clientlist->AddClient(client);
						client->TryToConnect();
					}
				}
				break;
			}
			default:
				;
		}
		return true;
	}
	catch(CFileException* error){
		OUTPUT_DEBUG_TRACE();
		error->m_strFileName = "server packet";
		AddDebugLogLine(false,IDS_ERR_PACKAGEHANDLING,GetErrorMessage(error));
		error->Delete();
		return false;
	}
	catch(CString error){
		OUTPUT_DEBUG_TRACE();
		AddDebugLogLine(false,IDS_ERR_PACKAGEHANDLING,error.GetBuffer());
		SetConnectionState(CS_DISCONNECTED);
		return false;
	}
}

void CServerSocket::ConnectToServer(CServer& server)
{
	EMULE_TRY
		
	if (&server == NULL) return;	//eklmn. check NULL case

	//check cur_server: if object already exist, then replace it
	safe_delete(cur_server);
	cur_server = new CServer(server);
	
	AddLogLine(false,IDS_CONNECTINGTO,cur_server->GetListName(),cur_server->GetFullIP(),cur_server->GetPort());

	if (theApp.glob_prefs->IsProxyASCWOP() )
	{
		if (theApp.glob_prefs->GetProxy().UseProxy == true)
		{
			theApp.glob_prefs->SetProxyASCWOP(true);
			theApp.glob_prefs->SetUseProxy(false);
			AddLogLine(false,_T("%s: %s"),GetResString(IDS_SERVERCON_WO_PROXY), GetResString(IDS_DISABLED).MakeLower());
		}
		else
			theApp.glob_prefs->SetProxyASCWOP(false);
	}
	SetConnectionState(CS_CONNECTING);
	//AddDebugLogLine(false, "-server->: Set status to CS_CONNECTING and call connect function");
	if (!this->Connect(server.GetAddress(),server.GetPort())){
		int error = this->GetLastError();
		if ( error != WSAEWOULDBLOCK){
			AddLogLine(false,IDS_ERR_CONNECTIONERROR,cur_server->GetListName(),cur_server->GetFullIP(),cur_server->GetPort(), error); 
			SetConnectionState(CS_FATALERROR);
			return;
		}
	}
	else 
	{
		info=server.GetListName();
		SetConnectionState(CS_CONNECTED);
	}

	EMULE_CATCH
}

void CServerSocket::OnError(int nErrorCode)
{
	EMULE_TRY
		
	SetConnectionState(CS_DISCONNECTED);
	AddDebugLogLine(false,IDS_ERR_SOCKET,cur_server->GetListName(),cur_server->GetFullIP(),cur_server->GetPort(), nErrorCode); 
	
	EMULE_CATCH
}

void CServerSocket::PacketReceived(Packet* packet)
{
	EMULE_TRY
		
	ProcessPacket(packet->pBuffer,packet->size,packet->opcode);
	
	EMULE_CATCH
}

void CServerSocket::OnClose(int nErrorCode){
	CEMSocket::OnClose(0);
	if (connectionstate == CS_WAITFORLOGIN){	 	
		SetConnectionState(CS_SERVERFULL);
	}
	else if (connectionstate == CS_CONNECTED){
		SetConnectionState(CS_DISCONNECTED);		
	}
	else{
		SetConnectionState(CS_NOTCONNECTED);
	}
	//serverconnect->DestroySocket(this);	//eklmn & I2K: merge CloseSocket & DestroySocket (LowID problem)
}

void CServerSocket::SetConnectionState(sint8 newstate)
{
	EMULE_TRY
		
	connectionstate = newstate;
	if (newstate < 1){
		serverconnect->ConnectionFailed(this);
	}
	else if (newstate == CS_CONNECTED || newstate == CS_WAITFORLOGIN){
		if (serverconnect)
			serverconnect->ConnectionEstablished(this);
	}
	
	EMULE_CATCH
}

#endif OLD_SOCKETS_ENABLED
