//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include "StdAfx.h"
#include "sharedfilelist.h"
#include "knownfilelist.h"
#include "packets.h"
#include <time.h>


#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


CSharedFileList::CSharedFileList(CPreferences* in_prefs,CServerConnect* in_server,CKnownFileList* in_filelist){
	app_prefs = in_prefs;
	server = in_server;
	filelist = in_filelist;
	output = 0;
	CSingleLock sLock(&list_mut,true);
	m_Files_map.InitHashTable(1024);
	sLock.Unlock();
	FindSharedFiles();
	m_lancast = new CLanCast(in_prefs);	// LANCAST (moosetea) - 
	delayreload=false;
}

// LANCAST (moosetea) - Broadcast the next hash 
void CSharedFileList::NextLANBroadcast(){
	if (theApp.m_app_state != theApp.APP_STATE_RUNNING )		//Cax2 - crash on exit bugfix
		return;

	static POSITION pos = 0;
	static CCKey bufKey;

	CKnownFile* cur_file;

	CSingleLock sLock(&list_mut,true);
	if (!m_Files_map.IsEmpty()){

		if (!pos)
			pos = m_Files_map.GetStartPosition();

		if (pos) {
			m_Files_map.GetNextAssoc(pos,bufKey,cur_file);
			m_lancast->BroadcastHash(cur_file);
		}
	}
	sLock.Unlock();
}

CSharedFileList::~CSharedFileList(){
	delete m_lancast; //memory leak [TwoBottle Mod]
}

void CSharedFileList::FindSharedFiles(){
	BOOL bEmpty;
	CSingleLock sLock1(&list_mut,true); // list thread safe
	bEmpty	=	m_Files_map.IsEmpty();
	sLock1.Unlock();
	if (!bEmpty){
		//--- xrmb:allyaknow ---
		CCKey		bufKey;
		CKnownFile* cur_file;		
		sLock1.Lock();
		POSITION	pos=m_Files_map.GetStartPosition();
		while(pos)
		{
			m_Files_map.GetNextAssoc(pos, bufKey, cur_file);
			cur_file->SetSharedFile(false);
		}
		//--- :xrmb ---
		m_Files_map.RemoveAll();
		sLock1.Unlock();
		theApp.downloadqueue->AddPartFilesToShare(); // readd partfiles
	}
	if(!app_prefs->shareddir_list.Find(app_prefs->GetIncomingDir()) && !app_prefs->shareddir_list.Find(app_prefs->GetIncomingDir()+'\\'))
		AddFilesFromDirectory(app_prefs->GetIncomingDir().GetBuffer());
	for (POSITION pos = app_prefs->shareddir_list.GetHeadPosition();pos != 0;app_prefs->shareddir_list.GetNext(pos))
		AddFilesFromDirectory(app_prefs->shareddir_list.GetAt(pos).GetBuffer());
	sLock1.Lock();
	if (waitingforhash_list.IsEmpty()) 
		AddLogLine(false, IDS_SHAREDFOUND,m_Files_map.GetCount());
	else
		AddLogLine(false, IDS_SHAREDFOUNDHASHING,m_Files_map.GetCount(),waitingforhash_list.GetCount());
	sLock1.Unlock();
	BuildSharedVDirForList();
	HashNextFile();
}

void CSharedFileList::AddFilesFromDirectory(const CString& directory){
	CString ext; //pruv3750
	CFileFind ff;
	CString searchpath;
	searchpath.Format(_T("%s\\*"),directory);
	bool end = !ff.FindFile(searchpath,0);	
	if (end)
		return;
	//ff.FindNextFile();
	while (!end){
		end = !ff.FindNextFile();
		if (ff.IsDirectory() || ff.IsDots() || ff.IsSystem() || ff.IsTemporary() || ff.GetLength()>=4294967295 )
			continue;
		// Ultras: prevent to add the zero lenghts files
		if( !ff.GetLength() ) continue;
		CTime lwtime;
		ff.GetLastWriteTime(lwtime);
		ext= ff.GetFileName().Right(ff.GetFileName().GetLength()-ff.GetFileName().ReverseFind('.')).MakeLower() ; //pruv3750
		if (!(ext==".part" || ext==".met" || ext==".dll" || ext==".sys" || ext==".drv" || ext==".tmp" || ext==".log" || ext==".torrent"))
		{
			uint32 fdate = mktime(lwtime.GetLocalTm());
			CKnownFile* toadd = filelist->FindKnownFile(ff.GetFileName().GetBuffer(),fdate,(uint32)ff.GetLength());
			if (toadd){
				toadd->SetPath(directory);
				CSingleLock sLock(&list_mut,true);
				//--- xrmb:allyaknow ---
				toadd->SetSharedFile(true);
				//--- :xrmb ---
				m_Files_map.SetAt(CCKey(toadd->GetFileHash()),toadd);
				sLock.Unlock();
			} else {
				//not in knownfilelist - start adding thread to hash file
				UnknownFile_Struct* tohash = new UnknownFile_Struct;
				tohash->directory = directory;
				tohash->name = ff.GetFileName();
				waitingforhash_list.AddTail(tohash);
			}
		}
	}
	ff.Close();
}

void CSharedFileList::SafeAddKFile(CKnownFile* toadd, bool bOnlyAdd){
	//--- xrmb:allyaknow ---
	toadd->SetSharedFile(true);
	//--- :xrmb ---

	// TODO: Check if the file is already known - only with another date
	CSingleLock sLock(&list_mut,true);
	m_Files_map.SetAt(CCKey(toadd->GetFileHash()),toadd);
	sLock.Unlock();
	if (bOnlyAdd)
		return;
	if (output)
		output->ShowFile(toadd);
	HashNextFile();
	// offer new file to server
#ifdef OLD_SOCKETS_ENABLED
	if (!server->IsConnected())
		return;
#endif //OLD_SOCKETS_ENABLED
	toadd->SetOfferedToServer(true);
	CMemFile files(100);
	uint32 filecount = 1;
	files.Write(&filecount,4);
	CreateOfferedFilePacket(*toadd,files);
	Packet* packet = new Packet(&files);
	packet->opcode = OP_OFFERFILES;	
	theApp.uploadqueue->AddUpDataOverheadServer(packet->size);
#ifdef OLD_SOCKETS_ENABLED
	server->SendPacket(packet,true);
#endif //OLD_SOCKETS_ENABLED
}

// removes first occurrence of 'toremove' in 'list'
void CSharedFileList::RemoveFile(CKnownFile* toremove){
	//--- xrmb:allyaknow ---
	toremove->SetSharedFile(false);
	//--- :xrmb ---
	output->RemoveFile(toremove);
	CSingleLock sLock(&list_mut,true);
	m_Files_map.RemoveKey(CCKey(toremove->GetFileHash()));
	filelist->RemoveFile(toremove);
}

void CSharedFileList::Reload(bool sendtoserver){
	AddDebugLogLine(false, _T("Reloading shared files"));
	if (waitingforhash_list.IsEmpty())
	{
	    FindSharedFiles();
	    if (output)
		    output->ShowFileList(this);
	    if (sendtoserver)
		    SendListToServer();
    }
	else
	{
		waitingforhash_list.RemoveAll();
		delayreload=true;
	}
}

void CSharedFileList::SetOutputCtrl(CSharedFilesCtrl* in_ctrl){
	output = in_ctrl;
	output->ShowFileList(this);
}

void CSharedFileList::SendListToServer(){
	CSingleLock sLock(&list_mut,true);
#ifdef OLD_SOCKETS_ENABLED
	if (m_Files_map.IsEmpty() || !server->IsConnected())
		return;
#endif //OLD_SOCKETS_ENABLED
	CMemFile files;
	CCKey bufKey;
	CKnownFile* cur_file;
	uint32 filecount;

	// send all files
#ifdef OLD_SOCKETS_ENABLED
	if (server->GetCurrentServer()->GetSoftFiles() && (uint32)m_Files_map.GetCount()<=server->GetCurrentServer()->GetSoftFiles() ) 
	{
		filecount = m_Files_map.GetCount();
		files.Write(&filecount,4);
		for (POSITION pos = m_Files_map.GetStartPosition();pos != 0;)
		{
			m_Files_map.GetNextAssoc(pos,bufKey,cur_file);
			CreateOfferedFilePacket(*cur_file,files);
			cur_file->SetOfferedToServer(true);
		}
	}
	else
	{
		// send subset
		POSITION pos, pos2;
		CTypedPtrList<CPtrList, CKnownFile*> sortedList;
		bool added=false;
		for (pos= m_Files_map.GetStartPosition(); pos!=0;)
		{
			m_Files_map.GetNextAssoc(pos, bufKey, cur_file);
			added=false;

			//insertsort into sortedList
			for (pos2 = sortedList.GetHeadPosition();pos2 != 0 && !added;sortedList.GetNext(pos2))
			{
				uint8 nPrioPos2= GetRealPrio(sortedList.GetAt(pos2)->GetPriority());
				uint8 nPrioFile= GetRealPrio(cur_file->GetPriority());
				if ((nPrioPos2 < nPrioFile) || ((nPrioPos2 == nPrioFile) && (sortedList.GetAt(pos2)->GetFileName().CompareNoCase(cur_file->GetFileName()) > 0)))
				{
					sortedList.InsertBefore(pos2,cur_file);
					added=true;
				}
			}
			if (!added)
				sortedList.AddTail(cur_file);
		}
		// add to packet
		filecount = server->GetCurrentServer()->GetSoftFiles();
		if (filecount == 0) // if we don't know the soft limit, we send all our files
			filecount = sortedList.GetCount();
		files.Write(&filecount,4);
		uint32 count=0;
		pos = sortedList.GetHeadPosition();
		while (pos != 0 && count<filecount)
		{
			count++;
			sortedList.GetAt(pos)->SetOfferedToServer(true);
			CreateOfferedFilePacket(*sortedList.GetNext(pos),files);
		}
		while (pos != 0 && count<(uint32)sortedList.GetCount())
		{
			count++;
			sortedList.GetNext(pos)->SetOfferedToServer(false);
		}
		sortedList.RemoveAll();
	}
#endif //OLD_SOCKETS_ENABLED
	if (output)
		output->UpdateOfferedItems();
	sLock.Unlock();
	Packet* packet = new Packet(&files);
	packet->opcode = OP_OFFERFILES;	
	theApp.uploadqueue->AddUpDataOverheadServer(packet->size);
#ifdef OLD_SOCKETS_ENABLED
	server->SendPacket(packet,true);
#endif //OLD_SOCKETS_ENABLED
}

void CSharedFileList::CreateOfferedFilePacket(CKnownFile& cur_file,CMemFile& files){
	files.Write(cur_file.GetFileHash(),16);
	char buffer[6];
	memset(buffer,0,6);
	files.Write(buffer,6);	
	uint32 fType =	cur_file.GetFileType();
	files.Write(&fType,4);
	CTag* nametag = new CTag(FT_FILENAME,cur_file.GetFileName());
	nametag->WriteTagToFile(files);
	delete nametag;
	CTag* sizetag = new CTag(FT_FILESIZE,cur_file.GetFileSize());
	sizetag->WriteTagToFile(files);
	delete sizetag;
	//TODO add tags for documents mp3 etc
}

// -khaos--+++> New param:  pbytesLargest, pointer to uint64.
//				Various other changes to accomodate our new statistic...
//				Point of this is to find the largest file currently shared.
uint64 CSharedFileList::GetDatasize(uint64 &pbytesLargest) {
	pbytesLargest=0;
	// <-----khaos-
	uint64 fsize;
	fsize=0;

	CCKey bufKey;
	CKnownFile* cur_file;
	CSingleLock sLock(&list_mut,true);
	for (POSITION pos = m_Files_map.GetStartPosition();pos != 0;)
	{
		m_Files_map.GetNextAssoc(pos,bufKey,cur_file);
		fsize+=cur_file->GetFileSize();
		// -khaos--+++> If this file is bigger than all the others...well duh.
		if (cur_file->GetFileSize() > pbytesLargest) pbytesLargest = cur_file->GetFileSize();
		// <-----khaos-
	}
	return fsize;
}

CKnownFile*	CSharedFileList::GetFileByID(const uchar* filehash){
	CKnownFile* result;
	CCKey tkey(filehash);
	CSingleLock sLock(&list_mut,true);
	if (m_Files_map.Lookup(tkey,result))
		return result;
	else
		return 0;
}

short CSharedFileList::GetFilePriorityByID(uchar* filehash){
	CKnownFile* tocheck = GetFileByID(filehash);
	if (tocheck)
		return tocheck->GetPriority();
	else
		return -10;	// file doesn't exist
}

void CSharedFileList::HashNextFile()
{
	if(delayreload==true)
	{
		delayreload=false;
		Reload();
		return;
	}
	if (waitingforhash_list.IsEmpty())
		return;
	UnknownFile_Struct* nextfile = waitingforhash_list.RemoveHead();
	theApp.m_FilesHashing.AddToHash(this,nextfile->directory,nextfile->name); 	
	delete nextfile;	
}

IMPLEMENT_DYNCREATE(CAddFileThread, CWinThread)
CAddFileThread::CAddFileThread(){
	m_pOwner = 0;	
}
void CAddFileThread::SetValues(CSharedFileList* pOwner, LPCTSTR in_directory, LPCTSTR in_filename, CPartFile* in_partfile_Owner){
	 m_pOwner = pOwner;
	 directory = in_directory;
	 filename = in_filename;
	 partfile_Owner = in_partfile_Owner;
}

int CAddFileThread::Run(){
	if (!(m_pOwner || partfile_Owner) || filename.IsEmpty())
		AfxEndThread(0,true);
	CSingleLock sLock1(&(theApp.hashing_mut)); // only one filehash at a time
	sLock1.Lock();
	CKnownFile* newrecord = new CKnownFile();
	if (newrecord->CreateFromFile(directory,filename)){
		if (theApp.emuledlg)
			PostMessage(theApp.emuledlg->m_hWnd,TM_FINISHEDHASHING,(m_pOwner ? 0:(WPARAM)partfile_Owner),(LPARAM)newrecord);
	
	}
	else{
		delete newrecord;
	}

	sLock1.Unlock();
	AfxEndThread(0,true);
	return 0;
}

void CSharedFileList::UpdateItem(CKnownFile* toupdate,bool resort) {output->UpdateItem(toupdate,resort);}

// #zegzav:vdir4list (BEGIN)
#define INCOMPLETE_DIR		_T("!Incomplete Files")
#define SHARED_DIR			_T("!Shared Files")

bool CSharedFileList::AddSharedVDirForList(bool bIncomplete, const CString &strNewDir)
{
	EMULE_TRY
	// clean dir
	CString strDir(strNewDir);
	PathRemoveBackslash(strDir.GetBuffer());
	if (strDir.IsEmpty())
		return false;

	// check already added
	POSITION pos= m_SharedVDirForList.GetStartPosition();
	while (pos)
	{
		CString strCurKey;
		CString strCurDir;
		m_SharedVDirForList.GetNextAssoc(pos, strCurKey, strCurDir);
		if (strCurDir.CompareNoCase(strDir) == 0)
			return false;
	}

	// set key
	CString strKey, strWantedKey;
	if (bIncomplete)
	{
		strWantedKey= INCOMPLETE_DIR;	// default name used by Hybrid for temp dir
	}
	else
	{
		strWantedKey= PathFindFileName((LPCTSTR)strDir);	// short name without complete path
		if ((strWantedKey.IsEmpty()) || ((strWantedKey.GetLength() > 1) && (strWantedKey[1] == ':')))
			strWantedKey= SHARED_DIR;
	}

	CString strDir0, strDir1;
	if ((m_SharedVDirForList.Lookup(strWantedKey, strDir0)) || (m_SharedVDirForList.Lookup(strWantedKey + _T(" (1)"), strDir1)))
	{
		// rename key 0
		if (!strDir0.IsEmpty())
		{
			m_SharedVDirForList[strWantedKey + _T(" (1)")]= strDir0;
			m_SharedVDirForList.RemoveKey(strWantedKey);
		}

		// build unique key
		int n= 2;
		do
		{
			strKey.Format(_T("%s (%d)"), (LPCTSTR)strWantedKey, n++);
		} while (m_SharedVDirForList.Lookup(strKey, strDir0));
	}
	else
		strKey= strWantedKey;
	m_SharedVDirForList[strKey]= strDir;

	return true;

	EMULE_CATCH
	return false;
}

void CSharedFileList::BuildSharedVDirForList()
{
	if (!app_prefs)
		return;

	POSITION pos;
	m_SharedVDirForList.RemoveAll();

	// temp dirs
	AddSharedVDirForList(true, app_prefs->GetTempDir());
	pos= app_prefs->tempdir_list.GetHeadPosition();
	while (pos)
		AddSharedVDirForList(true, app_prefs->tempdir_list.GetNext(pos));

	// incoming dir
	AddSharedVDirForList(false, app_prefs->GetIncomingDir());

	// shared dirs
    pos = app_prefs->shareddir_list.GetHeadPosition();
    while (pos)
		AddSharedVDirForList(false, app_prefs->shareddir_list.GetNext(pos));
	
	// category dirs
	for (int i= 0; i < app_prefs->GetCatCount(); i++)
		AddSharedVDirForList(false, app_prefs->GetCatPath(i));
}
// #zegzav:vdir4list (END)
