// TaskProcessor_Main.cpp: implementation of the CTaskProcessor_Main class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#ifdef NEW_SOCKETS

#include "TaskProcessor_Main.h"
#include "EmEngine.h"

bool CTaskProcessor_Main::Start()
{
	m_mapClients.InitHashTable(997);
	m_dwWaitTimeout = 500;
	return true;
}

void CTaskProcessor_Main::Stop()
{
	// destroy all clients that are possibly left in our map
	for (POSITION pos = m_mapClients.GetStartPosition(); pos; )
	{
		CEmClient* pClient;
		SOCKET hSocket;
		m_mapClients.GetNextAssoc(pos, hSocket, pClient);
		ASSERT(pClient);
		delete pClient;
	}
	m_mapClients.RemoveAll();
}

void CTaskProcessor_Main::ProcessTimeout()
{
//	AllocTcpConnect(inet_addr("193.111.198.139"), 4242, T_CLIENT_SERVER);

//	CEmClient_Web* pClient = (CEmClient_Web*) AllocTcpConnect(inet_addr("216.109.125.70"), 80, T_CLIENT_WEB);
//	if (pClient)
//		pClient->m_bIsJunky = true;
	m_dwWaitTimeout = INFINITE;
}

CEmClient* CTaskProcessor_Main::AllocClient(T_CLIENT_TYPE eType)
{
	switch (eType)
	{
	case T_CLIENT_PEER:
		return new CEmClient_Peer;
		break;
	case T_CLIENT_SERVER:
		return new CEmClient_Server;
		break;
	case T_CLIENT_WEB:
		return new CEmClient_Web;
		break;
	default:
		ASSERT(FALSE); // invalid type ???
	}
	return NULL;
}

CEmClient* CTaskProcessor_Main::AllocTcpConnect(ULONG nAddr, USHORT nPort, T_CLIENT_TYPE eClientType)
{
	CEmClient* pClient = AllocClient(eClientType);
	if (pClient)
	{
		pClient->m_nAddr = nAddr;
		pClient->m_nPort = nPort;
		SOCKET hSocket = m_stEngine.m_stTcp.AllocConnect(nAddr, nPort, eClientType);
		if (INVALID_SOCKET == hSocket)
		{
			delete pClient;
			pClient = NULL;
		} else
		{
			pClient->m_hSocket = hSocket;
			pClient->m_bIsConnected = false;

			m_mapClients.SetAt(hSocket, pClient);
		}
	} 
	else
	{
		ASSERT(FALSE);
		AddDebugLogLine(false, "no memory");
	}

	return pClient;
}

void CTaskProcessor_Main::KillClient(CEmClient* pClient)
{
	ASSERT(pClient);
	SOCKET hSocket = pClient->m_hSocket;
	pClient->OnDisconnected(m_stEngine);
	delete pClient;
	m_mapClients.RemoveKey(hSocket);
}

#endif