#include "StdAfx.h"
#include "emule.h"
#include "webserver.h"
#include "ED2KLink.h"
#include "MD5Sum.h"
#include "Smtp.h"
#include "ini2.h"
#include "QArray.h"
#include <stdlib.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#define HTTPInit _T("Server: eMule\r\nConnection: close\r\nContent-Type: text/html\r\n")
#define HTTPInitGZ _T("Server: eMule\r\nConnection: close\r\nContent-Type: text/html\r\nContent-Encoding: gzip\r\n")

#define WEB_SERVER_TEMPLATES_VERSION	1001
#define ELEMENT_COUNT(X) (sizeof(X) / sizeof(X[0]))

//SyruS CQArray-Sorting operators
bool operator > (QueueUsers & first, QueueUsers & second)
{
	return (first.sIndex.CompareNoCase(second.sIndex) > 0);
}

bool operator < (QueueUsers & first, QueueUsers & second)
{
	return (first.sIndex.CompareNoCase(second.sIndex) < 0);
}

static dwThread = 0;
static bool	WSdownloadColumnHidden[10];
static bool	WSuploadColumnHidden[5];
static bool	WSqueueColumnHidden[4];
static bool	WSsharedColumnHidden[6];
static bool	WSserverColumnHidden[10];
static bool	WSsearchColumnHidden[4];

CWebServer::CWebServer(void)
{
	m_Params.bShowUploadQueue = false;
	m_Params.bShowUploadQueueBanned = false;
	m_Params.bShowUploadQueueFriend = false;
	m_Params.bShowUploadQueueCredit = false;

	m_Params.DownloadSort = DOWN_SORT_NAME;
	m_Params.bDownloadSortReverse = false;
	m_Params.ServerSort = SERVER_SORT_NAME;
	m_Params.bServerSortReverse = false;
	m_Params.SharedSort = SHARED_SORT_NAME;
	m_Params.bSharedSortReverse = false;

	m_Params.sLastModified = _T("");
	m_Params.sETag = _T("");
	m_iSearchSortby=3;
	m_bSearchAsc=0;

	dwThread = GetCurrentThreadId();
	m_bServerWorking = false;

	m_nIntruderDetect = 0;
	m_nStartTempDisabledTime = 0;
	m_bIsTempDisabled = false;
}

void CWebServer::ReloadTemplates()
{
	EMULE_TRY

	CString sPrevLocale(setlocale(LC_TIME, NULL));
	_tsetlocale(LC_TIME, _T("English"));
	CTime t = CTime::GetCurrentTime();
	m_Params.sLastModified = t.FormatGmt("%a, %d %b %Y %H:%M:%S GMT");
	m_Params.sETag = MD5Sum(m_Params.sLastModified).GetHash();
	_tsetlocale(LC_TIME, sPrevLocale);

	CString sFile;
	if (theApp.glob_prefs->GetTemplate()=="" || theApp.glob_prefs->GetTemplate().MakeLower()=="emule.tmpl")
		sFile= theApp.glob_prefs->GetAppDir() + CString("eMule.tmpl");
	else sFile=theApp.glob_prefs->GetTemplate();

	CStdioFile file;
	if(file.Open(sFile, CFile::modeRead|CFile::typeText))
	{
		CString sAll;
		for(;;)
		{
			CString sLine;
			if(!file.ReadString(sLine))
				break;

			sAll += sLine + _T("\n");
		}
		file.Close();

		CString sVersion = _LoadTemplate(sAll,_T("TMPL_VERSION"));
		long lVersion = _tstol(sVersion);
		if(lVersion < WEB_SERVER_TEMPLATES_VERSION)
		{
			if(theApp.glob_prefs->GetWSIsEnabled() || m_bServerWorking)
				AddLogLine(true,IDS_WS_ERR_LOADTEMPLATE,sFile);
			if (m_bServerWorking)
				StopSockets(); 
            m_bServerWorking = false; 
			theApp.glob_prefs->SetWSIsEnabled(false);
		}
		else
		{
			m_Templates.sHeader = _LoadTemplate(sAll,_T("TMPL_HEADER"));
			m_Templates.sHeaderMetaRefresh = _LoadTemplate(sAll,_T("TMPL_HEADER_META_REFRESH"));
			m_Templates.sHeaderStylesheet = _LoadTemplate(sAll,_T("TMPL_HEADER_STYLESHEET"));
			m_Templates.sFooter = _LoadTemplate(sAll,_T("TMPL_FOOTER"));
			m_Templates.sServerList = _LoadTemplate(sAll,_T("TMPL_SERVER_LIST"));
			m_Templates.sServerLine = _LoadTemplate(sAll,_T("TMPL_SERVER_LINE"));
			m_Templates.sTransferImages = _LoadTemplate(sAll,_T("TMPL_TRANSFER_IMAGES"));
			m_Templates.sTransferList = _LoadTemplate(sAll,_T("TMPL_TRANSFER_LIST"));
			m_Templates.sTransferDownHeader = _LoadTemplate(sAll,_T("TMPL_TRANSFER_DOWN_HEADER"));
			m_Templates.sTransferDownFooter = _LoadTemplate(sAll,_T("TMPL_TRANSFER_DOWN_FOOTER"));
			m_Templates.sTransferDownLine = _LoadTemplate(sAll,_T("TMPL_TRANSFER_DOWN_LINE"));
			m_Templates.sTransferUpHeader = _LoadTemplate(sAll,_T("TMPL_TRANSFER_UP_HEADER"));
			m_Templates.sTransferUpFooter = _LoadTemplate(sAll,_T("TMPL_TRANSFER_UP_FOOTER"));
			m_Templates.sTransferUpLine = _LoadTemplate(sAll,_T("TMPL_TRANSFER_UP_LINE"));
			m_Templates.sTransferUpQueueShow = _LoadTemplate(sAll,_T("TMPL_TRANSFER_UP_QUEUE_SHOW"));
			m_Templates.sTransferUpQueueHide = _LoadTemplate(sAll,_T("TMPL_TRANSFER_UP_QUEUE_HIDE"));
			m_Templates.sTransferUpQueueLine = _LoadTemplate(sAll,_T("TMPL_TRANSFER_UP_QUEUE_LINE"));
			m_Templates.sTransferUpQueueBannedShow = _LoadTemplate(sAll,_T("TMPL_TRANSFER_UP_QUEUE_BANNED_SHOW"));
			m_Templates.sTransferUpQueueBannedHide = _LoadTemplate(sAll,_T("TMPL_TRANSFER_UP_QUEUE_BANNED_HIDE"));
			m_Templates.sTransferUpQueueBannedLine = _LoadTemplate(sAll,_T("TMPL_TRANSFER_UP_QUEUE_BANNED_LINE"));
			m_Templates.sTransferUpQueueFriendShow = _LoadTemplate(sAll,_T("TMPL_TRANSFER_UP_QUEUE_FRIEND_SHOW"));
			m_Templates.sTransferUpQueueFriendHide = _LoadTemplate(sAll,_T("TMPL_TRANSFER_UP_QUEUE_FRIEND_HIDE"));
			m_Templates.sTransferUpQueueFriendLine = _LoadTemplate(sAll,_T("TMPL_TRANSFER_UP_QUEUE_FRIEND_LINE"));
			m_Templates.sTransferUpQueueCreditShow = _LoadTemplate(sAll,_T("TMPL_TRANSFER_UP_QUEUE_CREDIT_SHOW"));
			m_Templates.sTransferUpQueueCreditHide = _LoadTemplate(sAll,_T("TMPL_TRANSFER_UP_QUEUE_CREDIT_HIDE"));
			m_Templates.sTransferUpQueueCreditLine = _LoadTemplate(sAll,_T("TMPL_TRANSFER_UP_QUEUE_CREDIT_LINE"));
			m_Templates.sTransferBadLink = _LoadTemplate(sAll,_T("TMPL_TRANSFER_BAD_LINK"));
			m_Templates.sSharedList = _LoadTemplate(sAll,_T("TMPL_SHARED_LIST"));
			m_Templates.sSharedLine = _LoadTemplate(sAll,_T("TMPL_SHARED_LINE"));
			m_Templates.sGraphs = _LoadTemplate(sAll,_T("TMPL_GRAPHS"));
			m_Templates.sLog = _LoadTemplate(sAll,_T("TMPL_LOG"));
			m_Templates.sServerInfo = _LoadTemplate(sAll,_T("TMPL_SERVERINFO"));
			m_Templates.sDebugLog = _LoadTemplate(sAll,_T("TMPL_DEBUGLOG"));
			m_Templates.sStats = _LoadTemplate(sAll,_T("TMPL_STATS"));
			m_Templates.sPreferences = _LoadTemplate(sAll,_T("TMPL_PREFERENCES"));
			m_Templates.sLogin = _LoadTemplate(sAll,_T("TMPL_LOGIN"));
			m_Templates.sAddServerBox = _LoadTemplate(sAll,_T("TMPL_ADDSERVERBOX"));
			m_Templates.sSearch = _LoadTemplate(sAll,_T("TMPL_SEARCH"));
			m_Templates.iProgressbarWidth = _tstoi(_LoadTemplate(sAll,_T("PROGRESSBARWIDTH")));
			m_Templates.sSearchHeader = _LoadTemplate(sAll,_T("TMPL_SEARCH_RESULT_HEADER"));			
			m_Templates.sSearchResultLine = _LoadTemplate(sAll,_T("TMPL_SEARCH_RESULT_LINE"));			
			m_Templates.sProgressbarImgs = _LoadTemplate(sAll,_T("PROGRESSBARIMGS"));
			m_Templates.sProgressbarImgsPercent = _LoadTemplate(sAll,_T("PROGRESSBARPERCENTIMG"));

			m_Templates.sProgressbarImgsPercent.Replace(_T("[PROGRESSGIFNAME]"),_T("%s"));
			m_Templates.sProgressbarImgsPercent.Replace(_T("[PROGRESSGIFINTERNAL]"),_T("%i"));
			m_Templates.sProgressbarImgs.Replace(_T("[PROGRESSGIFNAME]"),_T("%s"));
			m_Templates.sProgressbarImgs.Replace(_T("[PROGRESSGIFINTERNAL]"),_T("%i"));
		}
	}
	else
        if(m_bServerWorking)
		{
			AddLogLine(true,IDS_WEB_ERR_CANTLOAD, sFile);
			StopSockets(); 
            m_bServerWorking = false; 
			theApp.glob_prefs->SetWSIsEnabled(false);
		}

	EMULE_CATCH
}

CWebServer::~CWebServer(void)
{
	if (m_bServerWorking) StopSockets();
}

CString CWebServer::_LoadTemplate(CString sAll, CString sTemplateName)
{
	EMULE_TRY

	CString buffer;
	CString fullpath;
	fullpath.Format(_T("%spreferences.ini"),theApp.glob_prefs->GetConfigDir());
	CIni ini( fullpath );
	ini.GetArray(WSdownloadColumnHidden, ELEMENT_COUNT(WSdownloadColumnHidden), _T("downloadColumnHidden"), _T("WebServer")); 
	ini.GetArray(WSuploadColumnHidden, ELEMENT_COUNT(WSuploadColumnHidden), _T("uploadColumnHidden"), _T("WebServer")); 
	ini.GetArray(WSqueueColumnHidden, ELEMENT_COUNT(WSqueueColumnHidden), _T("queueColumnHidden"), _T("WebServer")); 
	ini.GetArray(WSsearchColumnHidden, ELEMENT_COUNT(WSsearchColumnHidden), _T("searchColumnHidden"), _T("WebServer")); 
	ini.GetArray(WSsharedColumnHidden, ELEMENT_COUNT(WSsharedColumnHidden), _T("sharedColumnHidden"), _T("WebServer")); 
	ini.GetArray(WSserverColumnHidden, ELEMENT_COUNT(WSserverColumnHidden), _T("serverColumnHidden"), _T("WebServer")); 
	ini.SaveAndClose(); 
	CString sRet = _T("");
	int nStart = sAll.Find(_T("<--") + sTemplateName + _T("-->"));
	int nEnd = sAll.Find(_T("<--") + sTemplateName + _T("_END-->"));
	if(nStart != -1 && nEnd != -1 && nStart<nEnd) //(0.29b)
	{
		nStart += sTemplateName.GetLength() + 7;
		sRet = sAll.Mid(nStart, nEnd - nStart - 1);
	} else	{
		if (sTemplateName=="TMPL_VERSION")
			AddLogLine(true,IDS_WS_ERR_LOADTEMPLATE,sTemplateName);
		if (nStart==-1) 
			AddDebugLogLine(false,IDS_WEB_ERR_CANTLOAD,sTemplateName);
	}
	return sRet;

	EMULE_CATCH

	return _T("");
}

void CWebServer::RestartServer() {	//Cax2 - restarts the server with the new port settings
	StopSockets();
	if (m_bServerWorking)	
		StartSockets(this);
}

void CWebServer::StartServer(void)
{
	EMULE_TRY
	if(m_bServerWorking != theApp.glob_prefs->GetWSIsEnabled())
		m_bServerWorking = theApp.glob_prefs->GetWSIsEnabled();
	else
		return;

	if (m_bServerWorking) 
	{
		ReloadTemplates();
		StartSockets(this); 
		m_nIntruderDetect = 0;
	    m_nStartTempDisabledTime = 0;
	    m_bIsTempDisabled = false;
	} else StopSockets();

	if(theApp.glob_prefs->GetWSIsEnabled() && m_bServerWorking)
		AddLogLine(false,_T("%s: %s"),_GetPlainResString(IDS_PW_HTTPD), _GetPlainResString(IDS_ENABLED).MakeLower());
	else
		AddLogLine(false,_T("%s: %s"),_GetPlainResString(IDS_PW_HTTPD), _GetPlainResString(IDS_DISABLED).MakeLower());

	EMULE_CATCH
}

void CWebServer::_RemoveServer(CString sIP, int nPort)
{
	EMULE_TRY

	CServer* server=theApp.serverlist->GetServerByAddress(sIP.GetBuffer(), nPort);
	if (server!=NULL) theApp.emuledlg->SendMessage(WEB_REMOVE_SERVER, (WPARAM)server, NULL);

	EMULE_CATCH
}

void CWebServer::_AddToStatic(CString sIP, int nPort)
{
	EMULE_TRY

	CServer* server=theApp.serverlist->GetServerByAddress(sIP.GetBuffer(), nPort);
	if (server!=NULL) theApp.emuledlg->SendMessage(WEB_ADD_TO_STATIC, (WPARAM)server, NULL);

	EMULE_CATCH
}

void CWebServer::_RemoveFromStatic(CString sIP, int nPort)
{
	EMULE_TRY

	CServer* server=theApp.serverlist->GetServerByAddress(sIP.GetBuffer(), nPort);
	if (server!=NULL) theApp.emuledlg->SendMessage(WEB_REMOVE_FROM_STATIC, (WPARAM)server, NULL);

	EMULE_CATCH
}

//+++
void CWebServer::_SetSharedFileJumpstart(CString hash)
{
	EMULE_TRY

	CKnownFile* cur_file;
	uchar fileid[16];
	DecodeBase16(hash.GetBuffer(),hash.GetLength(),fileid);

	cur_file=theApp.sharedfiles->GetFileByID(fileid);
	
	if (cur_file==0) return;

#ifdef OLD_SOCKETS_ENABLED
	cur_file->SetJumpstartEnabled(true);
#endif //OLD_SOCKETS_ENABLED

	theApp.sharedfiles->UpdateItem(cur_file);

	EMULE_CATCH
}

//+++
void CWebServer::_SetSharedFileNoJumpstart(CString hash)
{
	EMULE_TRY

	CKnownFile* cur_file;
	uchar fileid[16];
	DecodeBase16(hash.GetBuffer(),hash.GetLength(),fileid);

	cur_file=theApp.sharedfiles->GetFileByID(fileid);
	
	if (cur_file==0) return;

#ifdef OLD_SOCKETS_ENABLED
	cur_file->SetJumpstartEnabled(false);
#endif //OLD_SOCKETS_ENABLED

	theApp.sharedfiles->UpdateItem(cur_file);

	EMULE_CATCH
}

void CWebServer::AddStatsLine(UpDown line)
{
	EMULE_TRY

	m_Params.PointsForWeb.Add(line);
	if(m_Params.PointsForWeb.GetCount() > WEB_GRAPH_WIDTH)
		m_Params.PointsForWeb.RemoveAt(0);

	EMULE_CATCH
}

CString CWebServer::_SpecialChars(CString str, bool noquote /*=false*/) 
{
	EMULE_TRY

	str.Replace(_T("&"),_T("&amp;"));
	str.Replace(_T("<"),_T("&lt;"));
	str.Replace(_T(">"),_T("&gt;"));
	str.Replace(_T("\""),_T("&quot;"));
	if(noquote)
	{
		str.Replace(_T("'"), _T("&#8217;"));
		str.Replace(_T("\n"), _T("\\n"));
	}
	return str;

	EMULE_CATCH

	return _T("");
}

void CWebServer::_ConnectToServer(CString sIP, int nPort)
{
	EMULE_TRY

	CServer* server=theApp.serverlist->GetServerByAddress(sIP.GetBuffer(), nPort);
	if (server!=NULL) theApp.emuledlg->SendMessage(WEB_CONNECT_TO_SERVER, (WPARAM)server, NULL);

	EMULE_CATCH
}

void CWebServer::ProcessImgFileReq(ThreadData Data) {
	EMULE_TRY

	CWebServer *pThis = (CWebServer *)Data.pThis;
	if(pThis == NULL)
		return;

	CString filename=Data.sURL;
	CString contenttype;

	if (filename.Right(4).MakeLower()==".gif") contenttype="Content-Type: image/gif\r\n";
	else if (filename.Right(4).MakeLower()==".jpg"  || filename.Right(5).MakeLower()==".jpeg") contenttype="Content-Type: image/jpg\r\n";
	else if (filename.Right(4).MakeLower()==".bmp") contenttype="Content-Type: image/bmp\r\n";
	else if (filename.Right(4).MakeLower()==".png") contenttype="Content-Type: image/png\r\n";
	//(0.29b) //DonQ - additional filetypes
	else if (filename.Right(4).MakeLower()==".ico") contenttype="Content-Type: image/x-icon\r\n";
	else if (filename.Right(4).MakeLower()==".css") contenttype="Content-Type: text/css\r\n";
	else if (filename.Right(3).MakeLower()==".js") contenttype="Content-Type: text/javascript\r\n";

	contenttype += _T("Last-Modified: ") + pThis->m_Params.sLastModified + _T("\r\n") + _T("ETag: ") + pThis->m_Params.sETag + _T("\r\n");

	// DonGato: expire images after 1 year (overridable by browser)
	CString sPrevLocale(setlocale(LC_TIME, NULL));
	_tsetlocale(LC_TIME, _T("English"));
	CTime t = CTime::GetCurrentTime();
	t += 60*60*24*365;
	CString expires = t.FormatGmt("%a, %d %b %Y %H:%M:%S GMT");
	_tsetlocale(LC_TIME, sPrevLocale);
	contenttype += _T("Expires: ") + expires + _T("\r\n");

	filename.Delete(0);
	filename.Replace('/','\\');
	filename=CString(theApp.glob_prefs->GetAppDir())+_T("webserver\\")+filename;
	CFile file;
	if(file.Open(filename, CFile::modeRead|CFile::shareDenyWrite|CFile::typeBinary)) //(0.29b)
	{
		TCHAR* buffer=new TCHAR[file.GetLength()];
		int size=file.Read(buffer,file.GetLength());
		file.Close();
		Data.pSocket->SendContent(contenttype, buffer, size);
		delete[] buffer;
	}

	EMULE_CATCH
}

void CWebServer::ProcessURL(ThreadData Data)
{
	EMULE_TRY

	CWebServer *pThis = (CWebServer *)Data.pThis;
	if(pThis == NULL)
		return;

	SetThreadLocale(theApp.glob_prefs->GetLanguageID());

	//(0.29b)//////////////////////////////////////////////////////////////////
	// Here we are in real trouble! We are accessing the entire emule main thread
	// data without any syncronization!! Either we use the message pump for emuledlg
	// or use some hundreds of critical sections... For now, an exception handler
	// shoul avoid the worse things.
	//////////////////////////////////////////////////////////////////////////
	CoInitialize(NULL);

	EMULE_TRY	//(0.29b)
	bool isUseGzip = theApp.glob_prefs->GetWebUseGzip();
	CString Out = _T("");
	CString OutE = _T("");	// List Entry Templates
	CString OutS = _T("");	// ServerStatus Templates
	TCHAR *gzipOut = NULL;
	long gzipLen=0;
	
	CString HTTPProcessData = _T("");
	CString HTTPTemp = _T("");
	srand ( time(NULL) );
	DWORD dwCurTick = ::GetTickCount();

	long lSession = 0;
	if(_ParseURL(Data.sURL, _T("ses")) != "")
		lSession = _tstol(_ParseURL(Data.sURL, _T("ses")));

	//Added by Johnny-B -IntruderDetection
	if(pThis->GetIsTempDisabled() &&
	((dwCurTick - pThis->m_nStartTempDisabledTime) > (theApp.glob_prefs->GetWSTempDisableLogin() * 60000)))
	{
        pThis->m_bIsTempDisabled = false;
        pThis->m_nStartTempDisabledTime = 0;
		AddLogLine(true,IDS_WEB_INTRBLOCKEND);
	}//End Johnny-B

	if ((_ParseURL(Data.sURL, _T("w")) == "password")
	&& (!pThis->GetIsTempDisabled()))
	{
		CString test=MD5Sum(_ParseURL(Data.sURL, _T("p"))).GetHash();
		CString t_ulCurIP;//log intruder
		t_ulCurIP.Format(_T("%i.%i.%i.%i"),(uint8)pThis->m_ulCurIP,(uint8)(pThis->m_ulCurIP>>8),(uint8)(pThis->m_ulCurIP>>16),(uint8)(pThis->m_ulCurIP>>24));//log intruder

		if(MD5Sum(_ParseURL(Data.sURL, _T("p"))).GetHash() == theApp.glob_prefs->GetWSPass())
		{
			Session ses;
			ses.admin=true;
			ses.startTime = CTime::GetCurrentTime();
			ses.lSession = lSession = rand() * 10000L + rand();
			pThis->m_Params.Sessions.Add(ses);
			AddLogLine(true,IDS_WEB_ADMINLOGIN,t_ulCurIP);
			pThis->m_nIntruderDetect = 0;
		}
		else if(theApp.glob_prefs->GetWSIsLowUserEnabled() && theApp.glob_prefs->GetWSLowPass()!="" && MD5Sum(_ParseURL(Data.sURL, _T("p"))).GetHash() == theApp.glob_prefs->GetWSLowPass())
		{
			Session ses;
			ses.admin=false;
			ses.startTime = CTime::GetCurrentTime();
			ses.lSession = lSession = rand() * 10000L + rand();
			pThis->m_Params.Sessions.Add(ses);
			AddLogLine(true,IDS_WEB_GUESTLOGIN,t_ulCurIP);
			pThis->m_nIntruderDetect = 0;
		} else if(theApp.glob_prefs->IsWSIntruderDetectionEnabled())
			{
				pThis->m_nIntruderDetect++;
				AddLogLine(true,IDS_WEB_INTRTRIESLEFT,t_ulCurIP,theApp.glob_prefs->GetWSLoginAttemptsAllowed()-(pThis->m_nIntruderDetect));
				if (pThis->m_nIntruderDetect == theApp.glob_prefs->GetWSLoginAttemptsAllowed()){
					AddLogLine(true,IDS_WEB_INTRDETECTION, pThis->m_nIntruderDetect);
				if (theApp.glob_prefs->GetWSTempDisableLogin() > 0)
				{
					pThis->m_nStartTempDisabledTime = ::GetTickCount();
					pThis->m_bIsTempDisabled = true;
					pThis->AddLogLine(true,IDS_WEB_INTRBLOCKTIME, theApp.glob_prefs->GetWSTempDisableLogin());
					pThis->m_nIntruderDetect = 0;
					// DonGato - Sending message for WebServer intrussion (Login temporally disabled)
					CString MessageText;
					MessageText.Format(GetResString(IDS_WEB_INTRBLOCKTIME), theApp.glob_prefs->GetWSTempDisableLogin());
					theApp.smtpconnection->SendMail(MessageText, theApp.glob_prefs->GetNotifierPopOnWebServerError(), theApp.glob_prefs->IsSMTPWarningEnabled());
	                theApp.emuledlg->ShowNotifier(MessageText, TBN_WEBSERVER, false, theApp.glob_prefs->GetNotifierPopOnWebServerError());
				} else {
					theApp.glob_prefs->SetWSIsEnabled(false);
		            pThis->m_bServerWorking = false; 
					StopSockets();
					AddLogLine(true,IDS_WEB_INTRWEBDISABLE);
					pThis->m_nIntruderDetect = 0;
					// Purity - Sending message for WebServer intrussion (WebServer disabled)
					CString MessageText;
					MessageText.Format(GetResString(IDS_WEB_INTRWEBDISABLE));
					theApp.smtpconnection->SendMail(MessageText, theApp.glob_prefs->GetNotifierPopOnWebServerError(), theApp.glob_prefs->IsSMTPWarningEnabled());
	                theApp.emuledlg->ShowNotifier(MessageText, TBN_WEBSERVER, false, theApp.glob_prefs->GetNotifierPopOnWebServerError());
				}
			}
		}else{//End Johnny-B -IntruderDetection
			AddLogLine(true,IDS_WEB_BADLOGINATTEMPT);
		}
		isUseGzip = false; // [Julien]
	}

	CString sSession; sSession.Format(_T("%ld"), lSession);

	if (_ParseURL(Data.sURL, _T("w")) == "logout") 
		_RemoveSession(Data, lSession);
	else if (_ParseURL(Data.sURL, _T("w")) == "close") {
		//_RemoveSession(Data, lSession);
		theApp.m_app_state = theApp.APP_STATE_SHUTINGDOWN;
		SendMessage(theApp.emuledlg->m_hWnd,WM_CLOSE,0,0);
	} else if (_ParseURL(Data.sURL, _T("w")) == "shutdown") {
		//_RemoveSession(Data, lSession);
		HANDLE hToken; 
		TOKEN_PRIVILEGES tkp;	// Get a token for this process.
		try {
			if (!OpenProcessToken(GetCurrentProcess(), TOKEN_ADJUST_PRIVILEGES | TOKEN_QUERY, &hToken)) 
				throw; 
			LookupPrivilegeValue(NULL, SE_SHUTDOWN_NAME, &tkp.Privileges[0].Luid); 
			tkp.PrivilegeCount = 1;  // one privilege to set    
			tkp.Privileges[0].Attributes = SE_PRIVILEGE_ENABLED;	// Get the shutdown privilege for this process.
			AdjustTokenPrivileges(hToken, FALSE, &tkp, 0, (PTOKEN_PRIVILEGES)NULL, 0); 
			ExitWindowsEx(EWX_SHUTDOWN | EWX_FORCE, 0);
		}
		catch(...) {
			AddLogLine(true,IDS_WEB_SHUTDOWN+" "+IDS_FAILED);
		}
	}

	if(_IsLoggedIn(Data, lSession))
	{
		Out += _GetHeader(Data, lSession);
		CString sPage = _ParseURL(Data.sURL, _T("w"));
		if (sPage == "server") 
		{
			Out += _GetServerList(Data);
		}
		else
		if (sPage == "shared") 
		{ 
			Out += _GetSharedFilesList(Data);
		}
		else
		if (sPage == "transfer") 
		{
			Out += _GetTransferList(Data);
		}
		else
		if (sPage == "search") 
		{
			Out += _GetSearch(Data);
		}
		else
		if (sPage == "graphs")
		{
			Out += _GetGraphs(Data);
		}
		else
		if (sPage == "log") 
		{
			Out += _GetLog(Data);
		}
		if (sPage == "sinfo") 
		{
			Out += _GetServerInfo(Data);
		}
		if (sPage == "debuglog") 
		{
			Out += _GetDebugLog(Data);
		}
		if (sPage == "stats") 
		{
			Out += _GetStats(Data);
		}
		if (sPage == "options") 
		{
			isUseGzip = false;
			Out += _GetPreferences(Data);
		}
		Out += _GetFooter(Data);

		if (sPage == "")
			isUseGzip = false;

		if(isUseGzip)
		{
			bool bOk = false;
			try
			{
				uLongf destLen = Out.GetLength() + 1024;
				gzipOut = new TCHAR[destLen];
				if(_GzipCompress((Bytef*)gzipOut, &destLen, (const Bytef*)(TCHAR*)Out.GetBuffer(0), Out.GetLength(), Z_DEFAULT_COMPRESSION) == Z_OK)
				{
					bOk = true;
					gzipLen = destLen;
				}
			}
			catch(...)
			{
			}
			if(!bOk)
			{
				isUseGzip = false;
				if(gzipOut != NULL)
				{
					delete[] gzipOut;
					gzipOut = NULL;
				}
			}
		}
	}
	else 
	{
		isUseGzip = false;
		Out += _GetLoginScreen(Data);
	}

	// send answer ...
	if(!isUseGzip)
	{
		Data.pSocket->SendContent(HTTPInit, Out, Out.GetLength());
	}
	else
	{
		Data.pSocket->SendContent(HTTPInitGZ, gzipOut, gzipLen);
	}
	if(gzipOut != NULL)
		delete[] gzipOut;

	EMULE_CATCH
	CoUninitialize(); //(0.29b)

	EMULE_CATCH
}

CString CWebServer::_ParseURLArray(CString URL, CString fieldname) {
	EMULE_TRY

	CString res,temp;

	while (URL.GetLength()>0) {
		int pos=URL.MakeLower().Find(fieldname.MakeLower() + _T("="));
		if (pos>-1) {
			temp=_ParseURL(URL,fieldname);
			if (temp=="") break;
			res.Append(temp + _T("|"));
			URL.Delete(pos,10);
		} else break;
	}
	return res;

	EMULE_CATCH
	return _T("");
}

CString CWebServer::_ParseURL(CString URL, CString fieldname)
{
	EMULE_TRY

	CString value = _T("");
	CString Parameter = _T("");
	TCHAR fromReplace[4] = _T("");	// decode URL
	TCHAR toReplace[2] = _T("");	// decode URL
	int i = 0;
	int findPos = -1;
	int findLength = 0;

	if (URL.Find(_T("?")) > -1) {
		Parameter = URL.Mid(URL.Find(_T("?"))+1, URL.GetLength()-URL.Find(_T("?"))-1);

		// search the fieldname beginning / middle and strip the rest...
		if (Parameter.Find(fieldname + _T("=")) == 0) {
			findPos = 0;
			findLength = fieldname.GetLength() + 1;
		}
		if (Parameter.Find(_T("&") + fieldname + _T("=")) > -1) {
			findPos = Parameter.Find(_T("&") + fieldname + _T("="));
			findLength = fieldname.GetLength() + 2;
		}
		if (findPos > -1) {
			Parameter = Parameter.Mid(findPos + findLength, Parameter.GetLength());
			if (Parameter.Find(_T("&")) > -1) {
				Parameter = Parameter.Mid(0, Parameter.Find(_T("&")));
			}
	
			value = Parameter;

			// decode value ...
			value.Replace(_T("+"), _T(" "));
			for (i = 0 ; i <= 255 ; i++) {
				_stprintf(fromReplace, _T("%%%02x"), i);
				toReplace[0] = (char)i;
				toReplace[1] = NULL;
				value.Replace(fromReplace, toReplace);
				_stprintf(fromReplace, _T("%%%02X"), i);
				value.Replace(fromReplace, toReplace);
			}
		}
	}

	return value;

	EMULE_CATCH

	return _T("");
}

CString CWebServer::_GetHeader(ThreadData Data, long lSession)
{
	EMULE_TRY

	CWebServer *pThis = (CWebServer *)Data.pThis;
	if(pThis == NULL)
		return _T("");

	CString sSession; sSession.Format(_T("%ld"), lSession);

	CString Out = pThis->m_Templates.sHeader;

	Out.Replace(_T("[CharSet]"), _GetWebCharSet());

	if(theApp.glob_prefs->GetWebPageRefresh() != 0)
	{
		CString sPage = _ParseURL(Data.sURL, _T("w"));
		if ((sPage == "transfer") ||
			(sPage == "server") ||
			(sPage == "graphs") ||
			(sPage == "log") ||
			(sPage == "sinfo") ||
			(sPage == "debuglog") ||
			(sPage == "stats"))
		{
			CString sT = pThis->m_Templates.sHeaderMetaRefresh;
			CString sRefresh; sRefresh.Format(_T("%d"), theApp.glob_prefs->GetWebPageRefresh());
			sT.Replace(_T("[RefreshVal]"), sRefresh);
			CString catadd="";
			if (sPage == "transfer")
				catadd="&cat="+_ParseURL(Data.sURL, "cat");
			sT.Replace(_T("[wCommand]"), _ParseURL(Data.sURL, _T("w"))+catadd);
			Out.Replace(_T("[HeaderMeta]"), sT);
		}
	}

	bool bAdmin = IsSessionAdmin(Data,sSession);
	CString admin;
	if (bAdmin)
		admin.Format(_T("admin"));

	int cat=atoi(_ParseURL(Data.sURL,"cat"));

	CString sCat; (cat!=0)?sCat.Format("&cat=%i",cat):sCat="";

	Out.Replace(_T("[admin]"), admin);
	Out.Replace(_T("[Session]"), sSession);
	Out.Replace(_T("[HeaderMeta]"), _T("")); // In case there are no meta
	Out.Replace(_T("[eMuleAppName]"), _GetPlainResString(IDS_EMULE_PLUS));
	Out.Replace(_T("[version]"), CURRENT_VERSION_LONG);
	Out.Replace(_T("[StyleSheet]"), pThis->m_Templates.sHeaderStylesheet);
	Out.Replace(_T("[WebControl]"), _GetPlainResString(IDS_WEB_CONTROL));
	Out.Replace(_T("[Transfer]"), _GetPlainResString(IDS_CD_TRANS));
	Out.Replace(_T("[Server]"), _GetPlainResString(IDS_SV_SERVERLIST));
	Out.Replace(_T("[Shared]"), _GetPlainResString(IDS_SHAREDFILES));
	Out.Replace(_T("[Graphs]"), _GetPlainResString(IDS_GRAPHS));
	Out.Replace(_T("[Log]"), _GetPlainResString(IDS_SV_LOG));
	Out.Replace(_T("[ServerInfo]"), _GetPlainResString(IDS_SV_SERVERINFO));
	Out.Replace(_T("[DebugLog]"), _GetPlainResString(IDS_SV_DEBUGLOG));
	Out.Replace(_T("[Stats]"), _GetPlainResString(IDS_SF_STATISTICS));
	Out.Replace(_T("[Options]"), _GetPlainResString(IDS_EM_PREFS));
	Out.Replace(_T("[Ed2klink]"), _GetPlainResString(IDS_SW_LINK));
	Out.Replace(_T("[Close]"), _GetPlainResString(IDS_WEB_CLOSE));
	Out.Replace(_T("[Shutdown]"), _GetPlainResString(IDS_WEB_SHUTDOWN));
	Out.Replace(_T("[Logout]"), _GetPlainResString(IDS_WEB_LOGOUT));
	Out.Replace(_T("[WebOptions]"), _GetPlainResString(IDS_WEB_OPTIONS));
	Out.Replace(_T("[Search]"), _GetPlainResString(IDS_SW_SEARCHBOX));
	Out.Replace(_T("[Download]"), _GetPlainResString(IDS_SW_DOWNLOAD));
	Out.Replace(_T("[Start]"), _GetPlainResString(IDS_SW_START));
	InsertCatBox(Out,0,"",false,false,sSession,"");

	Out.Replace(_T("[FileIsHashing]"), _GetPlainResString(IDS_HASHING, true));
	Out.Replace(_T("[FileIsErroneous]"), _GetPlainResString(IDS_ERRORLIKE, true));
	Out.Replace(_T("[FileIsCompleting]"), _GetPlainResString(IDS_COMPLETING, true));
	Out.Replace(_T("[FileDetails]"), _GetPlainResString(IDS_FD_TITLE, true));
	Out.Replace(_T("[ClearCompleted]"), _GetPlainResString(IDS_DL_CLEAR, true));
	Out.Replace(_T("[A4AF]"), _GetPlainResString(IDS_ALL_A4AF_TO_HERE, true));
	Out.Replace(_T("[AUTOA4AF]"), _GetPlainResString(IDS_ALL_A4AF_TO_HERE, true) + " (" + _GetPlainResString(IDS_PRIOAUTO, true) + ")"); // Purity: Auto-A4AF
	Out.Replace(_T("[DropSCB]"), _GetPlainResString(IDS_DROPSUPERCOMPRESSED, true));
	Out.Replace(_T("[NoNeededSources]"), _GetPlainResString(IDS_DROPNONEEDEDSRCS, true));
	Out.Replace(_T("[Resume]"), _GetPlainResString(IDS_DL_RESUME, true));
	Out.Replace(_T("[Stop]"), _GetPlainResString(IDS_DL_STOP, true));
	Out.Replace(_T("[ConfirmCancel]"), _GetPlainResString(IDS_Q_CANCELDL2, true));
	Out.Replace(_T("[Cancel]"), _GetPlainResString(IDS_MAIN_BTN_CANCEL, true));
	Out.Replace(_T("[Connect]"), _GetPlainResString(IDS_IRC_CONNECT, true));
	Out.Replace(_T("[ConfirmRemove]"), _GetPlainResString(IDS_WEB_CONFIRM_REMOVE_SERVER, true));
	Out.Replace(_T("[ConfirmClose]"), _GetPlainResString(IDS_MAIN_CLOSE, true));
	Out.Replace(_T("[ConfirmShutdown]"), _GetPlainResString(IDS_MAIN_SHUTDOWN, true));
	Out.Replace(_T("[RemoveServer]"), _GetPlainResString(IDS_REMOVETHIS, true));
	Out.Replace(_T("[StaticServer]"), _GetPlainResString(IDS_STATICSERVER, true));
	Out.Replace(_T("[ConfirmJumpstart]"), _GetPlainResString(IDS_JS_DISABLE, true) + _T("\\n"));//+++
	Out.Replace(_T("[Jumpstart]"), _GetPlainResString(IDS_PRIOJUMPSTART, true));//+++
	Out.Replace(_T("[Friend]"), _GetPlainResString(IDS_FRIEND, true));
	Out.Replace(_T("[CatSel]"), sCat);

	Out.Replace(_T("[PriorityVeryLow]"), _GetPlainResString(IDS_PRIOVERYLOW, true));
	Out.Replace(_T("[PriorityLow]"), _GetPlainResString(IDS_PRIOLOW, true));
	Out.Replace(_T("[PriorityNormal]"), _GetPlainResString(IDS_PRIONORMAL, true));
	Out.Replace(_T("[PriorityHigh]"), _GetPlainResString(IDS_PRIOHIGH, true));
	Out.Replace(_T("[PriorityRelease]"), _GetPlainResString(IDS_PRIORELEASE, true));
	Out.Replace(_T("[PriorityAuto]"), _GetPlainResString(IDS_PRIOAUTO, true));

	CString HTTPConState = _T("");
	CString HTTPConText = _T("");
	CString HTTPHelp = _T("");
	CString HTTPHelpU = _T("0");
	CString HTTPHelpM = _T("0");
	CString HTTPHelpV = _T("0");
	CString HTTPHelpF = _T("0");
	TCHAR HTTPHeader[100] = _T("");
#ifdef OLD_SOCKETS_ENABLED

	CString sCmd = _ParseURL(Data.sURL, _T("c"));
	bool disconnectissued = (sCmd.CompareNoCase(_T("disconnect")) == 0);
	bool connectissued = (sCmd.CompareNoCase(_T("connect")) == 0);

	if ((theApp.serverconnect->IsConnecting() && !disconnectissued) || connectissued) { 
		HTTPConState = "connecting";
		HTTPConText = _GetPlainResString(IDS_CONNECTING);
	}
	else if (theApp.serverconnect->IsConnected() && !disconnectissued) 
	{
		if(!theApp.serverconnect->IsLowID())
			HTTPConState = "high";
		else
			HTTPConState = "low";
		CServer* cur_server = theApp.serverconnect->GetCurrentServer();
		if(cur_server->GetListName().GetLength() > SHORT_LENGTH)
			HTTPConText = CString(cur_server->GetListName().Left(SHORT_LENGTH) + _T("..."));
		else
			HTTPConText = CString(cur_server->GetListName());
		_stprintf(HTTPHeader, _T("%i"), cur_server->GetUsers());
		HTTPHelpU = CString(HTTPHeader);
		_stprintf(HTTPHeader, _T("%i"), cur_server->GetMaxUsers());
		HTTPHelpM = CString(HTTPHeader);
		_stprintf(HTTPHeader, _T("%i"), cur_server->GetFiles());
		HTTPHelpF = CString(HTTPHeader);
		_stprintf(HTTPHeader, _T("%.1f "), ((float)cur_server->GetUsers() / cur_server->GetMaxUsers()) * 100);
		HTTPHelpV = CString(HTTPHeader);

	} else
#endif //OLD_SOCKETS_ENABLED
	{
		HTTPConState = "disconnected";
		HTTPConText = _GetPlainResString(IDS_DISCONNECTED);
		if (IsSessionAdmin(Data,sSession)) HTTPConText+=_T(" (<a href=\"/?ses=") + sSession + _T("&w=server&c=connect\">")+_GetPlainResString(IDS_CONNECTTOANYSERVER)+_T("</a>)");
	}
	int allUsers = 0;
	int allFiles = 0;
	for (uint32 sc=0;sc<theApp.serverlist->GetServerCount();sc++)
	{
		CServer* cur_server = theApp.serverlist->GetServerAt(sc);
	    allUsers += cur_server->GetUsers();
	    allFiles += cur_server->GetFiles();
	}
	HTTPHelp.Format(_T("%i"), allUsers);
	Out.Replace(_T("[AllUsers]"), HTTPHelp);
	HTTPHelp.Format(_T("%i"), allFiles);
	Out.Replace(_T("[AllFiles]"), HTTPHelp);
	Out.Replace(_T("[ConState]"), HTTPConState);
	Out.Replace(_T("[ConText]"), HTTPConText);
	if(theApp.glob_prefs->GetMaxUpload() == UNLIMITED)
		_stprintf(HTTPHeader, "%.1f", ((float)theApp.uploadqueue->GetDatarate()/1024)/(theApp.glob_prefs->GetMaxGraphUploadRate())*100); 
	else
		_stprintf(HTTPHeader, "%.1f", ((float)theApp.uploadqueue->GetDatarate()/1024)/(theApp.glob_prefs->GetMaxUpload())*100); 
	Out.Replace(_T("[UploadValue]"), HTTPHeader);

	if(theApp.glob_prefs->GetMaxDownload() == UNLIMITED)
		_stprintf(HTTPHeader, "%.1f", ((float)theApp.downloadqueue->GetDatarate()/1024)/(theApp.glob_prefs->GetMaxGraphDownloadRate())*100);
	else
		_stprintf(HTTPHeader, "%.1f", ((float)theApp.downloadqueue->GetDatarate()/1024)/(theApp.glob_prefs->GetMaxDownload())*100);
	Out.Replace(_T("[DownloadValue]"), HTTPHeader);
#ifdef OLD_SOCKETS_ENABLED
	_stprintf(HTTPHeader, "%.1f", ((float)theApp.listensocket->GetOpenSockets())/(theApp.glob_prefs->GetMaxConnections())*100);
	Out.Replace(_T("[ConnectionValue]"), HTTPHeader);
	_stprintf(HTTPHeader, "%.1f", ((float)theApp.uploadqueue->GetDatarate()/1024)); 
	Out.Replace(_T("[CurUpload]"), HTTPHeader);
	_stprintf(HTTPHeader, "%.1f", ((float)theApp.downloadqueue->GetDatarate()/1024));
	Out.Replace(_T("[CurDownload]"), HTTPHeader);
	_stprintf(HTTPHeader, "%.0f", ((float)theApp.listensocket->GetOpenSockets()));
	Out.Replace(_T("[CurConnection]"), HTTPHeader);
#endif //OLD_SOCKETS_ENABLED
	HTTPHelp.Format(_T("%i"),theApp.glob_prefs->GetMaxUpload());
	if (HTTPHelp == "65535")  HTTPHelp = GetResString(IDS_PW_UNLIMITED);
	Out.Replace(_T("[MaxUpload]"), HTTPHelp);
	HTTPHelp.Format(_T("%i"),theApp.glob_prefs->GetMaxDownload());
	if (HTTPHelp == "65535") HTTPHelp = GetResString(IDS_PW_UNLIMITED);
	Out.Replace(_T("[MaxDownload]"), HTTPHelp);
	HTTPHelp.Format(_T("%i"),theApp.glob_prefs->GetMaxConnections());
	if (HTTPHelp == "65535") HTTPHelp = GetResString(IDS_PW_UNLIMITED);
	Out.Replace(_T("[MaxConnection]"), HTTPHelp);
	Out.Replace(_T("[UserValue]"), HTTPHelpV);
	Out.Replace(_T("[MaxUsers]"), HTTPHelpM);
	Out.Replace(_T("[CurUsers]"), HTTPHelpU);
	Out.Replace(_T("[CurFiles]"), HTTPHelpF);
	Out.Replace(_T("[Connection]"), _GetPlainResString(IDS_PW_CONNECTION));
	Out.Replace(_T("[QuickStats]"), _GetPlainResString(IDS_PW_QUICKSTATS));

	Out.Replace(_T("[Users]"), _GetPlainResString(IDS_UUSERS));
	Out.Replace(_T("[Files]"), _GetPlainResString(IDS_FILES));
	Out.Replace(_T("[Con]"), _GetPlainResString(IDS_ST_ACTIVECONNECTIONS));
	Out.Replace(_T("[Up]"), _GetPlainResString(IDS_PW_CON_UPLBL));
	Out.Replace(_T("[Down]"), _GetPlainResString(IDS_PW_CON_DOWNLBL));
	return Out;

	EMULE_CATCH

	return _T("");
}

CString CWebServer::_GetFooter(ThreadData Data)
{
	EMULE_TRY

	CWebServer *pThis = (CWebServer *)Data.pThis;
	if(pThis == NULL)
		return _T("");

	return pThis->m_Templates.sFooter;

	EMULE_CATCH

	return _T("");
}

CString CWebServer::_GetServerList(ThreadData Data)
{
	EMULE_TRY

	CWebServer *pThis = (CWebServer *)Data.pThis;
	if(pThis == NULL)
		return _T("");

	CString HTTPTemp;

	CString sSession = _ParseURL(Data.sURL, _T("ses"));
	bool bAdmin = IsSessionAdmin(Data,sSession);
	CString sAddServerBox = _GetAddServerBox(Data);

	CString sCmd = _ParseURL(Data.sURL, _T("c"));
	CString sIP = _ParseURL(Data.sURL, _T("ip"));
	int nPort = _tstoi(_ParseURL(Data.sURL, _T("port")));
	if (bAdmin &&  sCmd.CompareNoCase(_T("connect")) == 0)
	{
		if(sIP.IsEmpty())
			theApp.emuledlg->SendMessage(WM_COMMAND, MP_CONNECT);
		else
			_ConnectToServer(sIP, nPort);
	}	
	else if (bAdmin && sCmd.CompareNoCase(_T("disconnect")) == 0) 
	{
#ifdef OLD_SOCKETS_ENABLED
		if (theApp.serverconnect->IsConnecting())
		{
			theApp.serverconnect->StopConnectionTry();
			theApp.emuledlg->ShowConnectionState(false);
		}
		else
#endif //OLD_SOCKETS_ENABLED
			theApp.emuledlg->SendMessage(WM_COMMAND, MP_DISCONNECT);
	}	
	else if (bAdmin && sCmd.CompareNoCase(_T("remove")) == 0) 
	{
		if(!sIP.IsEmpty())
			_RemoveServer(sIP, nPort);
	}	
	else if (bAdmin && sCmd.CollateNoCase(_T("addtostatic")) == 0) 
	{
		if(!sIP.IsEmpty())
			_AddToStatic(sIP, nPort);
	}
	else if (bAdmin && sCmd.CompareNoCase(_T("removefromstatic")) == 0) 
	{
		if(!sIP.IsEmpty())
			_RemoveFromStatic(sIP, nPort);
	}
	else if (bAdmin && sCmd.CompareNoCase(_T("priolow")) == 0) //DonGato: priority change
	{
		if(!sIP.IsEmpty())
		{
			CServer* server=theApp.serverlist->GetServerByAddress(sIP.GetBuffer(), nPort);
			server->SetPreference((CServer::eServerPriority)PR_LOW);
			if (theApp.emuledlg->serverwnd) 
				theApp.emuledlg->serverwnd.serverlistctrl.RefreshServer(*server);
		}
	}
	else if (bAdmin && sCmd.CompareNoCase(_T("prionormal")) == 0) //DonGato: priority change
	{
		if(!sIP.IsEmpty())
		{
			CServer* server=theApp.serverlist->GetServerByAddress(sIP.GetBuffer(), nPort);
			server->SetPreference((CServer::eServerPriority)PR_NORMAL);
			if (theApp.emuledlg->serverwnd) 
				theApp.emuledlg->serverwnd.serverlistctrl.RefreshServer(*server);
		}
	}
	else if (bAdmin && sCmd.CompareNoCase(_T("priohigh")) == 0) //DonGato: priority change
	{
		if(!sIP.IsEmpty())
		{
			CServer* server=theApp.serverlist->GetServerByAddress(sIP.GetBuffer(), nPort);
			server->SetPreference((CServer::eServerPriority)PR_HIGH);
			if (theApp.emuledlg->serverwnd) 
				theApp.emuledlg->serverwnd.serverlistctrl.RefreshServer(*server);
		}
	}
	else if (sCmd.CompareNoCase(_T("menu")) == 0) 
	{
		int iMenu = atoi(_ParseURL(Data.sURL, _T("m")));
		bool bValue = atoi(_ParseURL(Data.sURL, _T("v")));
		WSserverColumnHidden[iMenu] = bValue;
		CString buffer;
		CString fullpath;
		fullpath.Format(_T("%spreferences.ini"),theApp.glob_prefs->GetConfigDir());
		CIni ini( fullpath );
		ini.SetArray(WSserverColumnHidden, ELEMENT_COUNT(WSserverColumnHidden), _T("serverColumnHidden"), _T("WebServer")); 
		ini.SaveAndClose(); 
	}
	CString sSort = _ParseURL(Data.sURL, _T("sort"));
	if (sSort != "") 
	{
		if(sSort == "name")
			pThis->m_Params.ServerSort = SERVER_SORT_NAME;
		else
		if(sSort == "ip")
			pThis->m_Params.ServerSort = SERVER_SORT_IP;
		else
		if(sSort == "description")
			pThis->m_Params.ServerSort = SERVER_SORT_DESCRIPTION;
		else
		if(sSort == "ping")
			pThis->m_Params.ServerSort = SERVER_SORT_PING;
		else
		if(sSort == "users")
			pThis->m_Params.ServerSort = SERVER_SORT_USERS;
		else
		if(sSort == "files")
			pThis->m_Params.ServerSort = SERVER_SORT_FILES;
		else
		if(sSort == "priority")
			pThis->m_Params.ServerSort = SERVER_SORT_PRIORITY;
		else
		if(sSort == "failed")
			pThis->m_Params.ServerSort = SERVER_SORT_FAILED;
		else
		if(sSort == "limit")
			pThis->m_Params.ServerSort = SERVER_SORT_LIMIT;
		else
		if(sSort == "version")
			pThis->m_Params.ServerSort = SERVER_SORT_VERSION;
	}
	sSort.SetString(_ParseURL(Data.sURL, _T("sortreverse")));
	if (sSort.IsEmpty()) {
		pThis->m_Params.bServerSortReverse = false;
	} else {
		pThis->m_Params.bServerSortReverse = (sSort.CompareNoCase(_T("true")) == 0);
	}

	CString sServerSortRev;
	if(pThis->m_Params.bServerSortReverse)
		sServerSortRev = "false";
	else
		sServerSortRev = "true";

	CString Out = pThis->m_Templates.sServerList;
    Out.Replace(_T("[AddServerBox]"), sAddServerBox);
	Out.Replace(_T("[Session]"), sSession);
	if(pThis->m_Params.ServerSort == SERVER_SORT_NAME)
		Out.Replace(_T("[SortName]"), _T("&sortreverse=") + sServerSortRev);
	else
		Out.Replace(_T("[SortName]"), _T(""));
	if(pThis->m_Params.ServerSort == SERVER_SORT_IP)
		Out.Replace(_T("[SortIP]"), _T("&sortreverse=") + sServerSortRev);
	else
		Out.Replace(_T("[SortIP]"), _T(""));
	if(pThis->m_Params.ServerSort == SERVER_SORT_DESCRIPTION)
		Out.Replace(_T("[SortDescription]"), _T("&sortreverse=") + sServerSortRev);
	else
		Out.Replace(_T("[SortDescription]"), _T(""));
	if(pThis->m_Params.ServerSort == SERVER_SORT_PING)
		Out.Replace(_T("[SortPing]"), _T("&sortreverse=") + sServerSortRev);
	else
		Out.Replace(_T("[SortPing]"), _T(""));
	if(pThis->m_Params.ServerSort == SERVER_SORT_USERS)
		Out.Replace(_T("[SortUsers]"), _T("&sortreverse=") + sServerSortRev);
	else
		Out.Replace(_T("[SortUsers]"), _T(""));
	if(pThis->m_Params.ServerSort == SERVER_SORT_FILES)
		Out.Replace(_T("[SortFiles]"), _T("&sortreverse=") + sServerSortRev);
	else
		Out.Replace(_T("[SortFiles]"), _T(""));
	if(pThis->m_Params.ServerSort == SERVER_SORT_PRIORITY)
		Out.Replace(_T("[SortPriority]"), _T("&sortreverse=") + sServerSortRev);
	else
		Out.Replace(_T("[SortPriority]"), _T(""));
	if(pThis->m_Params.ServerSort == SERVER_SORT_FAILED)
		Out.Replace(_T("[SortFailed]"), _T("&sortreverse=") + sServerSortRev);
	else
		Out.Replace(_T("[SortFailed]"), _T(""));
	if(pThis->m_Params.ServerSort == SERVER_SORT_LIMIT)
		Out.Replace(_T("[SortLimit]"), _T("&sortreverse=") + sServerSortRev);
	else
		Out.Replace(_T("[SortLimit]"), _T(""));
	if(pThis->m_Params.ServerSort == SERVER_SORT_VERSION)
		Out.Replace(_T("[SortVersion]"), _T("&sortreverse=") + sServerSortRev);
	else
		Out.Replace(_T("[SortVersion]"), _T(""));
	Out.Replace(_T("[ServerList]"), _GetPlainResString(IDS_SV_SERVERLIST));
	if (!WSserverColumnHidden[0]){
		Out.Replace(_T("[ServernameH]"), _GetPlainResString(IDS_SL_SERVERNAME));
		Out.Replace(_T("[ServernameM]"), _GetPlainResString(IDS_SL_SERVERNAME)); }
	else {
		Out.Replace(_T("[ServernameH]"), "");
		Out.Replace(_T("[ServernameM]"), _GetPlainResString(IDS_SL_SERVERNAME)); }
	if (!WSserverColumnHidden[1]) {
		Out.Replace(_T("[AddressH]"), _GetPlainResString(IDS_IP));
		Out.Replace(_T("[AddressM]"), _GetPlainResString(IDS_IP)); }
	else {
		Out.Replace(_T("[AddressH]"), "");
		Out.Replace(_T("[AddressM]"), _GetPlainResString(IDS_IP)); }
	if (!WSserverColumnHidden[2]) {
		Out.Replace(_T("[DescriptionH]"), _GetPlainResString(IDS_DESCRIPTION));
		Out.Replace(_T("[DescriptionM]"), _GetPlainResString(IDS_DESCRIPTION)); }
	else {
		Out.Replace(_T("[DescriptionH]"), "");
		Out.Replace(_T("[DescriptionM]"), _GetPlainResString(IDS_DESCRIPTION)); }
	if (!WSserverColumnHidden[3]) {
		Out.Replace(_T("[PingH]"), _GetPlainResString(IDS_PING));
		Out.Replace(_T("[PingM]"), _GetPlainResString(IDS_PING)); }
	else {
		Out.Replace(_T("[PingH]"), "");
		Out.Replace(_T("[PingM]"), _GetPlainResString(IDS_PING)); }
	if (!WSserverColumnHidden[4]) {
		Out.Replace(_T("[UsersH]"), _GetPlainResString(IDS_UUSERS));
		Out.Replace(_T("[UsersM]"), _GetPlainResString(IDS_UUSERS)); }
	else {
		Out.Replace(_T("[UsersH]"), "");
		Out.Replace(_T("[UsersM]"), _GetPlainResString(IDS_UUSERS)); }
	if (!WSserverColumnHidden[5]) {
		Out.Replace(_T("[FilesH]"), _GetPlainResString(IDS_FILES));
		Out.Replace(_T("[FilesM]"), _GetPlainResString(IDS_FILES)); }
	else {
		Out.Replace(_T("[FilesH]"), "");
		Out.Replace(_T("[FilesM]"), _GetPlainResString(IDS_FILES)); }
	if (!WSserverColumnHidden[6]) {
		Out.Replace(_T("[PriorityH]"), _GetPlainResString(IDS_PRIORITY));
		Out.Replace(_T("[PriorityM]"), _GetPlainResString(IDS_PRIORITY)); }
	else {
		Out.Replace(_T("[PriorityH]"), "");
		Out.Replace(_T("[PriorityM]"), _GetPlainResString(IDS_PRIORITY)); }
	if (!WSserverColumnHidden[7]) {
		Out.Replace(_T("[FailedH]"), _GetPlainResString(IDS_UFAILED));
		Out.Replace(_T("[FailedM]"), _GetPlainResString(IDS_UFAILED)); }
	else {
		Out.Replace(_T("[FailedH]"), "");
		Out.Replace(_T("[FailedM]"), _GetPlainResString(IDS_UFAILED)); }
	if (!WSserverColumnHidden[8]) {
		Out.Replace(_T("[LimitH]"), _GetPlainResString(IDS_SERVER_SOFTHARDLIMIT));
		Out.Replace(_T("[LimitM]"), _GetPlainResString(IDS_SERVER_SOFTHARDLIMIT)); }
	else {
		Out.Replace(_T("[LimitH]"), "");
		Out.Replace(_T("[LimitM]"), _GetPlainResString(IDS_SERVER_SOFTHARDLIMIT)); }
	if (!WSserverColumnHidden[9]) {
		Out.Replace(_T("[VersionH]"), _GetPlainResString(IDS_SERVER_VERSION));
		Out.Replace(_T("[VersionM]"), _GetPlainResString(IDS_SERVER_VERSION)); }
	else {
		Out.Replace(_T("[VersionH]"), "");
		Out.Replace(_T("[VersionM]"), _GetPlainResString(IDS_SERVER_VERSION)); }
	Out.Replace(_T("[Actions]"), _GetPlainResString(IDS_WEB_ACTIONS));

	CString OutE = pThis->m_Templates.sServerLine;

	CArray<ServerEntry, ServerEntry> ServerArray;

	CServer* tempServer;
	// Populating array
	for (uint32 sc=0;sc<theApp.serverlist->GetServerCount();sc++)
	{
		CServer* cur_file = theApp.serverlist->GetServerAt(sc);
		ServerEntry Entry;
		Entry.sServerName = _SpecialChars(cur_file->GetListName());
		Entry.sServerIP = cur_file->GetAddress();
		Entry.nServerPort = cur_file->GetPort();
		Entry.sServerDescription = _SpecialChars(cur_file->GetDescription());
		Entry.nServerPing = cur_file->GetPing();
        Entry.nServerUsers = cur_file->GetUsers();
		Entry.nServerMaxUsers = cur_file->GetMaxUsers();
		Entry.nServerFiles = cur_file->GetFiles();
		Entry.bServerStatic = cur_file->IsStaticMember();	// Purity (Add Remove from Static Server List in WebServer)
		if(cur_file->GetPreferences()== CServer::ePR_HIGH) {
			Entry.sServerPriority = _GetPlainResString(IDS_PRIOHIGH);
			Entry.nServerPriority = 2;
		} else if(cur_file->GetPreferences()== CServer::ePR_NORMAL) {
			Entry.sServerPriority = _GetPlainResString(IDS_PRIONORMAL);
			Entry.nServerPriority = 1;
		} else if(cur_file->GetPreferences()== CServer::ePR_LOW) {
			Entry.sServerPriority = _GetPlainResString(IDS_PRIOLOW);
			Entry.nServerPriority = 0;
		}
		Entry.nServerFailed = cur_file->GetFailedCount();
		Entry.nServerSoftLimit = cur_file->GetSoftFiles();
		Entry.nServerHardLimit = cur_file->GetHardFiles();
		Entry.sServerVersion = cur_file->GetVersion();
		int counter=0;
		CString temp="";
		CString newip="";
		for(int j=0; j<4; j++) {  
			temp = Entry.sServerIP.Tokenize(_T("."),counter); 
			if (temp.GetLength() == 1) newip.AppendFormat("00" + temp); 
			else if (temp.GetLength() == 2) newip.AppendFormat("0" + temp); 
			else if (temp.GetLength() == 3) newip.AppendFormat("" + temp);
        }
		Entry.sServerFullIP = newip;

#ifdef OLD_SOCKETS_ENABLED
		if (theApp.serverconnect->IsConnecting()) { 
			tempServer = theApp.serverconnect->GetConnectingServer();
			if (tempServer->GetFullIP() == cur_file->GetFullIP())
				Entry.sServerState = "connecting";
			else
				Entry.sServerState = "disconnected";
		}
		else if (theApp.serverconnect->IsConnected()) {
			tempServer = theApp.serverconnect->GetCurrentServer();
			if (tempServer->GetFullIP() == cur_file->GetFullIP()) {  
				if (!theApp.serverconnect->IsLowID())  
					Entry.sServerState = "high";  
				else  
					Entry.sServerState = "low";
			} else
				Entry.sServerState = "disconnected";
		} // end IsConnected
		else
#endif //OLD_SOCKETS_ENABLED
			Entry.sServerState = "disconnected";

		ServerArray.Add(Entry);
	}
	// Sorting (simple bubble sort, we don't have tons of data here)
	bool bSorted = true;
	for(int nMax = 0;bSorted && nMax < ServerArray.GetCount()*2; nMax++)
	{
		bSorted = false;
		for(int i = 0; i < ServerArray.GetCount() - 1; i++)
		{
			bool bSwap = false;
			switch(pThis->m_Params.ServerSort)
			{
			case SERVER_SORT_NAME:
				bSwap = ServerArray[i].sServerName.CompareNoCase(ServerArray[i+1].sServerName) > 0;
				break;
			case SERVER_SORT_IP:
				bSwap = ServerArray[i].sServerFullIP < ServerArray[i+1].sServerFullIP;
				break;
			case SERVER_SORT_DESCRIPTION:
				bSwap = ServerArray[i].sServerDescription.CompareNoCase(ServerArray[i+1].sServerDescription) < 0;
				break;
			case SERVER_SORT_PING:
				bSwap = ServerArray[i].nServerPing < ServerArray[i+1].nServerPing;
				break;
			case SERVER_SORT_USERS:
				bSwap = ServerArray[i].nServerUsers < ServerArray[i+1].nServerUsers;
				break;
			case SERVER_SORT_FILES:
				bSwap = ServerArray[i].nServerFiles < ServerArray[i+1].nServerFiles;
				break;
			case SERVER_SORT_PRIORITY:
				bSwap = ServerArray[i].nServerPriority < ServerArray[i+1].nServerPriority;
				break;
			case SERVER_SORT_FAILED:
				bSwap = ServerArray[i].nServerFailed < ServerArray[i+1].nServerFailed;
				break;
			case SERVER_SORT_LIMIT:
				bSwap = ServerArray[i].nServerSoftLimit < ServerArray[i+1].nServerSoftLimit;
				break;
			case SERVER_SORT_VERSION:
				bSwap = ServerArray[i].sServerVersion < ServerArray[i+1].sServerVersion;
				break;
			}
			if(pThis->m_Params.bServerSortReverse)
			{
				bSwap = !bSwap;
			}
			if(bSwap)
			{
				bSorted = true;
				ServerEntry TmpEntry = ServerArray[i];
				ServerArray[i] = ServerArray[i+1];
				ServerArray[i+1] = TmpEntry;
			}
		}
	}

	// Displaying
	CString sList = _T("");
	for(int i = 0; i < ServerArray.GetCount(); i++)
	{
		CString HTTPProcessData = OutE;	// Copy Entry Line to Temp

		CString admin = "";			//admin rights
		CString ed2k = "";			//ed2k
		CString session = "";		//session
		CString ip = "";			//ip
		CString port = "";			//port
		CString isstatic = "";		//serverstatic
		CString srvpriority = "";	//priority

		CString sServerPort;
		sServerPort.Format(_T("%d"), ServerArray[i].nServerPort);
		ed2k.Format(CString(_T("ed2k://|server|") + ServerArray[i].sServerIP + "|" + sServerPort + _T("|/")));

		if (ServerArray[i].sServerIP == _ParseURL(Data.sURL,_T("ip")) && sServerPort == _ParseURL(Data.sURL,_T("port")))
            HTTPProcessData.Replace(_T("[LastChangedDataset]"), "checked");
		else
            HTTPProcessData.Replace(_T("[LastChangedDataset]"), "checked_no");

		if (bAdmin)
			admin.Format(_T("admin"));

		session.Format(CString(sSession));
		ip.Format(CString(ServerArray[i].sServerIP));
		port.Format(CString(sServerPort));

		// DonGato: static server showing
		if (ServerArray[i].bServerStatic)
		{
			isstatic.Format(_T("staticsrv"));
			HTTPProcessData.Replace(_T("[ServerType]"), _T("static"));
		}
		else
			HTTPProcessData.Replace(_T("[ServerType]"), _T("none"));

		//DonGato: priority change
		switch(ServerArray[i].nServerPriority)
		{
			case 0:
				srvpriority.Format(_T("Low"));
				break;
			case 1:
				srvpriority.Format(_T("Normal"));
				break;
			case 2:
				srvpriority.Format(_T("High"));
				break;
		}

		HTTPProcessData.Replace(_T("[admin]"), admin);
		HTTPProcessData.Replace(_T("[ed2k]"), ed2k);
		HTTPProcessData.Replace(_T("[session]"), session);
		HTTPProcessData.Replace(_T("[ip]"), ip);
		HTTPProcessData.Replace(_T("[port]"), port);
		HTTPProcessData.Replace(_T("[isstatic]"), isstatic);
		HTTPProcessData.Replace(_T("[server-priority]"), srvpriority); //DonGato: priority change

		// DonGato: reduced large servernames or descriptions
		if (!WSserverColumnHidden[0]) {
			if(ServerArray[i].sServerName.GetLength() > SHORT_LENGTH)
				HTTPProcessData.Replace(_T("[Servername]"), _T("<acronym title=\"") + ServerArray[i].sServerName + _T("\">") + ServerArray[i].sServerName.Left(SHORT_LENGTH) + _T("...") + _T("</acronym>"));
			else
				HTTPProcessData.Replace(_T("[Servername]"), ServerArray[i].sServerName);
		} else
			HTTPProcessData.Replace(_T("[Name]"), "");
		if (!WSserverColumnHidden[1]) {
			CString sPort; sPort.Format(_T(":%d"), ServerArray[i].nServerPort);
			HTTPProcessData.Replace(_T("[Address]"), ServerArray[i].sServerIP + sPort);
		} else
			HTTPProcessData.Replace(_T("[Address]"), "");
		if (!WSserverColumnHidden[2]) {
			if(ServerArray[i].sServerDescription.GetLength() > SHORT_LENGTH)
				HTTPProcessData.Replace(_T("[Description]"), _T("<acronym title=\"") + ServerArray[i].sServerDescription + _T("\">") + ServerArray[i].sServerDescription.Left(SHORT_LENGTH) + _T("...") + _T("</acronym>"));
			else
				HTTPProcessData.Replace(_T("[Description]"), ServerArray[i].sServerDescription);
		} else
			HTTPProcessData.Replace(_T("[Description]"), "");
		if (!WSserverColumnHidden[3]) {
			CString sPing; sPing.Format(_T("%d"), ServerArray[i].nServerPing);
			HTTPProcessData.Replace(_T("[Ping]"), sPing);
		} else
			HTTPProcessData.Replace(_T("[Ping]"), "");
		CString sT;
		if (!WSserverColumnHidden[4]) {
			if(ServerArray[i].nServerUsers > 0)
			{
				if(ServerArray[i].nServerMaxUsers > 0)
					sT.Format(_T("%d (%d)"), ServerArray[i].nServerUsers, ServerArray[i].nServerMaxUsers);
				else
					sT.Format(_T("%d"), ServerArray[i].nServerUsers);
			}
			HTTPProcessData.Replace(_T("[Users]"), sT);
		} else
			HTTPProcessData.Replace(_T("[Users]"), "");
		sT = _T("");
		if (!WSserverColumnHidden[5]) {
			if(ServerArray[i].nServerFiles > 0)
				sT.Format(_T("%d"), ServerArray[i].nServerFiles);
			HTTPProcessData.Replace(_T("[Files]"), sT);
		} else
			HTTPProcessData.Replace(_T("[Files]"), "");
		if (!WSserverColumnHidden[6])
			HTTPProcessData.Replace(_T("[Priority]"), ServerArray[i].sServerPriority);
		else
			HTTPProcessData.Replace(_T("[Priority]"), "");
		if (!WSserverColumnHidden[7]) {
			CString sFailed; sFailed.Format(_T("%d"), ServerArray[i].nServerFailed);
			HTTPProcessData.Replace(_T("[Failed]"), sFailed);
		} else
			HTTPProcessData.Replace(_T("[Failed]"), "");
		if (!WSserverColumnHidden[8]) {
			CString sSoftLimit; sSoftLimit.Format(_T("%d"), ServerArray[i].nServerSoftLimit);
			CString sHardLimit; sHardLimit.Format(_T("%d"), ServerArray[i].nServerHardLimit);
			HTTPProcessData.Replace(_T("[Limit]"), sSoftLimit + " (" + sHardLimit + ")");
		} else
			HTTPProcessData.Replace(_T("[Limit]"), "");
		if (!WSserverColumnHidden[9]) {
			if(ServerArray[i].sServerVersion.GetLength() > SHORT_LENGTH_MIN)
				HTTPProcessData.Replace(_T("[Version]"), _T("<acronym title=\"") + ServerArray[i].sServerVersion + _T("\">") + ServerArray[i].sServerVersion.Left(SHORT_LENGTH) + _T("...") + _T("</acronym>"));
			else
				HTTPProcessData.Replace(_T("[Version]"), ServerArray[i].sServerVersion);
		} else
			HTTPProcessData.Replace(_T("[Version]"), "");
		HTTPProcessData.Replace(_T("[ServerState]"), ServerArray[i].sServerState);
		sList += HTTPProcessData;
	}
	Out.Replace(_T("[ServersList]"), sList);

	return Out;

	EMULE_CATCH

	return _T("");
}

CString CWebServer::_GetTransferList(ThreadData Data)
{
	EMULE_TRY

	CWebServer *pThis = (CWebServer *)Data.pThis;
	if(pThis == NULL)
		return _T("");

	int cat=atoi(_ParseURL(Data.sURL,"cat"));
	CString sCat; (cat!=0)?sCat.Format("&cat=%i",cat):sCat="";

	CString HTTPTemp;

	CString sSession = _ParseURL(Data.sURL, _T("ses"));

	CString admin;
	bool bAdmin = IsSessionAdmin(Data,sSession);	//SyruS cached (used often here)

	if (bAdmin)
		admin.Format(_T("admin"));

	CString Out = _T("");

	//SyruS show completed files (0.28b) plus clear completed
	CString sClear(_ParseURL(Data.sURL, _T("clearcompleted")));
	if (bAdmin && !sClear.IsEmpty()) 
	{
		if (sClear.CompareNoCase(_T("all")) == 0) {
			theApp.emuledlg->transferwnd.m_wndDownloadList.ClearCompleted();
		} else {
			uchar FileHash[16];
			if (_GetFileHash(sClear, FileHash))	//failsafe
				theApp.emuledlg->transferwnd.m_wndDownloadList.ClearCompleted(FileHash);
		}
	}

	HTTPTemp = _ParseURL(Data.sURL, _T("ed2k"));

	if (bAdmin && !HTTPTemp.IsEmpty()) 
		theApp.emuledlg->searchwnd.AddEd2kLinksToDownload(HTTPTemp,cat);
	else 
		HTTPTemp.Empty();	//free for reusing

	HTTPTemp = _ParseURL(Data.sURL, _T("c"));

	if (HTTPTemp.CompareNoCase(_T("menudown")) == 0) 
	{
		int iMenu = atoi(_ParseURL(Data.sURL, _T("m")));
		bool bValue = atoi(_ParseURL(Data.sURL, _T("v")));
		WSdownloadColumnHidden[iMenu] = bValue;
		CString buffer;
		CString fullpath;
		fullpath.Format(_T("%spreferences.ini"),theApp.glob_prefs->GetConfigDir());
		CIni ini( fullpath );
		ini.SetArray(WSdownloadColumnHidden, ELEMENT_COUNT(WSdownloadColumnHidden), _T("downloadColumnHidden"), _T("WebServer")); 
		ini.SaveAndClose(); 
	}
	else if (HTTPTemp.CompareNoCase(_T("menuup")) == 0) 
	{
		int iMenu = atoi(_ParseURL(Data.sURL, _T("m")));
		bool bValue = atoi(_ParseURL(Data.sURL, _T("v")));
		WSuploadColumnHidden[iMenu] = bValue;
		CString buffer;
		CString fullpath;
		fullpath.Format(_T("%spreferences.ini"),theApp.glob_prefs->GetConfigDir());
		CIni ini( fullpath );
		ini.SetArray(WSuploadColumnHidden, ELEMENT_COUNT(WSuploadColumnHidden), _T("uploadColumnHidden"), _T("WebServer")); 
		ini.SaveAndClose(); 
	}
	else if (HTTPTemp.CompareNoCase(_T("menuqueue")) == 0) 
	{
		int iMenu = atoi(_ParseURL(Data.sURL, _T("m")));
		bool bValue = atoi(_ParseURL(Data.sURL, _T("v")));
		WSqueueColumnHidden[iMenu] = bValue;
		CString buffer;
		CString fullpath;
		fullpath.Format(_T("%spreferences.ini"),theApp.glob_prefs->GetConfigDir());
		CIni ini( fullpath );
		ini.SetArray(WSqueueColumnHidden, ELEMENT_COUNT(WSqueueColumnHidden), _T("queueColumnHidden"), _T("WebServer")); 
		ini.SaveAndClose(); 
	}
	else if (HTTPTemp.CompareNoCase(_T("menuprio")) == 0) //DonGato: category priority change
	{
		if(bAdmin)
		{
			CString sPrio(_ParseURL(Data.sURL, _T("p")));

			if(sPrio.CompareNoCase(_T("low")) == 0) 
			{
				theApp.downloadqueue->SetCatPrio(cat,PR_LOW);
			}
			else if(sPrio.CompareNoCase(_T("normal")) == 0)
			{
				theApp.downloadqueue->SetCatPrio(cat,PR_NORMAL);
			}
			else if(sPrio.CompareNoCase(_T("high")) == 0)
			{
				theApp.downloadqueue->SetCatPrio(cat,PR_HIGH);
			}
			else if(sPrio.CompareNoCase(_T("auto")) == 0)
			{
				theApp.downloadqueue->SetCatPrio(cat,PR_AUTO);
			}
		}
	}

	CString sOp(_ParseURL(Data.sURL, _T("op")));
	CString sFile(_ParseURL(Data.sURL, _T("file")));
	CString sUser(_ParseURL(Data.sURL, _T("userhash")));
	uchar FileHash[16];
	uchar UserHash[16];
	if (!sOp.IsEmpty() && !sFile.IsEmpty() && /*failsafe*/ _GetFileHash(sFile, FileHash))
	{
 		//unsafe _GetFileHash(_ParseURL(Data.sURL, _T("file")), FileHash);
		CPartFile* found_file = theApp.downloadqueue->GetFileByID(FileHash);
		if(found_file)
		{	// SyruS all actions require a found file (removed double-check inside)
			if(bAdmin && sOp.CompareNoCase(_T("a4af")) == 0) // DonGato: added A4ADF action
			{
				found_file->DownloadAllA4AF();
			}
			else if(bAdmin && sOp.CompareNoCase(_T("autoa4af")) == 0)
			{
				theApp.downloadqueue->SetA4AFAutoFile(found_file); // Purity: Auto-A4AF
			}
			else if(bAdmin && sOp.CompareNoCase(_T("noautoa4af")) == 0)
			{
				theApp.downloadqueue->SetA4AFAutoFile(NULL); // Purity: Auto-A4AF
			}
			else if(bAdmin && sOp.CompareNoCase(_T("stop")) == 0)
			{
				found_file->StopFile();
			}
			else if(bAdmin && sOp.CompareNoCase(_T("resume")) == 0)
			{
				found_file->ResumeFile();
			}
			else if(bAdmin && sOp.CompareNoCase(_T("cancel")) == 0)
			{
				found_file->DeleteFile();
			}
			else if(bAdmin && sOp.CompareNoCase(_T("priolow")) == 0) //DonGato: priority change
			{
				found_file->SetAutoPriority(false);
				found_file->SetPriority(PR_LOW);
			}
			else if(bAdmin && sOp.CompareNoCase(_T("prionormal")) == 0) //DonGato: priority change
			{
				found_file->SetAutoPriority(false);
				found_file->SetPriority(PR_NORMAL);
			}
			else if(bAdmin && sOp.CompareNoCase(_T("priohigh")) == 0) //DonGato: priority change
			{
				found_file->SetAutoPriority(false);
				found_file->SetPriority(PR_HIGH);
			}
			else if(bAdmin && sOp.CompareNoCase(_T("prioauto")) == 0) //DonGato: priority change
			{
				found_file->SetAutoPriority(true);
				found_file->SetPriority(PR_HIGH);
			}
			else if(bAdmin && sOp.CompareNoCase(_T("compressed")) == 0) // Purity: DropSCB
			{
				found_file->SetDiscardSuperCompressed( !found_file->GetDiscardSuperCompressed() );
			}
			else if(bAdmin && sOp.CompareNoCase(_T("noneededsources")) == 0) // Purity: DropNNS
			{
				found_file->RemoveNoNeededSources();
			}
			else if(bAdmin && sOp.CompareNoCase(_T("setcat")) == 0) // Purity: Change file category
			{
				int iMenu = atoi(_ParseURL(Data.sURL, _T("filecat")));
				found_file->SetCategory(iMenu);
			}
		}	//if(found_file) pre-check
	}
	else if (!sOp.IsEmpty() && !sUser.IsEmpty() && /*failsafe*/ _GetFileHash(sUser, UserHash))
	{
		CUpDownClient* cur_client = theApp.clientlist->FindClientByUserHash(UserHash);
		if(cur_client)
		{
			if(bAdmin && sOp.CompareNoCase(_T("addfriend")) == 0)
			{
				theApp.friendlist->AddFriend(cur_client);
			}
			else if(bAdmin && sOp.CompareNoCase(_T("removefriend")) == 0)
			{
				theApp.friendlist->RemoveFriend(cur_client);
			}
		}
	}
	CString sSort(_ParseURL(Data.sURL, _T("sort")));
	if (!sSort.IsEmpty()) 
	{
		if(sSort.CompareNoCase(_T("dname")) == 0)
			pThis->m_Params.DownloadSort = DOWN_SORT_NAME;
		else
		if(sSort.CompareNoCase(_T("dsize")) == 0)
			pThis->m_Params.DownloadSort = DOWN_SORT_SIZE;
		else
		if(sSort.CompareNoCase(_T("dtransferred")) == 0)
			pThis->m_Params.DownloadSort = DOWN_SORT_TRANSFERRED;
		else
		if(sSort.CompareNoCase(_T("dspeed")) == 0)
			pThis->m_Params.DownloadSort = DOWN_SORT_SPEED;
		else
		if(sSort.CompareNoCase(_T("dprogress")) == 0)
			pThis->m_Params.DownloadSort = DOWN_SORT_PROGRESS;		
		else
		if(sSort.CompareNoCase(_T("dsources")) == 0)
			pThis->m_Params.DownloadSort = DOWN_SORT_SOURCES;
		else
		if(sSort.CompareNoCase(_T("dpriority")) == 0)
			pThis->m_Params.DownloadSort = DOWN_SORT_PRIORITY;
		else
		if(sSort.CompareNoCase(_T("dcategory")) == 0)
			pThis->m_Params.DownloadSort = DOWN_SORT_CATEGORY;
		else
		if(sSort.CompareNoCase(_T("uuser")) == 0)
			pThis->m_Params.UploadSort = UP_SORT_USER;
		else
		if(sSort.CompareNoCase(_T("uversion")) == 0)
			pThis->m_Params.UploadSort = UP_SORT_VERSION;
		else
		if(sSort.CompareNoCase(_T("ufilename")) == 0)
			pThis->m_Params.UploadSort = UP_SORT_FILENAME;
		else
		if(sSort.CompareNoCase(_T("utransferred")) == 0)
			pThis->m_Params.UploadSort = UP_SORT_TRANSFERRED;
		else
		if(sSort.CompareNoCase(_T("uspeed")) == 0)
			pThis->m_Params.UploadSort = UP_SORT_SPEED;
		else
		if(sSort.CompareNoCase(_T("quser")) == 0)
			pThis->m_Params.QueueSort = QU_SORT_USER;
		else
		if(sSort.CompareNoCase(_T("qversion")) == 0)
			pThis->m_Params.QueueSort = QU_SORT_VERSION;
		else
		if(sSort.CompareNoCase(_T("qfilename")) == 0)
			pThis->m_Params.QueueSort = QU_SORT_FILENAME;
		else
		if(sSort.CompareNoCase(_T("qscore")) == 0)
			pThis->m_Params.QueueSort = QU_SORT_SCORE;
	}
	sSort.SetString(_ParseURL(Data.sURL, _T("sortreverse")));
	if (sSort.IsEmpty()) {
		pThis->m_Params.bDownloadSortReverse = false;
		pThis->m_Params.bUploadSortReverse = false;
		pThis->m_Params.bQueueSortReverse = false;
	} else {
		pThis->m_Params.bDownloadSortReverse = (sSort.CompareNoCase(_T("true")) == 0);
		pThis->m_Params.bUploadSortReverse = (sSort.CompareNoCase(_T("true")) == 0);
		pThis->m_Params.bQueueSortReverse = (sSort.CompareNoCase(_T("true")) == 0);
	}

	HTTPTemp.SetString(_ParseURL(Data.sURL, _T("showuploadqueue")));
	if (HTTPTemp.CompareNoCase(_T("true")) == 0)
		pThis->m_Params.bShowUploadQueue = true;
	else if (HTTPTemp.CompareNoCase(_T("false")) == 0)
		pThis->m_Params.bShowUploadQueue = false;

	HTTPTemp.SetString(_ParseURL(Data.sURL, _T("showuploadqueuebanned")));
	if (HTTPTemp.CompareNoCase(_T("true")) == 0)
		pThis->m_Params.bShowUploadQueueBanned = true;
	else if (HTTPTemp.CompareNoCase(_T("false")) == 0)
		pThis->m_Params.bShowUploadQueueBanned = false;

	HTTPTemp.SetString(_ParseURL(Data.sURL, _T("showuploadqueuefriend")));
	if (HTTPTemp.CompareNoCase(_T("true")) == 0)
		pThis->m_Params.bShowUploadQueueFriend = true;
	else if (HTTPTemp.CompareNoCase(_T("false")) == 0)
		pThis->m_Params.bShowUploadQueueFriend = false;

	HTTPTemp.SetString(_ParseURL(Data.sURL, _T("showuploadqueuecredit")));
	if (HTTPTemp.CompareNoCase(_T("true")) == 0)
		pThis->m_Params.bShowUploadQueueCredit = true;
	else if (HTTPTemp.CompareNoCase(_T("false")) == 0)
		pThis->m_Params.bShowUploadQueueCredit = false;

	CString sDownloadSortRev;
	if(pThis->m_Params.bDownloadSortReverse)
		sDownloadSortRev = "false";
	else
		sDownloadSortRev = "true";

	CString sUploadSortRev;
	if(pThis->m_Params.bUploadSortReverse)
		sUploadSortRev = "false";
	else
		sUploadSortRev = "true";

	CString sQueueSortRev;
	if(pThis->m_Params.bQueueSortReverse)
		sQueueSortRev = "false";
	else
		sQueueSortRev = "true";

	Out += pThis->m_Templates.sTransferImages;
	Out += pThis->m_Templates.sTransferList;
	Out.Replace(_T("[DownloadHeader]"), pThis->m_Templates.sTransferDownHeader);
	Out.Replace(_T("[DownloadFooter]"), pThis->m_Templates.sTransferDownFooter);
	Out.Replace(_T("[UploadHeader]"), pThis->m_Templates.sTransferUpHeader);
	Out.Replace(_T("[UploadFooter]"), pThis->m_Templates.sTransferUpFooter);
	Out.Replace(_T("[Session]"), sSession);

	InsertCatBox(Out,cat,"",true,true,sSession,"");

	if(pThis->m_Params.DownloadSort == DOWN_SORT_NAME)
		Out.Replace(_T("[SortDName]"), _T("&sortreverse=") + sDownloadSortRev);
	else
		Out.Replace(_T("[SortDName]"), _T(""));
	if(pThis->m_Params.DownloadSort == DOWN_SORT_SIZE)
		Out.Replace(_T("[SortDSize]"), _T("&sortreverse=") + sDownloadSortRev);
	else
		Out.Replace(_T("[SortDSize]"), _T(""));
	if(pThis->m_Params.DownloadSort == DOWN_SORT_TRANSFERRED)
		Out.Replace(_T("[SortDTransferred]"), _T("&sortreverse=") + sDownloadSortRev);
	else
		Out.Replace(_T("[SortDTransferred]"), _T(""));
	if(pThis->m_Params.DownloadSort == DOWN_SORT_SPEED)
		Out.Replace(_T("[SortDSpeed]"), _T("&sortreverse=") + sDownloadSortRev);
	else
		Out.Replace(_T("[SortDSpeed]"), _T(""));
	if(pThis->m_Params.DownloadSort == DOWN_SORT_PROGRESS)
		Out.Replace(_T("[SortDProgress]"), _T("&sortreverse=") + sDownloadSortRev);
	else
		Out.Replace(_T("[SortDProgress]"), _T(""));
	if(pThis->m_Params.DownloadSort == DOWN_SORT_SOURCES)
		Out.Replace(_T("[SortDSources]"), _T("&sortreverse=") + sDownloadSortRev);
	else
		Out.Replace(_T("[SortDSources]"), _T(""));
	if(pThis->m_Params.DownloadSort == DOWN_SORT_PRIORITY)
		Out.Replace(_T("[SortDPriority]"), _T("&sortreverse=") + sDownloadSortRev);
	else
		Out.Replace(_T("[SortDPriority]"), _T(""));
	if(pThis->m_Params.DownloadSort == DOWN_SORT_CATEGORY)
		Out.Replace(_T("[SortDCategory]"), _T("&sortreverse=") + sDownloadSortRev);
	else
		Out.Replace(_T("[SortDCategory]"), _T(""));
	if(pThis->m_Params.UploadSort == UP_SORT_USER)
		Out.Replace(_T("[SortUUser]"), _T("&sortreverse=") + sUploadSortRev);
	else
		Out.Replace(_T("[SortUUser]"), _T(""));
	if(pThis->m_Params.UploadSort == UP_SORT_VERSION)
		Out.Replace(_T("[SortUVersion]"), _T("&sortreverse=") + sUploadSortRev);
	else
		Out.Replace(_T("[SortUVersion]"), _T(""));
	if(pThis->m_Params.UploadSort == UP_SORT_FILENAME)
		Out.Replace(_T("[SortUFilename]"), _T("&sortreverse=") + sUploadSortRev);
	else
		Out.Replace(_T("[SortUFilename]"), _T(""));
	if(pThis->m_Params.UploadSort == UP_SORT_TRANSFERRED)
		Out.Replace(_T("[SortUTransferred]"), _T("&sortreverse=") + sUploadSortRev);
	else
		Out.Replace(_T("[SortUTransferred]"), _T(""));
	if(pThis->m_Params.UploadSort == UP_SORT_SPEED)
		Out.Replace(_T("[SortUSpeed]"), _T("&sortreverse=") + sUploadSortRev);
	else
		Out.Replace(_T("[SortUSpeed]"), _T(""));

	if (!WSdownloadColumnHidden[0]){
		Out.Replace(_T("[DFilename]"), _GetPlainResString(IDS_DL_FILENAME));
		Out.Replace(_T("[DFilenameM]"), _GetPlainResString(IDS_DL_FILENAME)); }
	else {
		Out.Replace(_T("[DFilename]"), "");
		Out.Replace(_T("[DFilenameM]"), _GetPlainResString(IDS_DL_FILENAME)); }
	if (!WSdownloadColumnHidden[1]){
		Out.Replace(_T("[DSize]"), _GetPlainResString(IDS_DL_SIZE));
		Out.Replace(_T("[DSizeM]"), _GetPlainResString(IDS_DL_SIZE)); }
	else {
		Out.Replace(_T("[DSize]"), "");
		Out.Replace(_T("[DSizeM]"), _GetPlainResString(IDS_DL_SIZE)); }
	if (!WSdownloadColumnHidden[2]){
		Out.Replace(_T("[DTransferred]"), _GetPlainResString(IDS_COMPLETE));
		Out.Replace(_T("[DTransferredM]"), _GetPlainResString(IDS_COMPLETE)); }
	else {
		Out.Replace(_T("[DTransferred]"), "");
		Out.Replace(_T("[DTransferredM]"), _GetPlainResString(IDS_COMPLETE)); }
	if (!WSdownloadColumnHidden[3]){
		Out.Replace(_T("[DProgress]"), _GetPlainResString(IDS_DL_PROGRESS));
		Out.Replace(_T("[DProgressM]"), _GetPlainResString(IDS_DL_PROGRESS)); }
	else {
		Out.Replace(_T("[DProgress]"), "");
		Out.Replace(_T("[DProgressM]"), _GetPlainResString(IDS_DL_PROGRESS)); }
	if (!WSdownloadColumnHidden[4]){
		Out.Replace(_T("[DSpeed]"), _GetPlainResString(IDS_DL_SPEED));
		Out.Replace(_T("[DSpeedM]"), _GetPlainResString(IDS_DL_SPEED)); }
	else {
		Out.Replace(_T("[DSpeed]"), "");
		Out.Replace(_T("[DSpeedM]"), _GetPlainResString(IDS_DL_SPEED)); }
	if (!WSdownloadColumnHidden[5]){
		Out.Replace(_T("[DSources]"), _GetPlainResString(IDS_DL_SOURCES));
		Out.Replace(_T("[DSourcesM]"), _GetPlainResString(IDS_DL_SOURCES)); }
	else {
		Out.Replace(_T("[DSources]"), "");
		Out.Replace(_T("[DSourcesM]"), _GetPlainResString(IDS_DL_SOURCES)); }
	if (!WSdownloadColumnHidden[6]){
		Out.Replace(_T("[DPriority]"), _GetPlainResString(IDS_PRIORITY));
		Out.Replace(_T("[DPriorityM]"), _GetPlainResString(IDS_PRIORITY)); }
	else {
		Out.Replace(_T("[DPriority]"), "");
		Out.Replace(_T("[DPriorityM]"), _GetPlainResString(IDS_PRIORITY)); }
	if (!WSdownloadColumnHidden[7]){
		Out.Replace(_T("[DCategory]"), _GetPlainResString(IDS_CAT));
		Out.Replace(_T("[DCategoryM]"), _GetPlainResString(IDS_CAT)); }
	else {
		Out.Replace(_T("[DCategory]"), "");
		Out.Replace(_T("[DCategoryM]"), _GetPlainResString(IDS_CAT)); }
	if (!WSuploadColumnHidden[0]){
		Out.Replace(_T("[UUser]"), _GetPlainResString(IDS_QL_USERNAME));
		Out.Replace(_T("[UUserM]"), _GetPlainResString(IDS_QL_USERNAME)); }
	else {
		Out.Replace(_T("[UUser]"), "");
		Out.Replace(_T("[UUserM]"), _GetPlainResString(IDS_QL_USERNAME)); }
	if (!WSuploadColumnHidden[1]){
		Out.Replace(_T("[UVersion]"), _GetPlainResString(IDS_INFLST_USER_CLIENTVERSION));
		Out.Replace(_T("[UVersionM]"), _GetPlainResString(IDS_INFLST_USER_CLIENTVERSION)); }
	else {
		Out.Replace(_T("[UVersion]"), "");
		Out.Replace(_T("[UVersionM]"), _GetPlainResString(IDS_INFLST_USER_CLIENTVERSION)); }
	if (!WSuploadColumnHidden[2]){
		Out.Replace(_T("[UFilename]"), _GetPlainResString(IDS_DL_FILENAME));
		Out.Replace(_T("[UFilenameM]"), _GetPlainResString(IDS_DL_FILENAME)); }
	else {
		Out.Replace(_T("[UFilename]"), "");
		Out.Replace(_T("[UFilenameM]"), _GetPlainResString(IDS_DL_FILENAME)); }
	if (!WSuploadColumnHidden[3]){
		Out.Replace(_T("[UTransferred]"), _GetPlainResString(IDS_DL_ULDL));
		Out.Replace(_T("[UTransferredM]"), _GetPlainResString(IDS_DL_ULDL)); }
	else {
		Out.Replace(_T("[UTransferred]"), "");
		Out.Replace(_T("[UTransferredM]"), _GetPlainResString(IDS_DL_ULDL)); }
	if (!WSuploadColumnHidden[4]){
		Out.Replace(_T("[USpeed]"), _GetPlainResString(IDS_DL_SPEED));
		Out.Replace(_T("[USpeedM]"), _GetPlainResString(IDS_DL_SPEED)); }
	else {
		Out.Replace(_T("[USpeed]"), "");
		Out.Replace(_T("[USpeedM]"), _GetPlainResString(IDS_DL_SPEED)); }
	Out.Replace(_T("[DownloadList]"), _GetPlainResString(IDS_ST_ACTIVEDOWNLOAD));
	Out.Replace(_T("[UploadList]"), _GetPlainResString(IDS_ST_ACTIVEUPLOAD));
	Out.Replace(_T("[Actions]"), _GetPlainResString(IDS_WEB_ACTIONS));
	Out.Replace(_T("[TotalDown]"), _GetPlainResString(IDS_INFLST_USER_TOTALDOWNLOAD));
	Out.Replace(_T("[TotalUp]"), _GetPlainResString(IDS_INFLST_USER_TOTALUPLOAD));
	Out.Replace(_T("[CatSel]"), sCat);
	Out.Replace(_T("[admin]"), admin); //DonGato: category priority change
	
	CString OutE = pThis->m_Templates.sTransferDownLine;

	float fTotalSize = 0, fTotalTransferred = 0, fTotalSpeed = 0;
	CArray<DownloadFiles, DownloadFiles> FilesArray;
	//SyruS show completed files (0.28b)
	CArray<CPartFile*,CPartFile*> partlist;
	if (!theApp.emuledlg->transferwnd.m_wndDownloadList.GetDisplayedFiles(&partlist))	//failsafe
	{
		partlist.RemoveAll();
		// Populating array
		for (POSITION pos=theApp.downloadqueue->filelist.GetHeadPosition(); pos != 0; theApp.downloadqueue->filelist.GetNext(pos))
		{
			CPartFile* cur_file = theApp.downloadqueue->filelist.GetAt(pos);
			partlist.Add(cur_file);
		}
	}
	for (int i=0; i<partlist.GetCount(); i++) {
		CPartFile* cur_file=partlist.GetAt(i);
		if (cur_file) {
			//DonGato: update priorities on Auto files
			if(cur_file->IsAutoPrioritized())
				cur_file->UpdateDownloadAutoPriority();

			DownloadFiles dFile;
			dFile.sFileName = _SpecialChars(cur_file->GetFileName());
			dFile.sFileNameJS = _SpecialChars(cur_file->GetFileName(), true);	//for javascript
			dFile.lFileSize = cur_file->GetFileSize();
			dFile.lFileTransferred = cur_file->GetCompletedSize();
			dFile.fCompleted = cur_file->GetPercentCompleted();
			dFile.lFileSpeed = cur_file->GetDatarate();
			dFile.nFileStatus = cur_file->GetStatus();
			dFile.sFileStatus = cur_file->GetPartfileStatus();
			dFile.bFileAutoPrio = cur_file->IsAutoPrioritized();
			dFile.nFilePrio = cur_file->GetPriority();
			CString strCategory = (cur_file->GetCategory()!=0)?theApp.glob_prefs->GetCategory(cur_file->GetCategory())->title:GetResString(IDS_ALLOTHERS);
			strCategory.Replace(_T("'"),_T("\'"));
			dFile.sCategory = strCategory;			
			dFile.sFileHash = EncodeBase16(cur_file->GetFileHash(), 16);
			dFile.lSourceCount = cur_file->GetSourceCount();
			dFile.lNotCurrentSourceCount = cur_file->GetNotCurrentSourcesCount();
			dFile.lTransferringSourceCount = cur_file->GetTransferingSrcCount();
			dFile.bCompress = cur_file->GetDiscardSuperCompressed(); // Purity: DropSCB
#ifdef OLD_SOCKETS_ENABLED
			if (theApp.serverconnect->IsConnected() && !theApp.serverconnect->IsLowID())
				dFile.sED2kLink = cur_file->CreateED2kSourceLink();
			else
#endif //OLD_SOCKETS_ENABLED
			dFile.sED2kLink = cur_file->CreateED2kLink();
			dFile.sFileInfo = _SpecialChars(cur_file->GetDownloadFileInfo());
			if (cat>0 && cur_file->GetCategory()!=cat) continue;
			if (cat<0) {
				switch (cat) {
					case -1 : if (cur_file->GetCategory()!=0) continue; break;
					case -2 : if (!cur_file->IsPartFile()) continue; break;
					case -3 : if (cur_file->IsPartFile()) continue; break;
					case -4 : if (!((cur_file->GetStatus()==PS_READY|| cur_file->GetStatus()==PS_EMPTY) && cur_file->GetTransferingSrcCount()==0)) continue; break;
					case -5 : if (!((cur_file->GetStatus()==PS_READY|| cur_file->GetStatus()==PS_EMPTY) && cur_file->GetTransferingSrcCount()>0)) continue; break;
					case -6 : if (cur_file->GetStatus()!=PS_ERROR) continue; break;
					case -7 : if (cur_file->GetStatus()!=PS_PAUSED) continue; break;
					case -8 : if (cur_file->GetStatus()!=PS_STOPPED) continue; break;
					case -9 : if (!cur_file->IsMovie()) continue; break;
					case -10 : if (ED2KFT_AUDIO != GetED2KFileTypeID(cur_file->GetFileName())) continue; break;
					case -11 : if (!cur_file->IsArchive()) continue; break;
					case -12 : if (ED2KFT_CDIMAGE != GetED2KFileTypeID(cur_file->GetFileName())) continue; break;
				}
			}
			FilesArray.Add(dFile);
		}
	}

	// Sorting (simple bubble sort, we don't have tons of data here)
	bool bSorted = true;
	for(int nMax = 0;bSorted && nMax < FilesArray.GetCount()*2; nMax++)
	{
		bSorted = false;
		for(int i = 0; i < FilesArray.GetCount() - 1; i++)
		{
			bool bSwap = false;
			switch(pThis->m_Params.DownloadSort)
			{
			case DOWN_SORT_NAME:
				bSwap = FilesArray[i].sFileName.CompareNoCase(FilesArray[i+1].sFileName) > 0;
				break;
			case DOWN_SORT_SIZE:
				bSwap = FilesArray[i].lFileSize < FilesArray[i+1].lFileSize;
				break;
			case DOWN_SORT_TRANSFERRED:
				bSwap = FilesArray[i].lFileTransferred < FilesArray[i+1].lFileTransferred;
				break;
			case DOWN_SORT_SPEED:
				bSwap = FilesArray[i].lFileSpeed < FilesArray[i+1].lFileSpeed;
				break;
			case DOWN_SORT_PROGRESS:
				bSwap = FilesArray[i].fCompleted  < FilesArray[i+1].fCompleted ;
				break;
			case DOWN_SORT_SOURCES:
				bSwap = FilesArray[i].lSourceCount  < FilesArray[i+1].lSourceCount ;
				break;
			case DOWN_SORT_PRIORITY:
				bSwap = FilesArray[i].nFilePrio < FilesArray[i+1].nFilePrio ;
				break;
			case DOWN_SORT_CATEGORY:
				bSwap = FilesArray[i].sCategory.CompareNoCase(FilesArray[i+1].sCategory) > 0;
				break;
			}
			if(pThis->m_Params.bDownloadSortReverse)
			{
				bSwap = !bSwap;
			}
			if(bSwap)
			{
				bSorted = true;
				DownloadFiles TmpFile = FilesArray[i];
				FilesArray[i] = FilesArray[i+1];
				FilesArray[i+1] = TmpFile;
			}
		}
	}

	CArray<UploadUsers, UploadUsers> UploadArray;
	for (POSITION pos = theApp.uploadqueue->uploadinglist.GetHeadPosition(); pos != 0;theApp.uploadqueue->uploadinglist.GetNext(pos))
	{
		CUpDownClient* cur_client = theApp.uploadqueue->uploadinglist.GetAt(pos);
		UploadUsers dUser;
		CString sTemp;
		dUser.sUserHash = FileHashToString(cur_client->GetUserHash());
		if (cur_client->GetDatarate() > 0)
			dUser.sActive = _T("downloading");
		else
			dUser.sActive = _T("waiting");
		sTemp = _SpecialChars(cur_client->GetUploadFileInfo());
		sTemp.Replace(_T("\n"), _T("<br>"));
		sTemp.Replace(_T("'"), _T("&#8217;"));
		dUser.sFileInfo = sTemp;
		sTemp.Format(_T("%u"), cur_client->GetClientSoft());
		if ((cur_client->credits->GetCurrentIdentState(cur_client->GetIP()) != IS_IDENTIFIED) && (cur_client->GetClientSoft() == 0))
			sTemp = "x";
		dUser.sClientSoft = sTemp;
		if (cur_client->IsBanned())
			dUser.sClientExtra = _T("banned");
		else if (cur_client->IsFriend())
			dUser.sClientExtra = _T("friend");
		else if (cur_client->credits->GetScoreRatio(cur_client->GetIP()) > 1)
			dUser.sClientExtra = _T("credit");
		else
			dUser.sClientExtra = _T("none");
		if(cur_client->GetUserName().GetLength() > SHORT_LENGTH_MIN)
			dUser.sUserName = _SpecialChars(cur_client->GetUserName().Left(SHORT_LENGTH_MIN)) + _T("...");
		else
			dUser.sUserName = _SpecialChars(cur_client->GetUserName());
		CKnownFile* file = theApp.sharedfiles->GetFileByID(cur_client->reqfileid);
		if (file)
			dUser.sFileName = _SpecialChars(CString(file->GetFileName()));
		else
			dUser.sFileName = _GetPlainResString(IDS_REQ_UNKNOWNFILE);
		dUser.nTransferredDown = cur_client->GetTransferedDown();
		dUser.nTransferredUp = cur_client->GetTransferedUp();
		dUser.nDatarate = cur_client->GetDatarate();
		dUser.sClientNameVersion = GetClientNameAndVersionString(cur_client);
		UploadArray.Add(dUser);
	}
		
	// Sorting (simple bubble sort, we don't have tons of data here)
	bSorted = true;
	for(int nMax = 0;bSorted && nMax < UploadArray.GetCount()*2; nMax++)
	{
		bSorted = false;
		for(int i = 0; i < UploadArray.GetCount() - 1; i++)
		{
			bool bSwap = false;
			switch(pThis->m_Params.UploadSort)
			{
			case UP_SORT_USER:
				bSwap = UploadArray[i].sUserName.CompareNoCase(UploadArray[i+1].sUserName) > 0;
				break;
			case UP_SORT_VERSION:
				bSwap = UploadArray[i].sClientNameVersion.CompareNoCase(UploadArray[i+1].sClientNameVersion) > 0;
				break;
			case UP_SORT_FILENAME:
				bSwap = UploadArray[i].sFileName.CompareNoCase(UploadArray[i+1].sFileName) > 0;
				break;
			case UP_SORT_TRANSFERRED:
				bSwap = UploadArray[i].nTransferredUp < UploadArray[i+1].nTransferredUp ;
				break;
			case UP_SORT_SPEED:
				bSwap = UploadArray[i].nDatarate < UploadArray[i+1].nDatarate ;
				break;
			}
			if(pThis->m_Params.bUploadSortReverse)
			{
				bSwap = !bSwap;
			}
			if(bSwap)
			{
				bSorted = true;
				UploadUsers TmpUser = UploadArray[i];
				UploadArray[i] = UploadArray[i+1];
				UploadArray[i+1] = TmpUser;
			}
		}
	}

	int nCountQueue = 0;
	int nCountQueueBanned = 0;
	int nCountQueueFriend = 0;
	int nCountQueueCredit = 0;
	int nCountQueueSecure = 0;
	int nCountQueueBannedSecure = 0;
	int nCountQueueFriendSecure = 0;
	int nCountQueueCreditSecure = 0;

//	CArray<QueueUsers, QueueUsers> QueueArray;
	CQArray<QueueUsers, QueueUsers> QueueArray;
	for (POSITION pos = theApp.uploadqueue->waitinglist.GetHeadPosition(); pos != 0;theApp.uploadqueue->waitinglist.GetNext(pos))
	{
		CUpDownClient* cur_client = theApp.uploadqueue->waitinglist.GetAt(pos);
		QueueUsers dUser;
		CString sTemp;

		bool bSecure = (cur_client->credits->GetCurrentIdentState(cur_client->GetIP()) == IS_IDENTIFIED);
		if (cur_client->IsBanned())
		{
			dUser.sClientExtra = _T("banned");
			nCountQueueBanned++;
			if (bSecure) nCountQueueBannedSecure++;
		}
		else if (cur_client->IsFriend())
		{
			dUser.sClientExtra = _T("friend");
			nCountQueueFriend++;
			if (bSecure) nCountQueueFriendSecure++;
		}
		else if (cur_client->credits->GetScoreRatio(cur_client->GetIP()) > 1)
		{
			dUser.sClientExtra = _T("credit");
			nCountQueueCredit++;
			if (bSecure) nCountQueueCreditSecure++;
		}
		else
		{
			dUser.sClientExtra = _T("none");
			nCountQueue++;
			if (bSecure) nCountQueueSecure++;
		}

		if(cur_client->GetUserName().GetLength() > SHORT_LENGTH_MIN)
			dUser.sUserName = _SpecialChars(cur_client->GetUserName().Left(SHORT_LENGTH_MIN)) + _T("...");
		else
			dUser.sUserName = _SpecialChars(cur_client->GetUserName());
		dUser.sClientNameVersion = GetClientNameAndVersionString(cur_client);
		CKnownFile* file = theApp.sharedfiles->GetFileByID(cur_client->reqfileid);
		if (file)
			dUser.sFileName = _SpecialChars(CString(file->GetFileName()));
		else
			dUser.sFileName = _GetPlainResString(IDS_REQ_UNKNOWNFILE);
		dUser.nScore = cur_client->GetScore(false);
		sTemp.Format(_T("%u"), cur_client->GetClientSoft());
		if ((cur_client->credits->GetCurrentIdentState(cur_client->GetIP()) != IS_IDENTIFIED) && (cur_client->GetClientSoft() == 0))
			sTemp = "x";
		dUser.sClientSoft = sTemp;
		dUser.sUserHash = FileHashToString(cur_client->GetUserHash());

		//SyruS CQArray-Sorting setting sIndex according to param
		switch(pThis->m_Params.QueueSort)
		{
		case QU_SORT_USER:
			dUser.sIndex = dUser.sUserName;
			break;
		case QU_SORT_VERSION:
			dUser.sIndex = dUser.sClientNameVersion;
			break;
		case QU_SORT_FILENAME:
			dUser.sIndex = dUser.sFileName;
			break;
		case QU_SORT_SCORE:
			dUser.sIndex.Format(_T("%09u"), dUser.nScore);
			break;
		default:
			dUser.sIndex.Empty();
		}

		QueueArray.Add(dUser);
	}

#ifdef _DEBUG
	DWORD dwStart = ::GetTickCount();
#endif
	QueueArray.QuickSort(!pThis->m_Params.bQueueSortReverse);	//SyruS CQArray-Sorting execute
#ifdef _DEBUG
	AddLogLine(false,_T("WebServer: Waitingqueue with %u elements sorted in %u ms"), QueueArray.GetSize(), ::GetTickCount()-dwStart);
#endif

	uchar FileHashA4AF[16]; // Purity: Auto-A4AF

	// Displaying
	CString sDownList = _T("");
	for(int i = 0; i < FilesArray.GetCount(); i++)
	{
		CString HTTPProcessData;
		HTTPProcessData = OutE;

		if (FilesArray[i].sFileHash == _ParseURL(Data.sURL,_T("file")) )
            HTTPProcessData.Replace(_T("[LastChangedDataset]"), "checked");
		else
            HTTPProcessData.Replace(_T("[LastChangedDataset]"), "checked_no");

		CString admin = "";			//admin rights
		CString finfo = "";			//fileinfo
		CString ed2k = "";			//ed2klink
		CString state = "";			//state
		CString scb = "";			//dropscb
		CString autoa4af = "";		//autoa4af
		CString fname = "";			//filename
		CString session = "";		//session
		CString filehash = "";		//filehash
		CString downpriority = "";	//priority

		bool bIsComplete = false;

		// Purity: Auto-A4AF
		_GetFileHash(FilesArray[i].sFileHash, FileHashA4AF);
		CPartFile* found_file = theApp.downloadqueue->GetFileByID(FileHashA4AF);

		if(FilesArray[i].lFileSpeed > 0)
			state.Format(_T("downloading"));
		else
			state.Format(_T("waiting"));

		switch(FilesArray[i].nFileStatus)
		{
			case PS_HASHING: 
			case PS_WAITINGFORHASH:
				state.Format(_T("hashing"));
				break;
			case PS_ERROR:
				state.Format(_T("error"));
				break;
			case PS_COMPLETING:
				state.Format(_T("completing"));
				break;
			case PS_COMPLETE:
				bIsComplete = true;
				state.Format(_T("complete"));
			case PS_STOPPED:	
			case PS_PAUSED:
				if(!bIsComplete)
					state.Format(_T("paused"));
			default:
				{
					CString JSfileinfo=FilesArray[i].sFileInfo;
					JSfileinfo.Replace(_T("\n"),_T("\\n"));
					JSfileinfo.Replace(_T("%"),_T("%%"));
					JSfileinfo.Replace(_T("'"),_T("&#8217;")); // EC 22/5 - Chararacter ' causes a problem in javascript:alert()

					CString JSED2kLink=FilesArray[i].sED2kLink;
					JSED2kLink.Replace(_T("'"),_T("&#8217;"));

					finfo.Format(JSfileinfo);
					ed2k.Format(JSED2kLink);
					fname.Format(FilesArray[i].sFileNameJS);
					session.Format(CString(sSession));
					filehash.Format(FilesArray[i].sFileHash);

					if (FilesArray[i].bCompress && !bIsComplete)
						scb.Format(_T("scb"));
					
					if (theApp.downloadqueue->GetA4AFAutoFile() == found_file && !bIsComplete)
						autoa4af.Format(_T("autoa4af"));
				}
				break;
		}	//switch

		if (bAdmin)
			admin.Format(_T("admin"));

		//DonGato: priority change
		if(FilesArray[i].bFileAutoPrio)
			downpriority.Format(_T("Auto"));
		else
		{
			switch(FilesArray[i].nFilePrio)
			{
				case 0:
					downpriority.Format(_T("Low"));
					break;
				case 1:
					downpriority.Format(_T("Normal"));
					break;
				case 2:
					downpriority.Format(_T("High"));
					break;
			}
		}

		HTTPProcessData.Replace(_T("[admin]"), admin);
		HTTPProcessData.Replace(_T("[finfo]"), finfo);
		HTTPProcessData.Replace(_T("[ed2k]"), ed2k);
		HTTPProcessData.Replace(_T("[DownState]"), state);
		HTTPProcessData.Replace(_T("[scb]"), scb);
		HTTPProcessData.Replace(_T("[autoa4af]"), autoa4af);
		HTTPProcessData.Replace(_T("[fname]"), fname);
		HTTPProcessData.Replace(_T("[session]"), session);
		HTTPProcessData.Replace(_T("[filehash]"), filehash);
		HTTPProcessData.Replace(_T("[CatSel]"), sCat);
		HTTPProcessData.Replace(_T("[down-priority]"), downpriority); //DonGato: priority change

		// Purity: showing status for files (A4AF & SCB)
		if (FilesArray[i].bCompress && !bIsComplete)
			HTTPProcessData.Replace(_T("[FileIsSCB]"), _T("scb"));
		else
			HTTPProcessData.Replace(_T("[FileIsSCB]"), _T("halfnone"));

		if (theApp.downloadqueue->GetA4AFAutoFile() == found_file && !bIsComplete)
			HTTPProcessData.Replace(_T("[FileIsA4AF]"), _T("a4af"));
		else
			HTTPProcessData.Replace(_T("[FileIsA4AF]"), _T("halfnone"));

		if (!WSdownloadColumnHidden[0]) {
			if(FilesArray[i].sFileName.GetLength() > SHORT_LENGTH_MAX)
				HTTPProcessData.Replace(_T("[ShortFileName]"), FilesArray[i].sFileName.Left(SHORT_LENGTH_MAX) + _T("..."));
			else
				HTTPProcessData.Replace(_T("[ShortFileName]"), FilesArray[i].sFileName);
		} else
			HTTPProcessData.Replace(_T("[ShortFileName]"), "");

		CString sTooltip = FilesArray[i].sFileInfo;
		sTooltip.Replace(_T("\n"), _T("<br>"));
		sTooltip.Replace(_T("'"), _T("&#8217;"));
		HTTPProcessData.Replace(_T("[FileInfo]"), sTooltip);

		fTotalSize += FilesArray[i].lFileSize;

		if (!WSdownloadColumnHidden[1])
			HTTPProcessData.Replace(_T("[2]"), CastItoXBytes(FilesArray[i].lFileSize));
		else
			HTTPProcessData.Replace(_T("[2]"), "");

		if (!WSdownloadColumnHidden[2]) {
			if(FilesArray[i].lFileTransferred > 0)
			{
				fTotalTransferred += FilesArray[i].lFileTransferred;
				HTTPProcessData.Replace(_T("[3]"), CastItoXBytes(FilesArray[i].lFileTransferred));
			}
			else
				HTTPProcessData.Replace(_T("[3]"), _T("-"));
		} else
			HTTPProcessData.Replace(_T("[3]"), "");

		if (!WSdownloadColumnHidden[3])
			HTTPProcessData.Replace(_T("[DownloadBar]"), _GetDownloadGraph(Data,FilesArray[i].sFileHash));
		else
			HTTPProcessData.Replace(_T("[DownloadBar]"), "");

		HTTPTemp = "";

		if (!WSdownloadColumnHidden[4]) {
			if(FilesArray[i].lFileSpeed > 0.0f)
			{
				fTotalSpeed += FilesArray[i].lFileSpeed;
				HTTPTemp.Format(_T("%8.2f %s"), FilesArray[i].lFileSpeed/1024.0 ,_GetPlainResString(IDS_KBYTESEC));
				HTTPProcessData.Replace(_T("[4]"), HTTPTemp);
			}
			else
				HTTPProcessData.Replace(_T("[4]"), _T("-"));
		} else
			HTTPProcessData.Replace(_T("[4]"), "");

		if (!WSdownloadColumnHidden[5]) {
			if(FilesArray[i].lSourceCount > 0)
			{
				HTTPTemp.Format(_T("%i&nbsp;/&nbsp;%8i&nbsp;(%i)"),
			    FilesArray[i].lSourceCount-FilesArray[i].lNotCurrentSourceCount,
			    FilesArray[i].lSourceCount,
			    FilesArray[i].lTransferringSourceCount);
				HTTPProcessData.Replace(_T("[5]"), HTTPTemp);
			}
			else
				HTTPProcessData.Replace(_T("[5]"), _T("-"));
		} else
			HTTPProcessData.Replace(_T("[5]"), "");

		if (!WSdownloadColumnHidden[6])
		{
			if(FilesArray[i].bFileAutoPrio)
			{
				switch(FilesArray[i].nFilePrio)
				{
					case 0: HTTPTemp=GetResString(IDS_PRIOAUTOLOW); break;
					case 1: HTTPTemp=GetResString(IDS_PRIOAUTONORMAL); break;
					case 2: HTTPTemp=GetResString(IDS_PRIOAUTOHIGH); break;
				}
			}
			else
			{
				switch(FilesArray[i].nFilePrio)
				{
					case  0: HTTPTemp=GetResString(IDS_PRIOLOW); break;
					case  1: HTTPTemp=GetResString(IDS_PRIONORMAL); break;
					case  2: HTTPTemp=GetResString(IDS_PRIOHIGH); break;
				}
			}
			HTTPProcessData.Replace(_T("[PrioVal]"), HTTPTemp);
		} else
			HTTPProcessData.Replace(_T("[PrioVal]"), "");

		if (!WSdownloadColumnHidden[7])
			HTTPProcessData.Replace(_T("[Category]"), FilesArray[i].sCategory);
		else
			HTTPProcessData.Replace(_T("[Category]"), "");

		InsertCatBox(HTTPProcessData,0,"",false,false,sSession,FilesArray[i].sFileHash);

		sDownList += HTTPProcessData;
	}

	Out.Replace(_T("[DownloadFilesList]"), sDownList);
	Out.Replace(_T("[TotalDownSize]"), CastItoXBytes(fTotalSize));

	Out.Replace(_T("[TotalDownTransferred]"), CastItoXBytes(fTotalTransferred));
	
	HTTPTemp.Format(_T("%8.2f %s"), fTotalSpeed/1024.0,_GetPlainResString(IDS_KBYTESEC));
	Out.Replace(_T("[TotalDownSpeed]"), HTTPTemp);
	OutE = pThis->m_Templates.sTransferUpLine;

	HTTPTemp.Format(_T("%i"),pThis->m_Templates.iProgressbarWidth);
	Out.Replace(_T("[PROGRESSBARWIDTHVAL]"),HTTPTemp);

	fTotalSize = 0;
	fTotalTransferred = 0;
	fTotalSpeed = 0;

	CString sUpList = _T("");
	for(int i = 0; i < UploadArray.GetCount(); i++)
	{
		CString HTTPProcessData;
		HTTPProcessData = OutE;

		HTTPProcessData.Replace(_T("[UserHash]"), UploadArray[i].sUserHash);
		bool bAdmin = IsSessionAdmin(Data,sSession);
		CString admin;
		if (bAdmin)
			admin.Format(_T("admin"));
		HTTPProcessData.Replace(_T("[admin]"), admin);
		HTTPProcessData.Replace(_T("[UpState]"), UploadArray[i].sActive);
		HTTPProcessData.Replace(_T("[FileInfo]"), UploadArray[i].sFileInfo);
		HTTPProcessData.Replace(_T("[ClientSoft]"), UploadArray[i].sClientSoft);
		HTTPProcessData.Replace(_T("[ClientExtra]"), UploadArray[i].sClientExtra);
		if (!WSuploadColumnHidden[0])
			HTTPProcessData.Replace(_T("[1]"), UploadArray[i].sUserName);
		else
			HTTPProcessData.Replace(_T("[1]"), "");
		if (!WSuploadColumnHidden[1])
			HTTPProcessData.Replace(_T("[ClientSoftV]"), UploadArray[i].sClientNameVersion);
		else
			HTTPProcessData.Replace(_T("[ClientSoftV]"), "");
		if (!WSuploadColumnHidden[2])
			HTTPProcessData.Replace(_T("[2]"), UploadArray[i].sFileName);
		else
			HTTPProcessData.Replace(_T("[2]"), "");
		if (!WSuploadColumnHidden[3]) {
			fTotalSize += UploadArray[i].nTransferredDown;
			fTotalTransferred += UploadArray[i].nTransferredUp;
			HTTPTemp.Format(_T("%s / %s"), CastItoXBytes(UploadArray[i].nTransferredDown),CastItoXBytes(UploadArray[i].nTransferredUp));
			HTTPProcessData.Replace(_T("[3]"), HTTPTemp); }
		else
			HTTPProcessData.Replace(_T("[3]"), "");
		if (!WSuploadColumnHidden[4]) {
			fTotalSpeed += UploadArray[i].nDatarate;
			HTTPTemp.Format(_T("%8.2f ") + _GetPlainResString(IDS_KBYTESEC), max((float)UploadArray[i].nDatarate/1024, 0.0));
			HTTPProcessData.Replace(_T("[4]"), HTTPTemp); }
		else
			HTTPProcessData.Replace(_T("[4]"), "");

		sUpList += HTTPProcessData;
	}
	Out.Replace(_T("[UploadFilesList]"), sUpList);
	HTTPTemp.Format(_T("%s / %s"), CastItoXBytes(fTotalSize), CastItoXBytes(fTotalTransferred));
	Out.Replace(_T("[TotalUpTransferred]"), HTTPTemp);
	HTTPTemp.Format(_T("%8.2f ") + _GetPlainResString(IDS_KBYTESEC), max(fTotalSpeed/1024, 0.0));
	Out.Replace(_T("[TotalUpSpeed]"), HTTPTemp);

	if(pThis->m_Params.bShowUploadQueue) 
	{
		Out.Replace(_T("[UploadQueue]"), pThis->m_Templates.sTransferUpQueueShow);
		Out.Replace(_T("[UploadQueueList]"), _GetPlainResString(IDS_ONQUEUE));
		CString OutE = pThis->m_Templates.sTransferUpQueueLine;
		CString sQueue = _T("");
		for(int i = 0; i < QueueArray.GetCount(); i++)
		{
			CString HTTPProcessData;
            TCHAR HTTPTempC[100] = _T("");
			if (QueueArray[i].sClientExtra == "none")
				HTTPProcessData = OutE;
			if (!WSqueueColumnHidden[0])
				HTTPProcessData.Replace(_T("[UserName]"), QueueArray[i].sUserName);
			else
				HTTPProcessData.Replace(_T("[UserName]"), "");
			if (!WSqueueColumnHidden[1])
				HTTPProcessData.Replace(_T("[ClientSoftV]"), QueueArray[i].sClientNameVersion);
			else
				HTTPProcessData.Replace(_T("[ClientSoftV]"), "");
			if (!WSqueueColumnHidden[2])
				HTTPProcessData.Replace(_T("[FileName]"), QueueArray[i].sFileName);
			else
				HTTPProcessData.Replace(_T("[FileName]"), "");
			if (!WSqueueColumnHidden[3]) {
				_stprintf(HTTPTempC, _T("%i") , QueueArray[i].nScore);
				HTTPProcessData.Replace(_T("[Score]"), HTTPTempC); }
			else
				HTTPProcessData.Replace(_T("[Score]"), "");
			HTTPProcessData.Replace(_T("[ClientSoft]"), QueueArray[i].sClientSoft);
			HTTPProcessData.Replace(_T("[ClientExtra]"), QueueArray[i].sClientExtra);
			HTTPProcessData.Replace(_T("[UserHash]"), QueueArray[i].sUserHash);
			bool bAdmin = IsSessionAdmin(Data,sSession);
			CString admin;
			if (bAdmin)
				admin.Format(_T("admin"));
			HTTPProcessData.Replace(_T("[admin]"), admin);
			sQueue += HTTPProcessData;
		}
		Out.Replace(_T("[QueueList]"), sQueue);
	}
	else {
		Out.Replace(_T("[UploadQueue]"), pThis->m_Templates.sTransferUpQueueHide);
	}

	if(pThis->m_Params.bShowUploadQueueBanned) 
	{
		Out.Replace(_T("[UploadQueueBanned]"), pThis->m_Templates.sTransferUpQueueBannedShow);
		Out.Replace(_T("[UploadQueueBannedList]"), _GetPlainResString(IDS_ONQUEUEBANNED));
		CString OutE2 = pThis->m_Templates.sTransferUpQueueBannedLine;
		CString sQueueBanned = _T("");
		for(int i = 0; i < QueueArray.GetCount(); i++)
		{
			CString HTTPProcessData;
            TCHAR HTTPTempC[100] = _T("");
			if (QueueArray[i].sClientExtra == "banned")
				HTTPProcessData = OutE2;
			if (!WSqueueColumnHidden[0])
				HTTPProcessData.Replace(_T("[UserName]"), QueueArray[i].sUserName);
			else
				HTTPProcessData.Replace(_T("[UserName]"), "");
			if (!WSqueueColumnHidden[1])
				HTTPProcessData.Replace(_T("[ClientSoftV]"), QueueArray[i].sClientNameVersion);
			else
				HTTPProcessData.Replace(_T("[ClientSoftV]"), "");
			if (!WSqueueColumnHidden[2])
				HTTPProcessData.Replace(_T("[FileName]"), QueueArray[i].sFileName);
			else
				HTTPProcessData.Replace(_T("[FileName]"), "");
			if (!WSqueueColumnHidden[3]) {
				_stprintf(HTTPTempC, _T("%i") , QueueArray[i].nScore);
				HTTPProcessData.Replace(_T("[Score]"), HTTPTempC); }
			else
				HTTPProcessData.Replace(_T("[Score]"), "");
			HTTPProcessData.Replace(_T("[ClientSoft]"), QueueArray[i].sClientSoft);
			HTTPProcessData.Replace(_T("[ClientExtra]"), QueueArray[i].sClientExtra);
			HTTPProcessData.Replace(_T("[UserHash]"), QueueArray[i].sUserHash);
			bool bAdmin = IsSessionAdmin(Data,sSession);
			CString admin;
			if (bAdmin)
				admin.Format(_T("admin"));
			HTTPProcessData.Replace(_T("[admin]"), admin);
			sQueueBanned += HTTPProcessData;
		}
		Out.Replace(_T("[QueueListBanned]"), sQueueBanned);
	}
	else {
		Out.Replace(_T("[UploadQueueBanned]"), pThis->m_Templates.sTransferUpQueueBannedHide);
	}

	if(pThis->m_Params.bShowUploadQueueFriend) 
	{
		Out.Replace(_T("[UploadQueueFriend]"), pThis->m_Templates.sTransferUpQueueFriendShow);
		Out.Replace(_T("[UploadQueueFriendList]"), _GetPlainResString(IDS_ONQUEUEFRIEND));
		CString OutE3 = pThis->m_Templates.sTransferUpQueueFriendLine;
		CString sQueueFriend = _T("");
		for(int i = 0; i < QueueArray.GetCount(); i++)
		{
			CString HTTPProcessData;
            TCHAR HTTPTempC[100] = _T("");
			if (QueueArray[i].sClientExtra == "friend")
				HTTPProcessData = OutE3;
			if (!WSqueueColumnHidden[0])
				HTTPProcessData.Replace(_T("[UserName]"), QueueArray[i].sUserName);
			else
				HTTPProcessData.Replace(_T("[UserName]"), "");
			if (!WSqueueColumnHidden[1])
				HTTPProcessData.Replace(_T("[ClientSoftV]"), QueueArray[i].sClientNameVersion);
			else
				HTTPProcessData.Replace(_T("[ClientSoftV]"), "");
			if (!WSqueueColumnHidden[2])
				HTTPProcessData.Replace(_T("[FileName]"), QueueArray[i].sFileName);
			else
				HTTPProcessData.Replace(_T("[FileName]"), "");
			if (!WSqueueColumnHidden[3]) {
				_stprintf(HTTPTempC, _T("%i") , QueueArray[i].nScore);
				HTTPProcessData.Replace(_T("[Score]"), HTTPTempC); }
			else
				HTTPProcessData.Replace(_T("[Score]"), "");
			HTTPProcessData.Replace(_T("[ClientSoft]"), QueueArray[i].sClientSoft);
			HTTPProcessData.Replace(_T("[ClientExtra]"), QueueArray[i].sClientExtra);
			HTTPProcessData.Replace(_T("[UserHash]"), QueueArray[i].sUserHash);
			bool bAdmin = IsSessionAdmin(Data,sSession);
			CString admin;
			if (bAdmin)
				admin.Format(_T("admin"));
			HTTPProcessData.Replace(_T("[admin]"), admin);
			sQueueFriend += HTTPProcessData;
		}
		Out.Replace(_T("[QueueListFriend]"), sQueueFriend);
	}
	else {
		Out.Replace(_T("[UploadQueueFriend]"), pThis->m_Templates.sTransferUpQueueFriendHide);
	}

	if(pThis->m_Params.bShowUploadQueueCredit) 
	{
		Out.Replace(_T("[UploadQueueCredit]"), pThis->m_Templates.sTransferUpQueueCreditShow);
		Out.Replace(_T("[UploadQueueCreditList]"), _GetPlainResString(IDS_ONQUEUECREDIT));
		CString OutE4 = pThis->m_Templates.sTransferUpQueueCreditLine;
		CString sQueueCredit = _T("");
		for(int i = 0; i < QueueArray.GetCount(); i++)
		{
			CString HTTPProcessData;
            TCHAR HTTPTempC[100] = _T("");
			if (QueueArray[i].sClientExtra == "credit")
				HTTPProcessData = OutE4;
			if (!WSqueueColumnHidden[0])
				HTTPProcessData.Replace(_T("[UserName]"), QueueArray[i].sUserName);
			else
				HTTPProcessData.Replace(_T("[UserName]"), "");
			if (!WSqueueColumnHidden[1])
				HTTPProcessData.Replace(_T("[ClientSoftV]"), QueueArray[i].sClientNameVersion);
			else
				HTTPProcessData.Replace(_T("[ClientSoftV]"), "");
			if (!WSqueueColumnHidden[2])
				HTTPProcessData.Replace(_T("[FileName]"), QueueArray[i].sFileName);
			else
				HTTPProcessData.Replace(_T("[FileName]"), "");
			if (!WSqueueColumnHidden[3]) {
				_stprintf(HTTPTempC, _T("%i") , QueueArray[i].nScore);
				HTTPProcessData.Replace(_T("[Score]"), HTTPTempC); }
			else
				HTTPProcessData.Replace(_T("[Score]"), "");
			HTTPProcessData.Replace(_T("[ClientSoft]"), QueueArray[i].sClientSoft);
			HTTPProcessData.Replace(_T("[ClientExtra]"), QueueArray[i].sClientExtra);
			HTTPProcessData.Replace(_T("[UserHash]"), QueueArray[i].sUserHash);
			bool bAdmin = IsSessionAdmin(Data,sSession);
			CString admin;
			if (bAdmin)
				admin.Format(_T("admin"));
			HTTPProcessData.Replace(_T("[admin]"), admin);
			sQueueCredit += HTTPProcessData;
		}
		Out.Replace(_T("[QueueListCredit]"), sQueueCredit);
	}
	else {
		Out.Replace(_T("[UploadQueueCredit]"), pThis->m_Templates.sTransferUpQueueCreditHide);
	}
	CString mCounter;
	mCounter.Format(_T("%i"),nCountQueue);
	Out.Replace(_T("[CounterQueue]"), mCounter);
	mCounter.Format(_T("%i"),nCountQueueBanned);
	Out.Replace(_T("[CounterQueueBanned]"), mCounter);
	mCounter.Format(_T("%i"),nCountQueueFriend);
	Out.Replace(_T("[CounterQueueFriend]"), mCounter);
	mCounter.Format(_T("%i"),nCountQueueCredit);
	Out.Replace(_T("[CounterQueueCredit]"), mCounter);
	mCounter.Format(_T("%i"),nCountQueueSecure);
	Out.Replace(_T("[CounterQueueSecure]"), mCounter);
	mCounter.Format(_T("%i"),nCountQueueBannedSecure);
	Out.Replace(_T("[CounterQueueBannedSecure]"), mCounter);
	mCounter.Format(_T("%i"),nCountQueueFriendSecure);
	Out.Replace(_T("[CounterQueueFriendSecure]"), mCounter);
	mCounter.Format(_T("%i"),nCountQueueCreditSecure);
	Out.Replace(_T("[CounterQueueCreditSecure]"), mCounter);
	mCounter.Format(_T("%i"),nCountQueue+nCountQueueBanned+nCountQueueFriend+nCountQueueCredit);
	Out.Replace(_T("[CounterAll]"), mCounter);
	mCounter.Format(_T("%i"),nCountQueueSecure+nCountQueueBannedSecure+nCountQueueFriendSecure+nCountQueueCreditSecure);
	Out.Replace(_T("[CounterAllSecure]"), mCounter);
	Out.Replace(_T("[CatSel]"), sCat);
	Out.Replace(_T("[Session]"), sSession);
	Out.Replace(_T("[ShowUploadQueue]"), _GetPlainResString(IDS_VIEWQUEUE));
	Out.Replace(_T("[ShowUploadQueueList]"), _GetPlainResString(IDS_WEB_SHOW_UPLOAD_QUEUE));
	Out.Replace(_T("[ShowUploadQueueListBanned]"), _GetPlainResString(IDS_WEB_SHOW_UPLOAD_QUEUE_BANNED));
	Out.Replace(_T("[ShowUploadQueueListFriend]"), _GetPlainResString(IDS_WEB_SHOW_UPLOAD_QUEUE_FRIEND));
	Out.Replace(_T("[ShowUploadQueueListCredit]"), _GetPlainResString(IDS_WEB_SHOW_UPLOAD_QUEUE_CREDIT));
	if(pThis->m_Params.QueueSort == QU_SORT_USER)
		Out.Replace(_T("[SortQUser]"), _T("&sortreverse=") + sQueueSortRev);
	else
		Out.Replace(_T("[SortQUser]"), _T(""));
	if(pThis->m_Params.QueueSort == QU_SORT_VERSION)
		Out.Replace(_T("[SortQVersion]"), _T("&sortreverse=") + sQueueSortRev);
	else
		Out.Replace(_T("[SortQVersion]"), _T(""));
	if(pThis->m_Params.QueueSort == QU_SORT_FILENAME)
		Out.Replace(_T("[SortQFilename]"), _T("&sortreverse=") + sQueueSortRev);
	else
		Out.Replace(_T("[SortQFilename]"), _T(""));
	if(pThis->m_Params.QueueSort == QU_SORT_SCORE)
		Out.Replace(_T("[SortQScore]"), _T("&sortreverse=") + sQueueSortRev);
	else
		Out.Replace(_T("[SortQScore]"), _T(""));

	if (!WSqueueColumnHidden[0]) {
		Out.Replace(_T("[UserNameTitle]"), _GetPlainResString(IDS_QL_USERNAME));
		Out.Replace(_T("[UserNameTitleM]"), _GetPlainResString(IDS_QL_USERNAME)); }
	else {
		Out.Replace(_T("[UserNameTitle]"), "");
		Out.Replace(_T("[UserNameTitleM]"), _GetPlainResString(IDS_QL_USERNAME)); }
	if (!WSqueueColumnHidden[1]) {
		Out.Replace(_T("[Version]"), _GetPlainResString(IDS_INFLST_USER_CLIENTVERSION));
		Out.Replace(_T("[VersionM]"), _GetPlainResString(IDS_INFLST_USER_CLIENTVERSION)); }
	else {
		Out.Replace(_T("[Version]"), "");
		Out.Replace(_T("[VersionM]"), _GetPlainResString(IDS_INFLST_USER_CLIENTVERSION)); }
	if (!WSqueueColumnHidden[2]) {
		Out.Replace(_T("[FileNameTitle]"), _GetPlainResString(IDS_DL_FILENAME));
		Out.Replace(_T("[FileNameTitleM]"), _GetPlainResString(IDS_DL_FILENAME)); }
	else {
		Out.Replace(_T("[FileNameTitle]"), "");
		Out.Replace(_T("[FileNameTitleM]"), _GetPlainResString(IDS_DL_FILENAME)); }
	if (!WSqueueColumnHidden[3]) {
		Out.Replace(_T("[ScoreTitle]"), _GetPlainResString(IDS_SCORE));
		Out.Replace(_T("[ScoreTitleM]"), _GetPlainResString(IDS_SCORE)); }
	else {
		Out.Replace(_T("[ScoreTitle]"), "");
		Out.Replace(_T("[ScoreTitleM]"), _GetPlainResString(IDS_SCORE)); }
	return Out;

	EMULE_CATCH

	return _T("");
}

CString CWebServer::_GetSharedFilesList(ThreadData Data)
{
	EMULE_TRY

	CWebServer *pThis = (CWebServer *)Data.pThis;
	if(pThis == NULL)
		return _T("");

	CString HTTPTemp;

	CString sSession = _ParseURL(Data.sURL, _T("ses"));
	bool bAdmin = IsSessionAdmin(Data,sSession);

	if (_ParseURL(Data.sURL, _T("sort")) != "") 
	{
		if(_ParseURL(Data.sURL, _T("sort")) == "name")
			pThis->m_Params.SharedSort = SHARED_SORT_NAME;
		else
		if(_ParseURL(Data.sURL, _T("sort")) == "size")
			pThis->m_Params.SharedSort = SHARED_SORT_SIZE;
		else
		if(_ParseURL(Data.sURL, _T("sort")) == "transferred")
			pThis->m_Params.SharedSort = SHARED_SORT_TRANSFERRED;
		else
		if(_ParseURL(Data.sURL, _T("sort")) == "alltimetransferred")
			pThis->m_Params.SharedSort = SHARED_SORT_ALL_TIME_TRANSFERRED;
		else
		if(_ParseURL(Data.sURL, _T("sort")) == "requests")
			pThis->m_Params.SharedSort = SHARED_SORT_REQUESTS;
		else
		if(_ParseURL(Data.sURL, _T("sort")) == "alltimerequests")
			pThis->m_Params.SharedSort = SHARED_SORT_ALL_TIME_REQUESTS;
		else
		if(_ParseURL(Data.sURL, _T("sort")) == "accepts")
			pThis->m_Params.SharedSort = SHARED_SORT_ACCEPTS;
		else
		if(_ParseURL(Data.sURL, _T("sort")) == "alltimeaccepts")
			pThis->m_Params.SharedSort = SHARED_SORT_ALL_TIME_ACCEPTS;
		else
		if(_ParseURL(Data.sURL, _T("sort")) == "priority")
			pThis->m_Params.SharedSort = SHARED_SORT_PRIORITY;

		if(_ParseURL(Data.sURL, _T("sortreverse")) == "")
			pThis->m_Params.bSharedSortReverse = false;
	}
	if (_ParseURL(Data.sURL, _T("sortreverse")) != "") 
		pThis->m_Params.bSharedSortReverse = (_ParseURL(Data.sURL, _T("sortreverse")) == "true");

	//DonGato: priority change
	if(_ParseURL(Data.sURL, _T("hash")) != "" && _ParseURL(Data.sURL, _T("prio")) != "" && IsSessionAdmin(Data,sSession))
	{
		CKnownFile* cur_file;
		uchar fileid[16];
		CString hash = _ParseURL(Data.sURL, _T("hash"));
		DecodeBase16(hash.GetBuffer(),hash.GetLength(),fileid);

		cur_file=theApp.sharedfiles->GetFileByID(fileid);

		if (cur_file!=0)
		{
			if(_ParseURL(Data.sURL, _T("prio")) == "verylow")
			{
				cur_file->SetAutoPriority(false);
				cur_file->SetPriority(PR_VERYLOW);
			}
			if(_ParseURL(Data.sURL, _T("prio")) == "low")
			{
				cur_file->SetAutoPriority(false);
				cur_file->SetPriority(PR_LOW);
			}
			if(_ParseURL(Data.sURL, _T("prio")) == "normal")
			{
				cur_file->SetAutoPriority(false);
				cur_file->SetPriority(PR_NORMAL);
			}
			if(_ParseURL(Data.sURL, _T("prio")) == "high")
			{
				cur_file->SetAutoPriority(false);
				cur_file->SetPriority(PR_HIGH);
			}
			if(_ParseURL(Data.sURL, _T("prio")) == "release")
			{
				cur_file->SetAutoPriority(false);
				cur_file->SetPriority(PR_VERYHIGH);
			}
			else if(_ParseURL(Data.sURL, _T("prio")) == "auto" && cur_file->IsPartFile())
			{
				cur_file->SetAutoPriority(true);
				cur_file->UpdateUploadAutoPriority();
			}

			theApp.sharedfiles->UpdateItem(cur_file); // DonGato: refresh GUI on priority change
		}
	}

	//+++
	CString sCmd = _ParseURL(Data.sURL, _T("c"));
	if (sCmd.CompareNoCase(_T("menu")) == 0) 
	{
		int iMenu = atoi(_ParseURL(Data.sURL, _T("m")));
		bool bValue = atoi(_ParseURL(Data.sURL, _T("v")));
		WSsharedColumnHidden[iMenu] = bValue;
		CString buffer;
		CString fullpath;
		fullpath.Format(_T("%spreferences.ini"),theApp.glob_prefs->GetConfigDir());
		CIni ini( fullpath );
		ini.SetArray(WSsharedColumnHidden, ELEMENT_COUNT(WSsharedColumnHidden), _T("sharedColumnHidden"), _T("WebServer")); 
		ini.SaveAndClose(); 
	}
	if (bAdmin && _ParseURL(Data.sURL, _T("jumpstart")) == "true")
		_SetSharedFileJumpstart(_ParseURL(Data.sURL, _T("hash")));
	else if (bAdmin &&  _ParseURL(Data.sURL, _T("jumpstart")) == "false")
		_SetSharedFileNoJumpstart(_ParseURL(Data.sURL, _T("hash")));

	if(_ParseURL(Data.sURL, _T("reload")) == "true")
		theApp.emuledlg->SendMessage(WEB_SHARED_FILES_RELOAD);

	CString sSharedSortRev;
	if(pThis->m_Params.bSharedSortReverse)
		sSharedSortRev = "false";
	else
		sSharedSortRev = "true";
    //Name sorting link
	CString Out = pThis->m_Templates.sSharedList;
	if(pThis->m_Params.SharedSort == SHARED_SORT_NAME)
		Out.Replace(_T("[SortName]"), _T("sort=name&sortreverse=") + sSharedSortRev);
	else
		Out.Replace(_T("[SortName]"), _T("sort=name"));
	//Size sorting Link
    if(pThis->m_Params.SharedSort == SHARED_SORT_SIZE)
		Out.Replace(_T("[SortSize]"), _T("sort=size&sortreverse=") + sSharedSortRev);
	else
		Out.Replace(_T("[SortSize]"), _T("sort=size"));
	//Priority sorting Link
    if(pThis->m_Params.SharedSort == SHARED_SORT_PRIORITY)
		Out.Replace(_T("[SortPriority]"), _T("sort=priority&sortreverse=") + sSharedSortRev);
	else
		Out.Replace(_T("[SortPriority]"), _T("sort=priority"));
    //Transferred sorting link
	if(pThis->m_Params.SharedSort == SHARED_SORT_TRANSFERRED)
	{
		if(pThis->m_Params.bSharedSortReverse)
            Out.Replace(_T("[SortTransferred]"), _T("sort=alltimetransferred&sortreverse=") + sSharedSortRev);
		else
			Out.Replace(_T("[SortTransferred]"), _T("sort=transferred&sortreverse=") + sSharedSortRev);
	}
	else
	if(pThis->m_Params.SharedSort == SHARED_SORT_ALL_TIME_TRANSFERRED)
	{
		if(pThis->m_Params.bSharedSortReverse)
            Out.Replace(_T("[SortTransferred]"), _T("sort=transferred&sortreverse=") + sSharedSortRev);
		else
			Out.Replace(_T("[SortTransferred]"), _T("sort=alltimetransferred&sortreverse=") + sSharedSortRev);
	}
	else
        Out.Replace(_T("[SortTransferred]"), _T("&sort=transferred&sortreverse=false"));
    //Request sorting link
	if(pThis->m_Params.SharedSort == SHARED_SORT_REQUESTS)
	{
		if(pThis->m_Params.bSharedSortReverse)
            Out.Replace(_T("[SortRequests]"), _T("sort=alltimerequests&sortreverse=") + sSharedSortRev);
		else
			Out.Replace(_T("[SortRequests]"), _T("sort=requests&sortreverse=") + sSharedSortRev);
	}
	else
	if(pThis->m_Params.SharedSort == SHARED_SORT_ALL_TIME_REQUESTS)
	{
		if(pThis->m_Params.bSharedSortReverse)
            Out.Replace(_T("[SortRequests]"), _T("sort=requests&sortreverse=") + sSharedSortRev);
		else
			Out.Replace(_T("[SortRequests]"), _T("sort=alltimerequests&sortreverse=") + sSharedSortRev);
	}
	else
        Out.Replace(_T("[SortRequests]"), _T("&sort=requests&sortreverse=false"));
    //Accepts sorting link
	if(pThis->m_Params.SharedSort == SHARED_SORT_ACCEPTS)
	{
		if(pThis->m_Params.bSharedSortReverse)
            Out.Replace(_T("[SortAccepts]"), _T("sort=alltimeaccepts&sortreverse=") + sSharedSortRev);
		else
			Out.Replace(_T("[SortAccepts]"), _T("sort=accepts&sortreverse=") + sSharedSortRev);
	}
	else
	if(pThis->m_Params.SharedSort == SHARED_SORT_ALL_TIME_ACCEPTS)
	{
		if(pThis->m_Params.bSharedSortReverse)
            Out.Replace(_T("[SortAccepts]"), _T("sort=accepts&sortreverse=") + sSharedSortRev);
		else
			Out.Replace(_T("[SortAccepts]"), _T("sort=alltimeaccepts&sortreverse=") + sSharedSortRev);
	}
	else
        Out.Replace(_T("[SortAccepts]"), _T("&sort=accepts&sortreverse=false"));

	if(_ParseURL(Data.sURL, _T("reload")) == "true")
	{
		CString resultlog = _SpecialChars(theApp.emuledlg->logtext);	//Pick-up last line of the log
		resultlog = resultlog.TrimRight('\n');
		resultlog = resultlog.Mid(resultlog.ReverseFind('\n'));
		Out.Replace(_T("[Message]"),resultlog);
	}
	else
        Out.Replace(_T("[Message]"),_T(""));

	if (!WSsharedColumnHidden[0]){
		Out.Replace(_T("[Filename]"), _GetPlainResString(IDS_DL_FILENAME));
		Out.Replace(_T("[FilenameM]"), _GetPlainResString(IDS_DL_FILENAME)); }
	else {
		Out.Replace(_T("[Filename]"), "");
		Out.Replace(_T("[FilenameM]"), _GetPlainResString(IDS_DL_FILENAME)); }
	if (!WSsharedColumnHidden[1]){
		Out.Replace(_T("[FileTransferred]"),  _GetPlainResString(IDS_SF_TRANSFERRED));
		Out.Replace(_T("[FileTransferredM]"),  _GetPlainResString(IDS_SF_TRANSFERRED)); }
	else {
		Out.Replace(_T("[FileTransferred]"),  "");
		Out.Replace(_T("[FileTransferredM]"),  _GetPlainResString(IDS_SF_TRANSFERRED)); }
	if (!WSsharedColumnHidden[2]){
		Out.Replace(_T("[FileRequests]"),  _GetPlainResString(IDS_SF_REQUESTS));
		Out.Replace(_T("[FileRequestsM]"),  _GetPlainResString(IDS_SF_REQUESTS)); }
	else {
		Out.Replace(_T("[FileRequests]"),  "");
		Out.Replace(_T("[FileRequestsM]"),  _GetPlainResString(IDS_SF_REQUESTS)); }
	if (!WSsharedColumnHidden[3]){
		Out.Replace(_T("[FileAccepts]"),  _GetPlainResString(IDS_SF_ACCEPTS));
		Out.Replace(_T("[FileAcceptsM]"),  _GetPlainResString(IDS_SF_ACCEPTS)); }
	else {
		Out.Replace(_T("[FileAccepts]"),  "");
		Out.Replace(_T("[FileAcceptsM]"),  _GetPlainResString(IDS_SF_ACCEPTS)); }
	if (!WSsharedColumnHidden[4]){
		Out.Replace(_T("[Size]"), _GetPlainResString(IDS_DL_SIZE));
		Out.Replace(_T("[SizeM]"), _GetPlainResString(IDS_DL_SIZE)); }
	else {
		Out.Replace(_T("[Size]"), "");
		Out.Replace(_T("[SizeM]"), _GetPlainResString(IDS_DL_SIZE)); }
	if (!WSsharedColumnHidden[5]){
		Out.Replace(_T("[Priority]"),  _GetPlainResString(IDS_PRIORITY));
		Out.Replace(_T("[PriorityM]"),  _GetPlainResString(IDS_PRIORITY)); }
	else {
		Out.Replace(_T("[Priority]"),  "");
		Out.Replace(_T("[PriorityM]"),  _GetPlainResString(IDS_PRIORITY)); }
	Out.Replace(_T("[Actions]"), _GetPlainResString(IDS_WEB_ACTIONS));
	Out.Replace(_T("[Reload]"), _GetPlainResString(IDS_SF_RELOAD));
	Out.Replace(_T("[Session]"), sSession);
	Out.Replace(_T("[SharedList]"), _GetPlainResString(IDS_SHAREDFILES));

	CString OutE = pThis->m_Templates.sSharedLine; 

	CArray<SharedFiles, SharedFiles> SharedArray;
	// Populating array
	for (POSITION pos = theApp.sharedfiles->m_Files_map.GetStartPosition();pos != 0;)
	{
		CCKey bufKey;
		CKnownFile* cur_file;
		CString buffer;
		
		theApp.sharedfiles->m_Files_map.GetNextAssoc(pos,bufKey,cur_file);

		SharedFiles dFile;
		//dFile.sFileName = _SpecialChars(cur_file->GetFileName());
		dFile.bIsPartFile = cur_file->IsPartFile();
		dFile.sFileName = cur_file->GetFileName();
		dFile.lFileSize = cur_file->GetFileSize();
#ifdef OLD_SOCKETS_ENABLED
		if (theApp.serverconnect->IsConnected()) 
			dFile.sED2kLink = cur_file->CreateED2kSourceLink();
		else
#endif //OLD_SOCKETS_ENABLED
            dFile.sED2kLink = cur_file->CreateED2kLink();

		dFile.nFileTransferred = cur_file->statistic.GetTransfered();
		dFile.nFileAllTimeTransferred = cur_file->statistic.GetAllTimeTransfered();
		dFile.nFileRequests = cur_file->statistic.GetRequests();
		dFile.nFileAllTimeRequests = cur_file->statistic.GetAllTimeRequests();
		dFile.nFileAccepts = cur_file->statistic.GetAccepts();
		dFile.nFileAllTimeAccepts = cur_file->statistic.GetAllTimeAccepts();

		dFile.sFileHash = EncodeBase16(cur_file->GetFileHash(), 16);
		if (cur_file->IsAutoPrioritized())
		{
			if (cur_file->GetPriority() == PR_NORMAL)
				dFile.sFilePriority = _GetPlainResString(IDS_PRIOAUTONORMAL);
			else if (cur_file->GetPriority() == PR_HIGH)
				dFile.sFilePriority = _GetPlainResString(IDS_PRIOAUTOHIGH);
			else if (cur_file->GetPriority() == PR_VERYHIGH)
				dFile.sFilePriority = _GetPlainResString(IDS_PRIOAUTORELEASE);
		}
		else
		{
			// unused  CString sPriority;
			if (cur_file->GetPriority() == PR_VERYLOW)
				dFile.sFilePriority = _GetPlainResString(IDS_PRIOVERYLOW);
			else if (cur_file->GetPriority() == PR_LOW)
				dFile.sFilePriority = _GetPlainResString(IDS_PRIOLOW);
			else if (cur_file->GetPriority() == PR_NORMAL)
				dFile.sFilePriority = _GetPlainResString(IDS_PRIONORMAL);
			else if (cur_file->GetPriority() == PR_HIGH)
				dFile.sFilePriority = _GetPlainResString(IDS_PRIOHIGH);
			else if (cur_file->GetPriority() == PR_VERYHIGH)
				dFile.sFilePriority = _GetPlainResString(IDS_PRIORELEASE);
#if JUMPSTART
			if (cur_file->GetJumpstartEnabled()) {
				buffer.Format(_T("%s[%s]"), _GetPlainResString(IDS_PRIOJUMPSTART),dFile.sFilePriority);
				dFile.sFilePriority = buffer;
			}
#endif
			// removed: dFile.sFilePriority = buffer;
		}
		dFile.nFilePriority = cur_file->GetPriority();
		dFile.bFileAutoPriority = cur_file->IsAutoPrioritized();
		SharedArray.Add(dFile);
	}

	// Sorting (simple bubble sort, we don't have tons of data here)
	bool bSorted = true;
	
	for(int nMax = 0;bSorted && nMax < SharedArray.GetCount()*2; nMax++)
	{
		bSorted = false;
		for(int i = 0; i < SharedArray.GetCount() - 1; i++)
		{
			bool bSwap = false;
			switch(pThis->m_Params.SharedSort)
			{
			case SHARED_SORT_NAME:
				bSwap = SharedArray[i].sFileName.CompareNoCase(SharedArray[i+1].sFileName) > 0;
				break;
			case SHARED_SORT_SIZE:
				bSwap = SharedArray[i].lFileSize < SharedArray[i+1].lFileSize;
				break;
			case SHARED_SORT_TRANSFERRED:
				bSwap = SharedArray[i].nFileTransferred < SharedArray[i+1].nFileTransferred;
				break;
			case SHARED_SORT_ALL_TIME_TRANSFERRED:
				bSwap = SharedArray[i].nFileAllTimeTransferred < SharedArray[i+1].nFileAllTimeTransferred;
				break;
			case SHARED_SORT_REQUESTS:
				bSwap = SharedArray[i].nFileRequests < SharedArray[i+1].nFileRequests;
				break;
			case SHARED_SORT_ALL_TIME_REQUESTS:
				bSwap = SharedArray[i].nFileAllTimeRequests < SharedArray[i+1].nFileAllTimeRequests;
				break;
			case SHARED_SORT_ACCEPTS:
				bSwap = SharedArray[i].nFileAccepts < SharedArray[i+1].nFileAccepts;
				break;
			case SHARED_SORT_ALL_TIME_ACCEPTS:
				bSwap = SharedArray[i].nFileAllTimeAccepts < SharedArray[i+1].nFileAllTimeAccepts;
				break;
			case SHARED_SORT_PRIORITY:
				//Very low priority is define equal to 4 ! Must adapte sorting code
				if(SharedArray[i].nFilePriority == 4)
				{
					if(SharedArray[i+1].nFilePriority == 4)
						bSwap = false;
					else
						bSwap = true;
				}        
				else
					if(SharedArray[i+1].nFilePriority == 4)
					{
						if(SharedArray[i].nFilePriority == 4)
							bSwap = true;
						else
							bSwap = false;
					}
					else
						bSwap = SharedArray[i].nFilePriority < SharedArray[i+1].nFilePriority;
				break;
			}
			if(pThis->m_Params.bSharedSortReverse)
			{
				bSwap = !bSwap;
			}
			if(bSwap)
			{
				bSorted = true;
				SharedFiles TmpFile = SharedArray[i];
				SharedArray[i] = SharedArray[i+1];
				SharedArray[i+1] = TmpFile;
			}
		}
	}
	// Displaying
	CString sSharedList = _T("");
	for(int i = 0; i < SharedArray.GetCount(); i++)
	{
		TCHAR HTTPTempC[100] = _T("");
		CString HTTPProcessData;
        HTTPProcessData = OutE;

		if (SharedArray[i].sFileHash == _ParseURL(Data.sURL,_T("hash")) )
            HTTPProcessData.Replace(_T("[LastChangedDataset]"), "checked");
		else
            HTTPProcessData.Replace(_T("[LastChangedDataset]"), "checked_no");

		CString admin = "";				//admin rights
		CString ed2k = "";				//ed2klink
		CString session = "";			//session
		CString hash = "";				//hash
		CString fname = "";				//filename
		CString sharedpriority = "";	//priority
		CString isjumpstart = "";		//jumpstart

		//DonGato: priority change
		switch(SharedArray[i].nFilePriority)
		{
			case PR_VERYLOW:
				sharedpriority.Format(_T("VeryLow"));
				break;
			case PR_LOW:
				sharedpriority.Format(_T("Low"));
				break;
			case PR_NORMAL:
				sharedpriority.Format(_T("Normal"));
				break;
			case PR_HIGH:
				sharedpriority.Format(_T("High"));
				break;
			case PR_VERYHIGH:
				sharedpriority.Format(_T("Release"));
				break;
		}
		if(SharedArray[i].bFileAutoPriority)
				sharedpriority.Format(_T("Auto"));

		if (bAdmin)
			admin.Format(_T("admin"));

		CString JSED2kLink=SharedArray[i].sED2kLink;
		JSED2kLink.Replace(_T("'"),_T("&#8217;"));

		ed2k.Format(JSED2kLink);
		session.Format(CString(sSession));
		hash.Format(CString(SharedArray[i].sFileHash));
		fname.Format(CString(SharedArray[i].sFileName));
		fname.Replace(_T("'"),_T("&#8217;"));

		// Purity: showing shared files priority for Release and JS
		CKnownFile* cur_file;
		uchar fileid[16];
		DecodeBase16(hash.GetBuffer(),hash.GetLength(),fileid);
		cur_file=theApp.sharedfiles->GetFileByID(fileid);

#ifdef OLD_SOCKETS_ENABLED
		if (cur_file->GetJumpstartEnabled())
		{
			isjumpstart.Format(_T("jumpstart"));//+++
			HTTPProcessData.Replace(_T("[FileIsPriority]"), "jumpstart");
		}
		else if (cur_file->GetPriority() == PR_VERYHIGH)
			HTTPProcessData.Replace(_T("[FileIsPriority]"), "release");
		else
			HTTPProcessData.Replace(_T("[FileIsPriority]"), "none");
#endif //OLD_SOCKETS_ENABLED

		HTTPProcessData.Replace(_T("[admin]"), admin);
		HTTPProcessData.Replace(_T("[ed2k]"), ed2k);
		HTTPProcessData.Replace(_T("[hash]"), hash);
		HTTPProcessData.Replace(_T("[fname]"), fname);
		HTTPProcessData.Replace(_T("[session]"), session);
		HTTPProcessData.Replace(_T("[shared-priority]"), sharedpriority); //DonGato: priority change
		HTTPProcessData.Replace(_T("[isjumpstart]"), isjumpstart);//+++

		if(SharedArray[i].bIsPartFile)
			HTTPProcessData.Replace(_T("[FileState]"), "filedown");
		else
			HTTPProcessData.Replace(_T("[FileState]"), "file");

		HTTPProcessData.Replace(_T("[FileName]"), _SpecialChars(SharedArray[i].sFileName));
		if (!WSsharedColumnHidden[0]){
			if(SharedArray[i].sFileName.GetLength() > SHORT_LENGTH)
				HTTPProcessData.Replace(_T("[ShortFileName]"), _SpecialChars(SharedArray[i].sFileName.Left(SHORT_LENGTH)) + _T("..."));
			else
				HTTPProcessData.Replace(_T("[ShortFileName]"), _SpecialChars(SharedArray[i].sFileName));
		} else
			HTTPProcessData.Replace(_T("[ShortFileName]"), "");
		if (!WSsharedColumnHidden[1]){
			_stprintf(HTTPTempC, _T("%s"),CastItoXBytes(SharedArray[i].nFileTransferred));
			HTTPProcessData.Replace(_T("[FileTransferred]"), CString(HTTPTempC));
			_stprintf(HTTPTempC, _T("%s"),CastItoXBytes(SharedArray[i].nFileAllTimeTransferred));
			HTTPProcessData.Replace(_T("[FileAllTimeTransferred]"), " ("+CString(HTTPTempC)+")");
		} else {
			HTTPProcessData.Replace(_T("[FileTransferred]"), "");
			HTTPProcessData.Replace(_T("[FileAllTimeTransferred]"), ""); }
		if (!WSsharedColumnHidden[2]){
			_stprintf(HTTPTempC, _T("%i"), SharedArray[i].nFileRequests);
			HTTPProcessData.Replace(_T("[FileRequests]"), CString(HTTPTempC));
			_stprintf(HTTPTempC, _T("%i"), SharedArray[i].nFileAllTimeRequests);
			HTTPProcessData.Replace(_T("[FileAllTimeRequests]"), " ("+CString(HTTPTempC)+")");
		} else {
			HTTPProcessData.Replace(_T("[FileRequests]"), "");
			HTTPProcessData.Replace(_T("[FileAllTimeRequests]"), ""); }
		if (!WSsharedColumnHidden[3]){
			_stprintf(HTTPTempC, _T("%i"), SharedArray[i].nFileAccepts);
			HTTPProcessData.Replace(_T("[FileAccepts]"), CString(HTTPTempC));
			_stprintf(HTTPTempC, _T("%i"), SharedArray[i].nFileAllTimeAccepts);
			HTTPProcessData.Replace(_T("[FileAllTimeAccepts]"), " ("+CString(HTTPTempC)+")");
		} else {
			HTTPProcessData.Replace(_T("[FileAccepts]"), "");
			HTTPProcessData.Replace(_T("[FileAllTimeAccepts]"), ""); }
		if (!WSsharedColumnHidden[4]){
			_stprintf(HTTPTempC, _T("%s"),CastItoXBytes(SharedArray[i].lFileSize));
			HTTPProcessData.Replace(_T("[FileSize]"), CString(HTTPTempC));
		} else
			HTTPProcessData.Replace(_T("[FileSize]"), "");
		if (!WSsharedColumnHidden[5]){
			HTTPProcessData.Replace(_T("[Priority]"), SharedArray[i].sFilePriority);
		} else
			HTTPProcessData.Replace(_T("[Priority]"), "");

		HTTPProcessData.Replace(_T("[FileHash]"), SharedArray[i].sFileHash);

		sSharedList += HTTPProcessData;
	}
	Out.Replace(_T("[SharedFilesList]"), sSharedList);
	Out.Replace(_T("[Session]"), sSession);

	return Out;

	EMULE_CATCH

	return _T("");
}

CString CWebServer::_GetGraphs(ThreadData Data)
{
	EMULE_TRY

	CWebServer *pThis = (CWebServer *)Data.pThis;
	if(pThis == NULL)
		return _T("");

	CString Out = pThis->m_Templates.sGraphs;

	CString sGraphDownload = _T(""), sGraphUpload = _T(""), sGraphCons = _T("");
	CString sTmp = _T("");
	int nPrevValUp = 0, nPrevValDown = 0;
	for(int i = 0; i < WEB_GRAPH_WIDTH; i++)
	{
		if(i < pThis->m_Params.PointsForWeb.GetCount())
		{
			if(i != 0)
			{
				sGraphDownload += _T(",");
				sGraphUpload += _T(",");
				sGraphCons += _T(",");
			}

			// download
			sTmp.Format(_T("%d") , (uint32) (pThis->m_Params.PointsForWeb[i].download*1024));
			sGraphDownload += sTmp;

			// upload
			sTmp.Format(_T("%d") , (uint32) (pThis->m_Params.PointsForWeb[i].upload*1024));
			sGraphUpload += sTmp;

			// connections
			sTmp.Format(_T("%d") , (uint32) (pThis->m_Params.PointsForWeb[i].connections));
			sGraphCons += sTmp;
		}
	}

	Out.Replace(_T("[GraphDownload]"), sGraphDownload);
	Out.Replace(_T("[GraphUpload]"), sGraphUpload);
	Out.Replace(_T("[GraphConnections]"), sGraphCons);

	Out.Replace(_T("[TxtDownload]"), _GetPlainResString(IDS_ST_ACTIVEDOWNLOAD));
	Out.Replace(_T("[TxtUpload]"), _GetPlainResString(IDS_ST_ACTIVEUPLOAD));
	Out.Replace(_T("[TxtTime]"), _GetPlainResString(IDS_TIME));
	Out.Replace(_T("[KByteSec]"), _GetPlainResString(IDS_KBYTESEC));
	Out.Replace(_T("[TxtConnections]"), _GetPlainResString(IDS_ST_ACTIVECONNECTIONS));

	CString sScale;
	sScale.Format(_T("%s"), CastSecondsToHM(theApp.glob_prefs->GetTrafficOMeterInterval() * WEB_GRAPH_WIDTH) );

	CString s1, s2, s3;
	s1.Format(_T("%d"), theApp.glob_prefs->GetMaxGraphDownloadRate() + 4);
	s2.Format(_T("%d"), theApp.glob_prefs->GetMaxGraphUploadRate() + 4);
	s3.Format(_T("%d"), theApp.glob_prefs->GetMaxConnections());

	Out.Replace(_T("[ScaleTime]"), sScale);
	Out.Replace(_T("[MaxDownload]"), s1);
	Out.Replace(_T("[MaxUpload]"), s2);
	Out.Replace(_T("[MaxConnections]"), s3);

	return Out;

	EMULE_CATCH

	return _T("");
}

CString CWebServer::_GetAddServerBox(ThreadData Data)
{
	EMULE_TRY

	CWebServer *pThis = (CWebServer *)Data.pThis;
	if(pThis == NULL)
		return _T("");

	CString sSession = _ParseURL(Data.sURL, _T("ses"));

	if (!IsSessionAdmin(Data,sSession)) return _T("");

	CString Out = pThis->m_Templates.sAddServerBox;
	if(_ParseURL(Data.sURL, _T("addserver")) == "true") 
	{ 
		if(_ParseURL(Data.sURL, _T("serveraddr")) != "" && _ParseURL(Data.sURL, _T("serverport")) != "")
		{
			theApp.emuledlg->serverwnd.AddServer(_ParseURL(Data.sURL, _T("serveraddr")),_ParseURL(Data.sURL, _T("serverport")),_ParseURL(Data.sURL, _T("servername"))); 
			CString resultlog = _SpecialChars(theApp.emuledlg->logtext);        //Pick-up last line of the log 

			CServer* server=theApp.serverlist->GetServerByAddress(_ParseURL(Data.sURL, _T("serveraddr")),atoi(_ParseURL(Data.sURL, _T("serverport"))));
			if (_ParseURL(Data.sURL, _T("priority")) == "low")
				server->SetPreference((CServer::eServerPriority)PR_LOW);
			else if (_ParseURL(Data.sURL, _T("priority")) == "normal")
				server->SetPreference((CServer::eServerPriority)PR_NORMAL);
			else if (_ParseURL(Data.sURL, _T("priority")) == "high")
				server->SetPreference((CServer::eServerPriority)PR_NORMAL);
			if (theApp.emuledlg->serverwnd) 
				theApp.emuledlg->serverwnd.serverlistctrl.RefreshServer(*server);

			if(_ParseURL(Data.sURL, _T("addtostatic")) == "true") 
			{
				_AddToStatic(_ParseURL(Data.sURL, _T("serveraddr")),atoi(_ParseURL(Data.sURL, _T("serverport")))); 
				resultlog += "<br>";
				resultlog += _SpecialChars(theApp.emuledlg->logtext);        //Pick-up last line of the log 
			}
			resultlog = resultlog.TrimRight('\n'); 
			resultlog = resultlog.Mid(resultlog.ReverseFind('\n')); 
			Out.Replace(_T("[Message]"), resultlog);
			if(_ParseURL(Data.sURL, _T("connectnow")) == "true") 
				_ConnectToServer(_ParseURL(Data.sURL, _T("serveraddr")),atoi(_ParseURL(Data.sURL, _T("serverport")))); 
		} 
		else
			Out.Replace(_T("[Message]"), _GetPlainResString(IDS_INVALIDSA));
	} 
	else if(_ParseURL(Data.sURL, _T("updateservermetfromurl")) == "true")
	{
		theApp.emuledlg->serverwnd.UpdateServerMetFromURL(_ParseURL(Data.sURL, _T("servermeturl")));
		CString resultlog = _SpecialChars(theApp.emuledlg->logtext);
		resultlog = resultlog.TrimRight('\n');
		resultlog = resultlog.Mid(resultlog.ReverseFind('\n'));
		Out.Replace(_T("[Message]"),resultlog);
	}
	else
		Out.Replace(_T("[Message]"), _T(""));
    Out.Replace(_T("[AddServer]"), _GetPlainResString(IDS_SV_NEWSERVER));
	Out.Replace(_T("[IP]"), _GetPlainResString(IDS_SV_ADDRESS));
	Out.Replace(_T("[Port]"), _GetPlainResString(IDS_SV_PORT));
	Out.Replace(_T("[Name]"), _GetPlainResString(IDS_SW_NAME));
	Out.Replace(_T("[Static]"), _GetPlainResString(IDS_STATICSERVER));
	Out.Replace(_T("[ConnectNow]"), _GetPlainResString(IDS_CONNECTNOW));
	Out.Replace(_T("[Priority]"), _GetPlainResString(IDS_PRIORITY));
	Out.Replace(_T("[Low]"), _GetPlainResString(IDS_PRIOLOW));
	Out.Replace(_T("[Normal]"), _GetPlainResString(IDS_PRIONORMAL));
	Out.Replace(_T("[High]"), _GetPlainResString(IDS_PRIOHIGH));
	Out.Replace(_T("[Add]"), _GetPlainResString(IDS_SV_ADD));
	Out.Replace(_T("[UpdateServerMetFromURL]"), _GetPlainResString(IDS_SV_MET));
	Out.Replace(_T("[URL]"), _GetPlainResString(IDS_SV_URL));
	Out.Replace(_T("[Apply]"), _GetPlainResString(IDS_PW_APPLY));
	Out.Replace(_T("[URL_Disconnect]"), IsSessionAdmin(Data,sSession)?CString(_T("/?ses=") + sSession + _T("&w=server&c=disconnect")):GetPermissionDenied());
	Out.Replace(_T("[URL_Connect]"), IsSessionAdmin(Data,sSession)?CString(_T("/?ses=") + sSession + _T("&w=server&c=connect")):GetPermissionDenied());
	Out.Replace(_T("[Disconnect]"), _GetPlainResString(IDS_IRC_DISCONNECT));
	Out.Replace(_T("[Connect]"), _GetPlainResString(IDS_CONNECTTOANYSERVER));
	Out.Replace(_T("[ServerOptions]"), CString(_GetPlainResString(IDS_FSTAT_CONNECTION)));
	Out.Replace(_T("[Execute]"), _GetPlainResString(IDS_IRC_PERFORM));

	return Out;

	EMULE_CATCH

	return _T("");
}

CString CWebServer::_GetLog(ThreadData Data)
{
	EMULE_TRY

	CWebServer *pThis = (CWebServer *)Data.pThis;
	if(pThis == NULL)
		return _T("");

	CString sSession = _ParseURL(Data.sURL, _T("ses"));

	CString Out = pThis->m_Templates.sLog;

	if (_ParseURL(Data.sURL, _T("clear")) == "yes" && IsSessionAdmin(Data,sSession))
	{
		theApp.emuledlg->ResetLog();
	}
	Out.Replace(_T("[Clear]"), _GetPlainResString(IDS_PW_RESET));
	Out.Replace(_T("[Log]"), _SpecialChars(theApp.emuledlg->logtext));
	Out.Replace(_T("[Session]"), sSession);

	return Out;

	EMULE_CATCH

	return _T("");
}

CString CWebServer::_GetServerInfo(ThreadData Data)
{
	EMULE_TRY

	CWebServer *pThis = (CWebServer *)Data.pThis;
	if(pThis == NULL)
		return _T("");

	CString sSession = _ParseURL(Data.sURL, _T("ses"));

	CString Out = pThis->m_Templates.sServerInfo;

	if (_ParseURL(Data.sURL, _T("clear")) == "yes" && IsSessionAdmin(Data,sSession))
	{
		theApp.emuledlg->serverwnd.servermsgbox.SetWindowText(_T(""));
	}
	Out.Replace(_T("[Clear]"), _GetPlainResString(IDS_PW_RESET));
	Out.Replace(_T("[ServerInfo]"), _SpecialChars(theApp.emuledlg->serverwnd.servermsgbox.GetHyperText()->GetText()));
	Out.Replace(_T("[Session]"), sSession);

	return Out;

	EMULE_CATCH

	return _T("");
}

CString CWebServer::_GetDebugLog(ThreadData Data)
{
	EMULE_TRY

	CWebServer *pThis = (CWebServer *)Data.pThis;
	if(pThis == NULL)
		return _T("");

	CString sSession = _ParseURL(Data.sURL, _T("ses"));

	CString Out = pThis->m_Templates.sDebugLog;

	if (_ParseURL(Data.sURL, _T("clear")) == "yes" && IsSessionAdmin(Data,sSession))
	{
		theApp.emuledlg->ResetDebugLog();
	}
	Out.Replace(_T("[Clear]"), _GetPlainResString(IDS_PW_RESET));
	Out.Replace(_T("[DebugLog]"), _SpecialChars(theApp.emuledlg->debuglog));
	Out.Replace(_T("[Session]"), sSession);

	return Out;

	EMULE_CATCH

	return _T("");
}

CString CWebServer::_GetStats(ThreadData Data)
{
	EMULE_TRY

	CWebServer *pThis = (CWebServer *)Data.pThis;
	if(pThis == NULL)
		return _T("");

	CString sSession = _ParseURL(Data.sURL, _T("ses"));

	// refresh statistics
	theApp.emuledlg->ShowStatistics();
	theApp.emuledlg->statisticswnd.ShowStatistics(true);

	CString Out = pThis->m_Templates.sStats;
	// eklmn: new stats
	Out.Replace(_T("[Stats]"), theApp.emuledlg->statisticswnd.stattree.GetHTMLForExport());

	return Out;

	EMULE_CATCH

	return _T("");
}

CString CWebServer::_GetPreferences(ThreadData Data)
{
	EMULE_TRY

	CWebServer *pThis = (CWebServer *)Data.pThis;
	if(pThis == NULL)
		return _T("");

	CString sSession = _ParseURL(Data.sURL, _T("ses"));

	CString Out = pThis->m_Templates.sPreferences;
	Out.Replace(_T("[Session]"), sSession);

	if ((_ParseURL(Data.sURL, _T("saveprefs")) == "true") && IsSessionAdmin(Data,sSession) ) {
		if(_ParseURL(Data.sURL, _T("gzip")) == "true" || _ParseURL(Data.sURL, _T("gzip")).MakeLower() == "on")
	    {
		    theApp.glob_prefs->SetWebUseGzip(true);
	    }
		if(_ParseURL(Data.sURL, _T("gzip")) == "false" || _ParseURL(Data.sURL, _T("gzip")) == "")
	    {
		    theApp.glob_prefs->SetWebUseGzip(false);
	    }
		if(_ParseURL(Data.sURL, _T("showuploadqueue")) == "true" || _ParseURL(Data.sURL, _T("showuploadqueue")).MakeLower() == "on" )
	    {
		    pThis->m_Params.bShowUploadQueue = true;
	    }
		if(_ParseURL(Data.sURL, _T("showuploadqueuebanned")) == "true" || _ParseURL(Data.sURL, _T("showuploadqueuebanned")).MakeLower() == "on" )
	    {
		    pThis->m_Params.bShowUploadQueueBanned = true;
	    }
		if(_ParseURL(Data.sURL, _T("showuploadqueuefriend")) == "true" || _ParseURL(Data.sURL, _T("showuploadqueuefriend")).MakeLower() == "on" )
	    {
		    pThis->m_Params.bShowUploadQueueFriend = true;
	    }
		if(_ParseURL(Data.sURL, _T("showuploadqueuecredit")) == "true" || _ParseURL(Data.sURL, _T("showuploadqueuecredit")).MakeLower() == "on" )
	    {
		    pThis->m_Params.bShowUploadQueueCredit = true;
	    }
	    if(_ParseURL(Data.sURL, _T("refresh")) != "")
	    {
		    theApp.glob_prefs->SetWebPageRefresh(_tstoi(_ParseURL(Data.sURL, _T("refresh"))));
	    }
		if(_ParseURL(Data.sURL, _T("maxdown")) != "")
	    {
			if(_tstoi(_ParseURL(Data.sURL, _T("maxdown")))>0)
			{
				theApp.glob_prefs->SetMaxDownload(_tstoi(_ParseURL(Data.sURL, _T("maxdown"))));
				theApp.glob_prefs->SetLimitlessDownload(false);	//SyruS disable limitless on setting a specific value
			}
			else
				theApp.glob_prefs->SetLimitlessDownload(true);	//SyruS enable limitless on setting to 0
		}
	    if(_ParseURL(Data.sURL, _T("maxup")) != "")
	    {
		    if(_tstoi(_ParseURL(Data.sURL, _T("maxup")))>0)
			    theApp.glob_prefs->SetMaxUpload(_tstoi(_ParseURL(Data.sURL, _T("maxup"))));
//		    else
//			    theApp.glob_prefs->SetMaxUpload(UNLIMITED);
	    }
	    if(_ParseURL(Data.sURL, _T("maxcapdown")) != "")
	    {
		    theApp.glob_prefs->SetMaxGraphDownloadRate(_tstoi(_ParseURL(Data.sURL, _T("maxcapdown"))));
	    }
	    if(_ParseURL(Data.sURL, _T("maxcapup")) != "")
	    {
		    theApp.glob_prefs->SetMaxGraphUploadRate(_tstoi(_ParseURL(Data.sURL, _T("maxcapup"))));
	    }

		if(_ParseURL(Data.sURL, _T("maxsources")) != "")
		{
			theApp.glob_prefs->SetMaxSourcePerFile(_tstoi(_ParseURL(Data.sURL, _T("maxsources"))));
		}
		if(_ParseURL(Data.sURL, _T("maxconnections")) != "")
		{
			theApp.glob_prefs->SetMaxConnections(_tstoi(_ParseURL(Data.sURL, _T("maxconnections"))));
		}
		if(_ParseURL(Data.sURL, _T("maxconnectionsperfive")) != "")
		{
			theApp.glob_prefs->SetMaxDownloadConperFive(_tstoi(_ParseURL(Data.sURL, _T("maxconnectionsperfive"))));
		}
	}

	// Fill form
	if(theApp.glob_prefs->GetWebUseGzip())
	{
		Out.Replace(_T("[UseGzipVal]"), _T("checked"));
	}
	else
	{
		Out.Replace(_T("[UseGzipVal]"), _T(""));
	}
    if(pThis->m_Params.bShowUploadQueue)
	{
		Out.Replace(_T("[ShowUploadQueueVal]"), _T("checked"));
	}
	else
	{
		Out.Replace(_T("[ShowUploadQueueVal]"), _T(""));
	}
    if(pThis->m_Params.bShowUploadQueueBanned)
	{
		Out.Replace(_T("[ShowUploadQueueBannedVal]"), _T("checked"));
	}
	else
	{
		Out.Replace(_T("[ShowUploadQueueBannedVal]"), _T(""));
	}
    if(pThis->m_Params.bShowUploadQueueFriend)
	{
		Out.Replace(_T("[ShowUploadQueueFriendVal]"), _T("checked"));
	}
	else
	{
		Out.Replace(_T("[ShowUploadQueueFriendVal]"), _T(""));
	}
    if(pThis->m_Params.bShowUploadQueueCredit)
	{
		Out.Replace(_T("[ShowUploadQueueCreditVal]"), _T("checked"));
	}
	else
	{
		Out.Replace(_T("[ShowUploadQueueCreditVal]"), _T(""));
	}
	CString sRefresh;
	
	sRefresh.Format(_T("%d"), theApp.glob_prefs->GetWebPageRefresh());
	Out.Replace(_T("[RefreshVal]"), sRefresh);

	sRefresh.Format(_T("%d"), theApp.glob_prefs->GetMaxSourcePerFile());
	Out.Replace(_T("[MaxSourcesVal]"), sRefresh);

	sRefresh.Format(_T("%d"), theApp.glob_prefs->GetMaxConnections());
	Out.Replace(_T("[MaxConnectionsVal]"), sRefresh);

	sRefresh.Format(_T("%d"), theApp.glob_prefs->GetMaxConperFive());
	Out.Replace(_T("[MaxConnectionsPer5Val]"), sRefresh);

	Out.Replace(_T("[KBS]"), _GetPlainResString(IDS_KBYTESEC)+_T(":"));
	Out.Replace(_T("[FileSettings]"), CString(_GetPlainResString(IDS_WEB_FILESETTINGS)+_T(":")));
	Out.Replace(_T("[LimitForm]"), _GetPlainResString(IDS_WEB_CONLIMITS)+_T(":"));
	Out.Replace(_T("[MaxSources]"), _GetPlainResString(IDS_PW_MAXSOURCES)+_T(":"));
	Out.Replace(_T("[MaxConnections]"), _GetPlainResString(IDS_PW_MAXC)+_T(":"));
	Out.Replace(_T("[MaxConnectionsPer5]"), _GetPlainResString(IDS_MAXCON5_TEXT)+_T(":"));
	Out.Replace(_T("[UseGzipForm]"), _GetPlainResString(IDS_WEB_GZIP_COMPRESSION));
	Out.Replace(_T("[UseGzipComment]"), _GetPlainResString(IDS_WEB_GZIP_COMMENT));
	Out.Replace(_T("[ShowUploadQueueForm]"), _GetPlainResString(IDS_WEB_SHOW_UPLOAD_QUEUE));
	Out.Replace(_T("[ShowUploadQueueBannedForm]"), _GetPlainResString(IDS_WEB_SHOW_UPLOAD_QUEUE_BANNED));
	Out.Replace(_T("[ShowUploadQueueFriendForm]"), _GetPlainResString(IDS_WEB_SHOW_UPLOAD_QUEUE_FRIEND));
	Out.Replace(_T("[ShowUploadQueueCreditForm]"), _GetPlainResString(IDS_WEB_SHOW_UPLOAD_QUEUE_CREDIT));
	Out.Replace(_T("[ShowUploadQueueComment]"), _GetPlainResString(IDS_WEB_UPLOAD_QUEUE_COMMENT));
	Out.Replace(_T("[ShowUploadQueueBannedComment]"), _GetPlainResString(IDS_WEB_UPLOAD_QUEUE_BANNED_COMMENT));
	Out.Replace(_T("[ShowUploadQueueFriendComment]"), _GetPlainResString(IDS_WEB_UPLOAD_QUEUE_FRIEND_COMMENT));
	Out.Replace(_T("[ShowUploadQueueCreditComment]"), _GetPlainResString(IDS_WEB_UPLOAD_QUEUE_CREDIT_COMMENT));
	Out.Replace(_T("[ShowQueue]"), _GetPlainResString(IDS_SHOW));
	Out.Replace(_T("[HideQueue]"), _GetPlainResString(IDS_HIDE));
	Out.Replace(_T("[ShowQueueBanned]"), _GetPlainResString(IDS_SHOW));
	Out.Replace(_T("[HideQueueBanned]"), _GetPlainResString(IDS_HIDE));
	Out.Replace(_T("[ShowQueueFriend]"), _GetPlainResString(IDS_SHOW));
	Out.Replace(_T("[HideQueueFriend]"), _GetPlainResString(IDS_HIDE));
	Out.Replace(_T("[ShowQueueCredit]"), _GetPlainResString(IDS_SHOW));
	Out.Replace(_T("[HideQueueCredit]"), _GetPlainResString(IDS_HIDE));
	Out.Replace(_T("[RefreshTimeForm]"), _GetPlainResString(IDS_WEB_REFRESH_TIME));
	Out.Replace(_T("[RefreshTimeComment]"), _GetPlainResString(IDS_WEB_REFRESH_COMMENT));
	Out.Replace(_T("[SpeedForm]"), _GetPlainResString(IDS_SPEED_LIMITS));
	Out.Replace(_T("[MaxDown]"), _GetPlainResString(IDS_ST_ACTIVEDOWNLOAD));
	Out.Replace(_T("[MaxUp]"), _GetPlainResString(IDS_ST_ACTIVEUPLOAD));
	Out.Replace(_T("[SpeedCapForm]"), _GetPlainResString(IDS_CAPACITY_LIMITS));
	Out.Replace(_T("[MaxCapDown]"), _GetPlainResString(IDS_ST_ACTIVEDOWNLOAD));
	Out.Replace(_T("[MaxCapUp]"), _GetPlainResString(IDS_ST_ACTIVEUPLOAD));
	Out.Replace(_T("[WebControl]"), _GetPlainResString(IDS_WEB_CONTROL));
	Out.Replace(_T("[eMuleAppName]"), _GetPlainResString(IDS_EMULE_PLUS));
	Out.Replace(_T("[Apply]"), _GetPlainResString(IDS_PW_APPLY));

	CString sT;
	int n = (int)theApp.glob_prefs->GetMaxDownload();
	if(n < 0 || n == 65535) n = 0;
	sT.Format(_T("%d"), n);
	Out.Replace(_T("[MaxDownVal]"), sT);
	n = (int)theApp.glob_prefs->GetMaxUpload();
	if(n < 0 || n == 65535) n = 0;
	sT.Format(_T("%d"), n);
	Out.Replace(_T("[MaxUpVal]"), sT);
	sT.Format(_T("%d"), theApp.glob_prefs->GetMaxGraphDownloadRate());
	Out.Replace(_T("[MaxCapDownVal]"), sT);
	sT.Format(_T("%d"), theApp.glob_prefs->GetMaxGraphUploadRate());
	Out.Replace(_T("[MaxCapUpVal]"), sT);

	return Out;

	EMULE_CATCH

	return _T("");
}

CString CWebServer::_GetLoginScreen(ThreadData Data)
{
	EMULE_TRY

	CWebServer *pThis = (CWebServer *)Data.pThis;
	if(pThis == NULL)
		return _T("");

	CString sSession = _ParseURL(Data.sURL, _T("ses"));

	CString Out = _T("");

	Out += pThis->m_Templates.sLogin;

	Out.Replace(_T("[CharSet]"), _GetWebCharSet());
	Out.Replace(_T("[eMuleAppName]"), _GetPlainResString(IDS_EMULE_PLUS));
	Out.Replace(_T("[version]"), CURRENT_VERSION_LONG);
	Out.Replace(_T("[Login]"), _GetPlainResString(IDS_WEB_LOGIN));
	Out.Replace(_T("[EnterPassword]"), _GetPlainResString(IDS_WEB_ENTER_PASSWORD));
	Out.Replace(_T("[LoginNow]"), _GetPlainResString(IDS_WEB_LOGIN_NOW));
	Out.Replace(_T("[WebControl]"), _GetPlainResString(IDS_WEB_CONTROL));

	if(pThis->m_nIntruderDetect >= 1)
		Out.Replace(_T("[FailedLogin]"), _T("<p class=\"failed\">") + _GetPlainResString(IDS_WEB_BADLOGINATTEMPT) + _T("</p>"));
	else
		Out.Replace(_T("[FailedLogin]"), "&nbsp;");

	return Out;

	EMULE_CATCH

	return _T("");
}

// We have to add gz-header and some other stuff
// to standard zlib functions
// in order to use gzip in web pages
int CWebServer::_GzipCompress(Bytef *dest, uLongf *destLen, const Bytef *source, uLong sourceLen, int level)
{ 
	EMULE_TRY

	const static int gz_magic[2] = {0x1f, 0x8b}; // gzip magic header
	int err;
	uLong crc;
	z_stream stream = {0};
	stream.zalloc = (alloc_func)0;
	stream.zfree = (free_func)0;
	stream.opaque = (voidpf)0;
	crc = crc32(0L, Z_NULL, 0);
	// init Zlib stream
	// NOTE windowBits is passed < 0 to suppress zlib header
	err = deflateInit2(&stream, level, Z_DEFLATED, -MAX_WBITS, MAX_MEM_LEVEL, Z_DEFAULT_STRATEGY);
	if (err != Z_OK)
		return err;

	sprintf((char*)dest , "%c%c%c%c%c%c%c%c%c%c", gz_magic[0], gz_magic[1],
		Z_DEFLATED, 0 /*flags*/, 0,0,0,0 /*time*/, 0 /*xflags*/, 255);
	// wire buffers
	stream.next_in = (Bytef*) source ;
	stream.avail_in = (uInt)sourceLen;
	stream.next_out = ((Bytef*) dest) + 10;
	stream.avail_out = *destLen - 18;
	// doit
	err = deflate(&stream, Z_FINISH);
	if (err != Z_STREAM_END)
	{
		deflateEnd(&stream);
		return err;
	}
	err = deflateEnd(&stream);
	crc = crc32(crc, (const Bytef *) source ,  sourceLen );
	//CRC
	*(((Bytef*) dest)+10+stream.total_out) = (Bytef)(crc & 0xFF);
	*(((Bytef*) dest)+10+stream.total_out+1) = (Bytef)((crc>>8) & 0xFF);
	*(((Bytef*) dest)+10+stream.total_out+2) = (Bytef)((crc>>16) & 0xFF);
	*(((Bytef*) dest)+10+stream.total_out+3) = (Bytef)((crc>>24) & 0xFF);
	// Length
	*(((Bytef*) dest)+10+stream.total_out+4) = (Bytef)( sourceLen  & 0xFF);
	*(((Bytef*) dest)+10+stream.total_out+5) = (Bytef)(( sourceLen >>8) & 0xFF);
	*(((Bytef*) dest)+10+stream.total_out+6) = (Bytef)(( sourceLen >>16) &	0xFF);
	*(((Bytef*) dest)+10+stream.total_out+7) = (Bytef)(( sourceLen >>24) &	0xFF);
	// return  destLength
	*destLen = 10 + stream.total_out + 8;
	return err;

	EMULE_CATCH

	return -1;
}

bool CWebServer::_IsLoggedIn(ThreadData Data, long lSession)
{
	EMULE_TRY

	CWebServer *pThis = (CWebServer *)Data.pThis;
	if(pThis == NULL)
		return "";

	_RemoveTimeOuts(Data,lSession);

	// find our session
	// i should have used CMap there, but i like CArray more ;-)
	for(int i = 0; i < pThis->m_Params.Sessions.GetSize(); i++)
	{
		if(pThis->m_Params.Sessions[i].lSession == lSession && lSession != 0)
		{
			// if found, also reset expiration time
			pThis->m_Params.Sessions[i].startTime = CTime::GetCurrentTime();
			return true;
		}
	}

	EMULE_CATCH

	return false;
}

void CWebServer::_RemoveTimeOuts(ThreadData Data, long lSession) {
	EMULE_TRY

	// remove expired sessions
	CWebServer *pThis = (CWebServer *)Data.pThis;
	pThis->UpdateSessionCount();

	EMULE_CATCH
}

bool CWebServer::_RemoveSession(ThreadData Data, long lSession)
{
	EMULE_TRY

	CWebServer *pThis = (CWebServer *)Data.pThis;
	if(pThis == NULL)
		return "";

	// find our session
	for(int i = 0; i < pThis->m_Params.Sessions.GetSize(); i++)
	{
		if(pThis->m_Params.Sessions[i].lSession == lSession && lSession != 0)
		{
			pThis->m_Params.Sessions.RemoveAt(i);
			CString t_ulCurIP;
			t_ulCurIP.Format(_T("%i.%i.%i.%i"),(uint8)pThis->m_ulCurIP,(uint8)(pThis->m_ulCurIP>>8),(uint8)(pThis->m_ulCurIP>>16),(uint8)(pThis->m_ulCurIP>>24));
			theApp.emuledlg->AddLogLine(true,IDS_WEB_SESSIONEND,t_ulCurIP);
			return true;
		}
	}

	EMULE_CATCH

	return false;
}

Session CWebServer::GetSessionByID(ThreadData Data,long sessionID) {
	EMULE_TRY
	CWebServer *pThis = (CWebServer *)Data.pThis;
	if(pThis != NULL) {
		for(int i = 0; i < pThis->m_Params.Sessions.GetSize(); i++)
		{
			if(pThis->m_Params.Sessions[i].lSession == sessionID && sessionID != 0)
				return pThis->m_Params.Sessions.GetAt(i);
		}
	}

	EMULE_CATCH

	Session ses;
	ses.admin=false;
	ses.startTime = 0;

	return ses;
}

bool CWebServer::IsSessionAdmin(ThreadData Data,CString SsessionID){
	EMULE_TRY
	long sessionID=_tstoi64(SsessionID);
	CWebServer *pThis = (CWebServer *)Data.pThis;
	if(pThis != NULL) {
		for(int i = 0; i < pThis->m_Params.Sessions.GetSize(); i++)
		{
			if(pThis->m_Params.Sessions[i].lSession == sessionID && sessionID != 0)
				return pThis->m_Params.Sessions[i].admin;
		}
	}
	EMULE_CATCH
	return false;
}

CString CWebServer::GetPermissionDenied() {
	return _T("javascript:alert(\'")+_GetPlainResString(IDS_ACCESSDENIED)+_T("\')");
}

bool CWebServer::_GetFileHash(CString sHash, uchar *FileHash)
{
	EMULE_TRY

	if (sHash.GetLength() != 32)
		return false;
	char hex_byte[3];
	int byte;
	hex_byte[2] = '\0';
	for (int i = 0; i < 16; i++) 
	{
		hex_byte[0] = sHash.GetAt(i*2);
		hex_byte[1] = sHash.GetAt((i*2 + 1));
		if (!_istxdigit(hex_byte[0]) || !_istxdigit(hex_byte[1])) {
			md4clr(FileHash);
			return false;
		} else {
			sscanf(hex_byte, "%02x", &byte);
			FileHash[i] = (uchar)byte;
		}
	}
	return true;

	EMULE_CATCH

	md4clr(FileHash);
	return false;
}

CString	CWebServer::_GetPlainResString(UINT nID, bool noquote)
{
	EMULE_TRY

	CString sRet = GetResString(nID);
	sRet.Replace(_T("&"), _T(""));
	if(noquote)
	{
		sRet.Replace(_T("'"), _T("&#8217;"));
		sRet.Replace(_T("\n"), _T("\\n"));
	}
	return sRet;

	EMULE_CATCH

	return _T("");
}

// EC + kuchin
CString	CWebServer::_GetWebCharSet()
{
	EMULE_TRY

	switch (theApp.glob_prefs->GetLanguageID())
	{
		case MAKELANGID(LANG_POLISH,SUBLANG_DEFAULT):				return _T("windows-1250");
		case MAKELANGID(LANG_RUSSIAN,SUBLANG_DEFAULT):				return _T("windows-1251");
		case MAKELANGID(LANG_GREEK,SUBLANG_DEFAULT):				return _T("ISO-8859-7");
		case MAKELANGID(LANG_HEBREW,SUBLANG_DEFAULT):				return _T("ISO-8859-8");
		case MAKELANGID(LANG_KOREAN,SUBLANG_DEFAULT):				return _T("EUC-KR");
		case MAKELANGID(LANG_CHINESE,SUBLANG_CHINESE_SIMPLIFIED):	return _T("GB2312");
		case MAKELANGID(LANG_CHINESE,SUBLANG_CHINESE_TRADITIONAL):	return _T("Big5");
		case MAKELANGID(LANG_LITHUANIAN,SUBLANG_DEFAULT):			return _T("windows-1257");
		case MAKELANGID(LANG_ROMANIAN,SUBLANG_DEFAULT):				return _T("windows-1250");
		case MAKELANGID(LANG_TURKISH,SUBLANG_DEFAULT):				return _T("windows-1254");
	}

	EMULE_CATCH

	// Western (Latin) includes Catalan, Danish, Dutch, English, Faeroese, Finnish, French,
	// German, Galician, Irish, Icelandic, Italian, Norwegian, Portuguese, Spanish and Swedish
	return _T("ISO-8859-1");
}

// Ornis: creating the progressbar. colored if ressources are given/available
CString CWebServer::_GetDownloadGraph(ThreadData Data,CString filehash)
{
	CWebServer *pThis = (CWebServer *)Data.pThis;
	if(pThis == NULL)
		return _T("");

	CPartFile* cur_file;
	uchar fileid[16];
	DecodeBase16(filehash.GetBuffer(),filehash.GetLength(),fileid);

	CString Out = _T("");
	CString temp = _T("");

	// cool style
	CString progresscolor[12];	//Ju1i3n - merged with eMulePlus by janes bong
	progresscolor[0]=_T("green.gif");
	progresscolor[1]=_T("black.gif");
	progresscolor[2]=_T("yellow.gif");
	progresscolor[3]=_T("red.gif");
	//Ju1i3n start - merged with eMulePlus by janes bong
	progresscolor[4]=_T("blue1.gif");
	progresscolor[5]=_T("blue2.gif");
	progresscolor[6]=_T("blue3.gif");
	progresscolor[7]=_T("blue4.gif");
	progresscolor[8]=_T("blue5.gif");
	progresscolor[9]=_T("blue6.gif");
	progresscolor[10]=_T("greenpercent.gif");
	progresscolor[11]=_T("transparent.gif");
	//Ju1i3n end

	//SyruS show completed files (0.28b)
	//CKnownFile* known_file=theApp.knownfiles->FindKnownFileByID(fileid);
	//if (known_file && !known_file->IsPartFile()) {
	cur_file=theApp.downloadqueue->GetFileByID(fileid); // DonGato: solve non completed files shown as complete
	if (cur_file==0 || !cur_file->IsPartFile()) {
		temp.Format(pThis->m_Templates.sProgressbarImgsPercent+_T("<br>"),progresscolor[10],pThis->m_Templates.iProgressbarWidth);
		Out+=temp;
		temp.Format(pThis->m_Templates.sProgressbarImgs,progresscolor[0],pThis->m_Templates.iProgressbarWidth);
		Out+=temp;
	}
	else {
		//cur_file=theApp.downloadqueue->GetFileByID(fileid);
		CString s_ChunkBar=cur_file->GetProgressString(pThis->m_Templates.iProgressbarWidth);

		// and now make a graph out of the array - need to be in a progressive way
		uint8 lastcolor=1;
		uint16 lastindex=0;
		for (uint16 i=0;i<pThis->m_Templates.iProgressbarWidth;i++) {
			if (lastcolor!= _tstoi(s_ChunkBar.Mid(i,1))  ) {
				if (i>lastindex) {
					temp.Format(pThis->m_Templates.sProgressbarImgs ,progresscolor[lastcolor],i-lastindex);

					Out+=temp;
				}
				lastcolor=_tstoi(s_ChunkBar.Mid(i,1));
				lastindex=i;
			}
		}
		temp.Format(pThis->m_Templates.sProgressbarImgs,progresscolor[lastcolor],pThis->m_Templates.iProgressbarWidth-lastindex);
		Out+=temp;
		int compl=(int)((pThis->m_Templates.iProgressbarWidth/100)*cur_file->GetPercentCompleted());
		(compl>0)?temp.Format(pThis->m_Templates.sProgressbarImgsPercent+_T("<br>"),progresscolor[10],compl) :temp.Format(pThis->m_Templates.sProgressbarImgsPercent+_T("<br>"),progresscolor[11],5);	//Ju1i3n - merged with eMulePlus by janes bong
		Out=temp+Out;
	}
	return Out;
}


CString	CWebServer::_GetSearch(ThreadData Data)
{
	EMULE_TRY

	CWebServer *pThis = (CWebServer *)Data.pThis;
	if(pThis == NULL)
		return _T("");

	CString sSession = _ParseURL(Data.sURL, _T("ses"));
	CString Out = pThis->m_Templates.sSearch;

	if (_ParseURL(Data.sURL, _T("downloads")) != "" && IsSessionAdmin(Data,sSession) ) {
		CString downloads=_ParseURLArray(Data.sURL,_T("downloads"));
		uint8 cat=atoi(_ParseURL(Data.sURL, "cat"));

		CString resToken;
		int curPos=0;
		resToken= downloads.Tokenize(_T("|"),curPos);
		while (resToken != "")
		{
			uchar fileid[16];
			DecodeBase16(resToken.GetBuffer(),resToken.GetLength(),fileid);
			theApp.searchlist->AddFileToDownloadByHash(fileid,cat);
			resToken= downloads.Tokenize(_T("|"),curPos);
		}
	}

	if(_ParseURL(Data.sURL, _T("tosearch")) != "" && IsSessionAdmin(Data,sSession) )
	{
		// perform search
		theApp.emuledlg->searchwnd.DeleteAllSearchs();
		theApp.emuledlg->searchwnd.DoNewSearch(
			_ParseURL(Data.sURL, _T("tosearch")),
			_ParseURL(Data.sURL, _T("type")),
			_tstol(_ParseURL(Data.sURL, _T("min")))*1048576,
			_tstol(_ParseURL(Data.sURL, _T("max")))*1048576,
			_tstoi(_ParseURL(Data.sURL, _T("avail"))),
			_ParseURL(Data.sURL, _T("ext")),
			(_ParseURL(Data.sURL, _T("global"))=="on")
			);
		Out.Replace(_T("[Message]"),_GetPlainResString(IDS_SW_SEARCHINGINFO));
	}
	else if(_ParseURL(Data.sURL, _T("tosearch")) != "" && !IsSessionAdmin(Data,sSession) ) {
		Out.Replace(_T("[Message]"),_GetPlainResString(IDS_ACCESSDENIED));
	}
	else Out.Replace(_T("[Message]"), _GetPlainResString(IDS_SW_REFETCHRES));

	CString sSort = _ParseURL(Data.sURL, "sort");	if (sSort.GetLength()>0) pThis->m_iSearchSortby=atoi(sSort);
	sSort = _ParseURL(Data.sURL, "sortAsc");		if (sSort.GetLength()>0) pThis->m_bSearchAsc=atoi(sSort);

	CString sCmd = _ParseURL(Data.sURL, _T("c"));
	if (sCmd.CompareNoCase(_T("menu")) == 0) 
	{
		int iMenu = atoi(_ParseURL(Data.sURL, _T("m")));
		bool bValue = atoi(_ParseURL(Data.sURL, _T("v")));
		WSsearchColumnHidden[iMenu] = bValue;
		CString buffer;
		CString fullpath;
		fullpath.Format(_T("%spreferences.ini"),theApp.glob_prefs->GetConfigDir());
		CIni ini( fullpath );
		ini.SetArray(WSsearchColumnHidden, ELEMENT_COUNT(WSsearchColumnHidden), _T("searchColumnHidden"), _T("WebServer")); 
		ini.SaveAndClose(); 
	}
	
	CString result=pThis->m_Templates.sSearchHeader + theApp.searchlist->GetWebList(pThis->m_Templates.sSearchResultLine,pThis->m_iSearchSortby,pThis->m_bSearchAsc,!WSsearchColumnHidden[0],!WSsearchColumnHidden[1],!WSsearchColumnHidden[2],!WSsearchColumnHidden[3]);

	if (theApp.glob_prefs->GetCatCount()>1) InsertCatBox(Out,0,"",false,false,sSession,""); else Out.Replace("[CATBOX]","");

	Out.Replace(_T("[SEARCHINFOMSG]"),_T(""));
	Out.Replace(_T("[RESULTLIST]"), result);
	Out.Replace(_T("[Result]"), GetResString(IDS_SW_RESULT) );
	Out.Replace(_T("[Session]"), sSession);
	Out.Replace(_T("[Name]"), _GetPlainResString(IDS_SW_NAME));
	Out.Replace(_T("[Type]"), _GetPlainResString(IDS_TYPE));
	Out.Replace(_T("[Any]"), _GetPlainResString(IDS_SEARCH_ANY));
	Out.Replace(_T("[Audio]"), _GetPlainResString(IDS_SEARCH_AUDIO));
	Out.Replace(_T("[Image]"), _GetPlainResString(IDS_SEARCH_PICS));
	Out.Replace(_T("[Video]"), _GetPlainResString(IDS_SEARCH_VIDEO));
	Out.Replace(_T("[Document]"), _GetPlainResString(IDS_SEARCH_DOC));
	Out.Replace(_T("[CDImage]"), _GetPlainResString(IDS_SEARCH_CDIMG));
	Out.Replace(_T("[Program]"), _GetPlainResString(IDS_SEARCH_PRG));
	Out.Replace(_T("[Archive]"), _GetPlainResString(IDS_SEARCH_ARC));
	Out.Replace(_T("[Search]"), _GetPlainResString(IDS_SW_SEARCHBOX));
	Out.Replace(_T("[Show]"), _GetPlainResString(IDS_SHOW));
	Out.Replace(_T("[Size]"), _GetPlainResString(IDS_DL_SIZE));
	Out.Replace(_T("[Sort]"), _GetPlainResString(IDS_PW_SORTING));
	Out.Replace(_T("[Collection]"), _GetPlainResString(IDS_COLLECTION));
	Out.Replace(_T("[Ascending]"), _GetPlainResString(IDS_ASCENDING));
	Out.Replace(_T("[Relevance]"), _GetPlainResString(IDS_RELEVANCE));
	Out.Replace(_T("[Match]"), _GetPlainResString(IDS_MATCH));
	Out.Replace(_T("[SizeMin]"), _GetPlainResString(IDS_SEARCHMINSIZE));
	Out.Replace(_T("[SizeMax]"), _GetPlainResString(IDS_SEARCHMAXSIZE));
	Out.Replace(_T("[Availabl]"), _GetPlainResString(IDS_SEARCHAVAIL));
	Out.Replace(_T("[Extention]"), _GetPlainResString(IDS_SEARCHEXTENSION_LBL));
	Out.Replace(_T("[Global]"), _GetPlainResString(IDS_SW_GLOBAL));
	Out.Replace(_T("[MB]"), _GetPlainResString(IDS_MBYTES));

	CString checked = "";
	if(theApp.glob_prefs->GetMethod()==1)
		checked = "checked";
	Out.Replace(_T("[checked]"), checked);

	if (!WSsearchColumnHidden[0]){
		Out.Replace(_T("[FilenameH]"), _GetPlainResString(IDS_DL_FILENAME));
		Out.Replace(_T("[FilenameM]"), _GetPlainResString(IDS_DL_FILENAME)); }
	else {
		Out.Replace(_T("[FilenameH]"), "");
		Out.Replace(_T("[FilenameM]"), _GetPlainResString(IDS_DL_FILENAME)); }
	if (!WSsearchColumnHidden[1]){
		Out.Replace(_T("[FilesizeH]"), _GetPlainResString(IDS_DL_SIZE));
		Out.Replace(_T("[FilesizeM]"), _GetPlainResString(IDS_DL_SIZE)); }
	else {
		Out.Replace(_T("[FilesizeH]"), "");
		Out.Replace(_T("[FilesizeM]"), _GetPlainResString(IDS_DL_SIZE)); }
	if (!WSsearchColumnHidden[2]){
		Out.Replace(_T("[FilehashH]"), _GetPlainResString(IDS_FILEHASH));
		Out.Replace(_T("[FilehashM]"), _GetPlainResString(IDS_FILEHASH)); }
	else {
		Out.Replace(_T("[FilehashH]"), "");
		Out.Replace(_T("[FilehashM]"), _GetPlainResString(IDS_FILEHASH)); }
	if (!WSsearchColumnHidden[3]){
		Out.Replace(_T("[SourcesH]"), _GetPlainResString(IDS_DL_SOURCES));
		Out.Replace(_T("[SourcesM]"), _GetPlainResString(IDS_DL_SOURCES)); }
	else {
		Out.Replace(_T("[SourcesH]"), "");
		Out.Replace(_T("[SourcesM]"), _GetPlainResString(IDS_DL_SOURCES)); }
	Out.Replace(_T("[Download]"), _GetPlainResString(IDS_DOWNLOAD));

	CString val;
	val.Format("%i",(pThis->m_iSearchSortby!=0 || (pThis->m_iSearchSortby==0 && pThis->m_bSearchAsc==0 ))?1:0 );
	Out.Replace("[SORTASCVALUE0]", val);
	val.Format("%i",(pThis->m_iSearchSortby!=1 || (pThis->m_iSearchSortby==1 && pThis->m_bSearchAsc==0 ))?1:0 );
	Out.Replace("[SORTASCVALUE1]", val);
	val.Format("%i",(pThis->m_iSearchSortby!=2 || (pThis->m_iSearchSortby==2 && pThis->m_bSearchAsc==0 ))?1:0 );
	Out.Replace("[SORTASCVALUE2]", val);
	val.Format("%i",(pThis->m_iSearchSortby!=3 || (pThis->m_iSearchSortby==3 && pThis->m_bSearchAsc==0 ))?1:0 );
	Out.Replace("[SORTASCVALUE3]", val);

	return Out;

	EMULE_CATCH
	return _T("");
}

int CWebServer::UpdateSessionCount() {
	EMULE_TRY

	int oldvalue=m_Params.Sessions.GetSize();
	for(int i = 0; i < m_Params.Sessions.GetSize();)
	{
		CTimeSpan ts = CTime::GetCurrentTime() - m_Params.Sessions[i].startTime;
		if(ts.GetTotalSeconds() > SESSION_TIMEOUT_SECS) {
			m_Params.Sessions.RemoveAt(i);
		}
		else
			i++;
	}

	return m_Params.Sessions.GetCount();

	EMULE_CATCH
	return 0;
}

void CWebServer::InsertCatBox(CString &Out,int preselect,CString boxlabel,bool jump,bool extraCats,CString sSession,CString sFileHash)
{
	CString tempBuf2,tempBuf3;
	if (jump)
		tempBuf2="onchange=GotoCat(this.form.cat.options[this.form.cat.selectedIndex].value)>";
	else
		tempBuf2=">";

	CString tempBuf="<form><select name=\"cat\" size=\"1\""+tempBuf2;

	for (int i=0;i< theApp.glob_prefs->GetCatCount();i++)
	{
		CString strCategory = (i==0)?_GetPlainResString(IDS_ALLOTHERS):theApp.glob_prefs->GetCategory(i)->title;
		strCategory.Replace(_T("'"),_T("\'"));
		tempBuf3= (i==preselect)? " selected":"";
		tempBuf2.Format("<option%s value=\"%i\">%s</option>",tempBuf3,i, strCategory);
		tempBuf.Append(tempBuf2);
	}
	if (extraCats)
	{
		if (theApp.glob_prefs->GetCatCount()>1)
		{
			tempBuf2.Format("<option>------------</option>");
			tempBuf.Append(tempBuf2);
		}
	
		for (int i=(theApp.glob_prefs->GetCatCount()>1)?1:2;i<=12;i++)
		{
			tempBuf3= ( (-i)==preselect)? " selected":"";
			tempBuf2.Format("<option%s value=\"%i\">%s</option>",tempBuf3,-i, GetSubCatLabel(-i));
			tempBuf.Append(tempBuf2);
		}
	}
	tempBuf.Append("</select></form>");
	Out.Replace("[CATBOX]",boxlabel+tempBuf);


	CString tempBuff2,tempBuff3, tempBuff4, strI;
	CString tempBuff="";

	for (int i=0;i< theApp.glob_prefs->GetCatCount();i++)
	{
		strI.Format(_T("%d"),i);
		if (i==preselect)
		{
			tempBuff3 = " src=checked.gif";
			if (i==0)
				tempBuff4 = _GetPlainResString(IDS_ALL);
			else
				tempBuff4 = theApp.glob_prefs->GetCategory(i)->title;
		}
		else
			tempBuff3 = " src=checked_no.gif";

		tempBuff2 = CString("<a href=&quot;/?ses=" + sSession + "&w=transfer&cat="); 
		tempBuff2+= strI; 
		tempBuff2+= CString("&quot;><div class=menuitems><img class=menuchecked"); 
		tempBuff2+= tempBuff3; 
		tempBuff2+= CString(">"); 
		CString strCategory = (i==0)?_GetPlainResString(IDS_ALL):theApp.glob_prefs->GetCategory(i)->title;
		strCategory.Replace(_T("'"),_T("\\'"));
		tempBuff2+= strCategory; 
		tempBuff2+= CString("&nbsp;</div></a>"); 
		tempBuff.Append(tempBuff2);
	}
	if (extraCats)
	{
		for (int i=(theApp.glob_prefs->GetCatCount()>1)?1:2;i<=12;i++)
		{
			strI.Format(_T("%d"),i);
			if ((-i)==preselect)
			{
				tempBuff3= " src=checked.gif";
				tempBuff4 = GetSubCatLabel(-i);
			}
			else
				tempBuff3 = " src=checked_no.gif";

			tempBuff2 = CString("<a href=&quot;/?ses=" + sSession + "&w=transfer&cat=-"); 
			tempBuff2+= strI;
			tempBuff2+= CString("&quot;><div class=menuitems><img class=menuchecked"); 
			tempBuff2+= tempBuff3;
			tempBuff2+= CString(">");
			tempBuff2+= GetSubCatLabel(-i);
			tempBuff2+= CString("&nbsp;</div></a>");
			tempBuff.Append(tempBuff2);
		}
	}
	Out.Replace("[CatBox]",tempBuff);
	Out.Replace("[Category]",tempBuff4);


	tempBuff="";
	tempBuff2="";

	for (int i=0;i< theApp.glob_prefs->GetCatCount();i++)
	{
		uchar FileHash[16];
		_GetFileHash(sFileHash, FileHash);

		CPartFile* found_file = theApp.downloadqueue->GetFileByID(FileHash);
		if(found_file)
			preselect = found_file->GetCategory();
		strI.Format(_T("%d"),i);
		if (i==preselect)
		{
			tempBuff3 = " src=checked.gif";
			if (i==0)
				tempBuff4 = _GetPlainResString(IDS_ALLOTHERS);
			else
				tempBuff4 = theApp.glob_prefs->GetCategory(i)->title;
		}
		else
			tempBuff3 = " src=checked_no.gif";

		tempBuff2 = CString("<a href=&quot;/?ses=" + sSession + "&w=transfer[CatSel]&op=setcat&file=" + sFileHash + "&filecat="); 
		tempBuff2+= strI; 
		tempBuff2+= CString("&quot;><div class=menuitems><img class=menuchecked");
		tempBuff2+= tempBuff3;
		tempBuff2+= CString(">");
		CString strCategory = (i==0)?_GetPlainResString(IDS_ALLOTHERS):theApp.glob_prefs->GetCategory(i)->title;
		strCategory.Replace(_T("'"),_T("\\'"));
		tempBuff2+= strCategory; 
		tempBuff2+= CString("&nbsp;</div></a>"); 
		tempBuff.Append(tempBuff2);
	}
	Out.Replace("[SetCatBox]",tempBuff);
}

CString CWebServer::GetSubCatLabel(int cat)
{
	switch (cat)
	{
		case -1: return _GetPlainResString(IDS_ALLOTHERS);
		case -2: return _GetPlainResString(IDS_STATUS_NOTCOMPLETED);
		case -3: return _GetPlainResString(IDS_DL_TRANSFCOMPL);
		case -4: return _GetPlainResString(IDS_WAITING);
		case -5: return _GetPlainResString(IDS_DOWNLOADING);
		case -6: return _GetPlainResString(IDS_ERRORLIKE);
		case -7: return _GetPlainResString(IDS_PAUSED);
		case -8: return _GetPlainResString(IDS_STOPPED);
		case -9: return _GetPlainResString(IDS_VIDEO);
		case -10: return _GetPlainResString(IDS_AUDIO);
		case -11: return _GetPlainResString(IDS_SEARCH_ARC);
		case -12: return _GetPlainResString(IDS_SEARCH_CDIMG);
	}

	return "?";
}
